@@ -24,6 +24,7 @@
 #include <QUrl>
 #include <QDir>
 #include <sqlite3.h>
+#include <cstring>
 
 #include "common/syncjournaldb.h"
 #include "version.h"
@@ -106,7 +107,7 @@ QString SyncJournalDb::makeDbName(const QString &localPath,
     const QString &remotePath,
     const QString &user)
 {
-    QString journalPath = QLatin1String("._sync_");
+    QString journalPath = QLatin1String(".sync_");
 
     QString key = QString::fromUtf8("%1@%2:%3").arg(user, remoteUrl.toString(), remotePath);
 
@@ -114,15 +115,6 @@ QString SyncJournalDb::makeDbName(const QString &localPath,
     journalPath.append(ba.left(6).toHex());
     journalPath.append(".db");
 
-    // If the journal doesn't exist and we can't create a file
-    // at that location, try again with a journal name that doesn't
-    // have the ._ prefix.
-    //
-    // The disadvantage of that filename is that it will only be ignored
-    // by client versions >2.3.2.
-    //
-    // See #5633: "._*" is often forbidden on samba shared folders.
-
     // If it exists already, the path is clearly usable
     QFile file(QDir(localPath).filePath(journalPath));
     if (file.exists()) {
@@ -137,19 +129,8 @@ QString SyncJournalDb::makeDbName(const QString &localPath,
         return journalPath;
     }
 
-    // Can we create it if we drop the underscore?
-    QString alternateJournalPath = journalPath.mid(2).prepend(".");
-    QFile file2(QDir(localPath).filePath(alternateJournalPath));
-    if (file2.open(QIODevice::ReadWrite)) {
-        // The alternative worked, use it
-        qCInfo(lcDb) << "Using alternate database path" << alternateJournalPath;
-        file2.close();
-        file2.remove();
-        return alternateJournalPath;
-    }
-
-    // Neither worked, just keep the original and throw errors later
-    qCWarning(lcDb) << "Could not find a writable database path" << file.fileName();
+    // Error during creation, just keep the original and throw errors later
+    qCWarning(lcDb) << "Could not find a writable database path" << file.fileName() << file.errorString();
     return journalPath;
 }
 
@@ -276,6 +257,13 @@ bool SyncJournalDb::sqlFail(const QString &log, const SqlQuery &query)
 
 bool SyncJournalDb::checkConnect()
 {
+    if (autotestFailCounter >= 0) {
+        if (!autotestFailCounter--) {
+            qCInfo(lcDb) << "Error Simulated";
+            return false;
+        }
+    }
+
     if (_db.isOpen()) {
         // Unfortunately the sqlite isOpen check can return true even when the underlying storage
         // has become unavailable - and then some operations may cause crashes. See #6049
@@ -313,18 +301,16 @@ bool SyncJournalDb::checkConnect()
         qCInfo(lcDb) << "sqlite3 version" << pragma1.stringValue(0);
     }
 
-    {
-        // Future version of the client (2.6) will have an index 'metadata_parent' which
-        // depends on a custom sqlite function which does not exist yet in 2.5.
-        // So make sure to remove the index if it exists, otherwise we will crash when inserting
-        // rows in the metadata database.
-        // This needs to be done before the synchronous mode is enabled.
-        // The 2.6 client will anyway re-creates this index if it does not exist.
-        SqlQuery query(_db);
-        query.prepare("DROP INDEX IF EXISTS metadata_parent;");
-        if (!query.exec()) {
-            return sqlFail("updateMetadataTableStructure: remove index metadata_parent", query);
-        }
+    // Set locking mode to avoid issues with WAL on Windows
+    static QByteArray locking_mode_env = qgetenv("OWNCLOUD_SQLITE_LOCKING_MODE");
+    if (locking_mode_env.isEmpty())
+        locking_mode_env = "EXCLUSIVE";
+    pragma1.prepare("PRAGMA locking_mode=" + locking_mode_env + ";");
+    if (!pragma1.exec()) {
+        return sqlFail("Set PRAGMA locking_mode", pragma1);
+    } else {
+        pragma1.next();
+        qCInfo(lcDb) << "sqlite3 locking_mode=" << pragma1.stringValue(0);
     }
 
     pragma1.prepare("PRAGMA journal_mode=" + _journalMode + ";");
@@ -345,15 +331,45 @@ bool SyncJournalDb::checkConnect()
         qCInfo(lcDb) << "sqlite3 with temp_store =" << env_temp_store;
     }
 
-    pragma1.prepare("PRAGMA synchronous = 1;");
+    // With WAL journal the NORMAL sync mode is safe from corruption,
+    // otherwise use the standard FULL mode.
+    QByteArray synchronousMode = "FULL";
+    if (QString::fromUtf8(_journalMode).compare(QStringLiteral("wal"), Qt::CaseInsensitive) == 0)
+        synchronousMode = "NORMAL";
+    pragma1.prepare("PRAGMA synchronous = " + synchronousMode + ";");
     if (!pragma1.exec()) {
         return sqlFail("Set PRAGMA synchronous", pragma1);
+    } else {
+        qCInfo(lcDb) << "sqlite3 synchronous=" << synchronousMode;
     }
+
     pragma1.prepare("PRAGMA case_sensitive_like = ON;");
     if (!pragma1.exec()) {
         return sqlFail("Set PRAGMA case_sensitivity", pragma1);
     }
 
+    {
+        // Future version of the client (2.6) will have an index 'metadata_parent' which
+        // depends on a custom sqlite function which does not exist yet in 2.5.
+        // So make sure to remove the index if it exists, otherwise we will crash when inserting
+        // rows in the metadata database.
+        // The 2.6 client will anyway re-creates this index if it does not exist.
+        SqlQuery query(_db);
+        query.prepare("DROP INDEX IF EXISTS metadata_parent;");
+        if (!query.exec()) {
+            return sqlFail("updateMetadataTableStructure: remove index metadata_parent", query);
+        }
+    }
+
+    sqlite3_create_function(_db.sqliteDb(), "parent_hash", 1, SQLITE_UTF8 | SQLITE_DETERMINISTIC, nullptr,
+                                [] (sqlite3_context *ctx,int, sqlite3_value **argv) {
+                                    auto text = reinterpret_cast<const char*>(sqlite3_value_text(argv[0]));
+                                    const char *end = std::strrchr(text, '/');
+                                    if (!end) end = text;
+                                    sqlite3_result_int64(ctx, c_jhash64(reinterpret_cast<const uint8_t*>(text),
+                                                                        end - text, 0));
+                                }, nullptr, nullptr);
+
     /* Because insert is so slow, we do everything in a transaction, and only need one call to commit */
     startTransaction();
 
@@ -445,12 +461,13 @@ bool SyncJournalDb::checkConnect()
         return sqlFail("Create table blacklist", createQuery);
     }
 
-    createQuery.prepare("CREATE TABLE IF NOT EXISTS poll("
+    createQuery.prepare("CREATE TABLE IF NOT EXISTS async_poll("
                         "path VARCHAR(4096),"
                         "modtime INTEGER(8),"
+                        "filesize BIGINT,"
                         "pollpath VARCHAR(4096));");
     if (!createQuery.exec()) {
-        return sqlFail("Create table poll", createQuery);
+        return sqlFail("Create table async_poll", createQuery);
     }
 
     // create the selectivesync table.
@@ -480,6 +497,15 @@ bool SyncJournalDb::checkConnect()
         return sqlFail("Create table datafingerprint", createQuery);
     }
 
+    // create the flags table.
+    createQuery.prepare("CREATE TABLE IF NOT EXISTS flags ("
+                        "path TEXT PRIMARY KEY,"
+                        "pinState INTEGER"
+                        ");");
+    if (!createQuery.exec()) {
+        return sqlFail("Create table flags", createQuery);
+    }
+
     // create the conflicts table.
     createQuery.prepare("CREATE TABLE IF NOT EXISTS conflicts("
                         "path TEXT PRIMARY KEY,"
@@ -504,7 +530,7 @@ bool SyncJournalDb::checkConnect()
     bool forceRemoteDiscovery = false;
 
     SqlQuery versionQuery("SELECT major, minor, patch FROM version;", _db);
-    if (!versionQuery.next()) {
+    if (!versionQuery.next().hasData) {
         // If there was no entry in the table, it means we are likely upgrading from 1.5
         qCInfo(lcDb) << "possibleUpgradeFromMirall_1_5 detected!";
         forceRemoteDiscovery = true;
@@ -528,11 +554,12 @@ bool SyncJournalDb::checkConnect()
             forceRemoteDiscovery = true;
         }
 
-        // There was a bug in versions <2.3.0 that could lead to stale
-        // local files and a remote discovery will fix them.
-        // See #5190 #5242.
-        if (major == 2 && minor < 3) {
-            qCInfo(lcDb) << "upgrade form client < 2.3.0 detected! forcing remote discovery";
+        // - There was a bug in versions <2.3.0 that could lead to stale
+        //   local files and a remote discovery will fix them.
+        //   See #5190 #5242.
+        // - New remote HasZSyncMetadata permission added, invalidate cache
+        if (major == 2 && minor < 5) {
+            qCInfo(lcDb) << "upgrade from client < 2.5.0 detected! forcing remote discovery";
             forceRemoteDiscovery = true;
         }
 
@@ -634,7 +661,7 @@ bool SyncJournalDb::updateMetadataTableStructure()
     bool re = true;
 
     // check if the file_id column is there and create it if not
-    if (!checkConnect()) {
+    if (columns.isEmpty()) {
         return false;
     }
 
@@ -692,6 +719,16 @@ bool SyncJournalDb::updateMetadataTableStructure()
         commitInternal("update database structure: add path index");
     }
 
+    if (1) {
+        SqlQuery query(_db);
+        query.prepare("CREATE INDEX IF NOT EXISTS metadata_parent ON metadata(parent_hash(path));");
+        if (!query.exec()) {
+            sqlFail("updateMetadataTableStructure: create index parent", query);
+            re = false;
+        }
+        commitInternal("update database structure: add parent index");
+    }
+
     if (columns.indexOf("ignoredChildrenRemote") == -1) {
         SqlQuery query(_db);
         query.prepare("ALTER TABLE metadata ADD COLUMN ignoredChildrenRemote INT;");
@@ -721,7 +758,10 @@ bool SyncJournalDb::updateMetadataTableStructure()
         commitInternal("update database structure: add contentChecksumTypeId col");
     }
 
-    if (!tableColumns("uploadinfo").contains("contentChecksum")) {
+    auto uploadInfoColumns = tableColumns("uploadinfo");
+    if (uploadInfoColumns.isEmpty())
+        return false;
+    if (!uploadInfoColumns.contains("contentChecksum")) {
         SqlQuery query(_db);
         query.prepare("ALTER TABLE uploadinfo ADD COLUMN contentChecksum TEXT;");
         if (!query.exec()) {
@@ -731,7 +771,10 @@ bool SyncJournalDb::updateMetadataTableStructure()
         commitInternal("update database structure: add contentChecksum col for uploadinfo");
     }
 
-    if (!tableColumns("conflicts").contains("basePath")) {
+    auto conflictsColumns = tableColumns("conflicts");
+    if (conflictsColumns.isEmpty())
+        return false;
+    if (!conflictsColumns.contains("basePath")) {
         SqlQuery query(_db);
         query.prepare("ALTER TABLE conflicts ADD COLUMN basePath TEXT;");
         if (!query.exec()) {
@@ -740,7 +783,6 @@ bool SyncJournalDb::updateMetadataTableStructure()
         }
     }
 
-
     return re;
 }
 
@@ -749,8 +791,7 @@ bool SyncJournalDb::updateErrorBlacklistTableStructure()
     auto columns = tableColumns("blacklist");
     bool re = true;
 
-    // check if the file_id column is there and create it if not
-    if (!checkConnect()) {
+    if (columns.isEmpty()) {
         return false;
     }
 
@@ -818,7 +859,7 @@ QVector<QByteArray> SyncJournalDb::tableColumns(const QByteArray &table)
     if (!query.exec()) {
         return columns;
     }
-    while (query.next()) {
+    while (query.next().hasData) {
         columns.append(query.baValue(1));
     }
     qCDebug(lcDb) << "Columns in the current journal: " << columns;
@@ -828,11 +869,6 @@ QVector<QByteArray> SyncJournalDb::tableColumns(const QByteArray &table)
 qint64 SyncJournalDb::getPHash(const QByteArray &file)
 {
     int64_t h;
-
-    if (file.isEmpty()) {
-        return -1;
-    }
-
     int len = file.length();
 
     h = c_jhash64((uint8_t *)file.data(), len, 0);
@@ -973,15 +1009,15 @@ bool SyncJournalDb::getFileRecord(const QByteArray &filename, SyncJournalFileRec
             return false;
         }
 
-        if (_getFileRecordQuery.next()) {
+        auto next = _getFileRecordQuery.next();
+        if (!next.ok) {
+            QString err = _getFileRecordQuery.error();
+            qCWarning(lcDb) << "No journal entry found for " << filename << "Error: " << err;
+            close();
+            return false;
+        }
+        if (next.hasData) {
             fillFileRecordFromGetQuery(*rec, _getFileRecordQuery);
-        } else {
-            int errId = _getFileRecordQuery.errorId();
-            if (errId != SQLITE_DONE) { // only do this if the problem is different from SQLITE_DONE
-                QString err = _getFileRecordQuery.error();
-                qCWarning(lcDb) << "No journal entry found for " << filename << "Error: " << err;
-                close();
-            }
         }
     }
     return true;
@@ -1010,7 +1046,10 @@ bool SyncJournalDb::getFileRecordByInode(quint64 inode, SyncJournalFileRecord *r
     if (!_getFileRecordQueryByInode.exec())
         return false;
 
-    if (_getFileRecordQueryByInode.next())
+    auto next = _getFileRecordQueryByInode.next();
+    if (!next.ok)
+        return false;
+    if (next.hasData)
         fillFileRecordFromGetQuery(*rec, _getFileRecordQueryByInode);
 
     return true;
@@ -1034,7 +1073,13 @@ bool SyncJournalDb::getFileRecordsByFileId(const QByteArray &fileId, const std::
     if (!_getFileRecordQueryByFileId.exec())
         return false;
 
-    while (_getFileRecordQueryByFileId.next()) {
+    forever {
+        auto next = _getFileRecordQueryByFileId.next();
+        if (!next.ok)
+            return false;
+        if (!next.hasData)
+            break;
+
         SyncJournalFileRecord rec;
         fillFileRecordFromGetQuery(rec, _getFileRecordQueryByFileId);
         rowCallback(rec);
@@ -1086,7 +1131,13 @@ bool SyncJournalDb::getFilesBelowPath(const QByteArray &path, const std::functio
         return false;
     }
 
-    while (query->next()) {
+    forever {
+        auto next = query->next();
+        if (!next.ok)
+            return false;
+        if (!next.hasData)
+            break;
+
         SyncJournalFileRecord rec;
         fillFileRecordFromGetQuery(rec, *query);
         rowCallback(rec);
@@ -1095,52 +1146,41 @@ bool SyncJournalDb::getFilesBelowPath(const QByteArray &path, const std::functio
     return true;
 }
 
-bool SyncJournalDb::postSyncCleanup(const QSet<QString> &filepathsToKeep,
-    const QSet<QString> &prefixesToKeep)
+bool SyncJournalDb::listFilesInPath(const QByteArray& path,
+                                    const std::function<void (const SyncJournalFileRecord &)>& rowCallback)
 {
     QMutexLocker locker(&_mutex);
 
-    if (!checkConnect()) {
+    if (_metadataTableIsEmpty)
+        return true;
+
+    if (!checkConnect())
         return false;
-    }
 
-    SqlQuery query(_db);
-    query.prepare("SELECT phash, path FROM metadata order by path");
-
-    if (!query.exec()) {
+    if (!_listFilesInPathQuery.initOrReset(QByteArrayLiteral(
+            GET_FILE_RECORD_QUERY " WHERE parent_hash(path) = ?1 ORDER BY path||'/' ASC"), _db))
         return false;
-    }
 
-    QByteArrayList superfluousItems;
+    _listFilesInPathQuery.bindValue(1, getPHash(path));
 
-    while (query.next()) {
-        const QString file = query.baValue(1);
-        bool keep = filepathsToKeep.contains(file);
-        if (!keep) {
-            foreach (const QString &prefix, prefixesToKeep) {
-                if (file.startsWith(prefix)) {
-                    keep = true;
-                    break;
-                }
-            }
-        }
-        if (!keep) {
-            superfluousItems.append(query.baValue(0));
-        }
-    }
+    if (!_listFilesInPathQuery.exec())
+        return false;
 
-    if (superfluousItems.count()) {
-        QByteArray sql = "DELETE FROM metadata WHERE phash in (" + superfluousItems.join(",") + ")";
-        qCInfo(lcDb) << "Sync Journal cleanup for" << superfluousItems;
-        SqlQuery delQuery(_db);
-        delQuery.prepare(sql);
-        if (!delQuery.exec()) {
+    forever {
+        auto next = _listFilesInPathQuery.next();
+        if (!next.ok)
             return false;
-        }
-    }
+        if (!next.hasData)
+            break;
 
-    // Incorporate results back into main DB
-    walCheckpoint();
+        SyncJournalFileRecord rec;
+        fillFileRecordFromGetQuery(rec, _listFilesInPathQuery);
+        if (!rec._path.startsWith(path) || rec._path.indexOf("/", path.size() + 1) > 0) {
+            qWarning(lcDb) << "hash collision " << path << rec._path;
+            continue;
+        }
+        rowCallback(rec);
+    }
 
     return true;
 }
@@ -1156,7 +1196,7 @@ int SyncJournalDb::getFileRecordCount()
         return -1;
     }
 
-    if (query.next()) {
+    if (query.next().hasData) {
         int count = query.intValue(0);
         return count;
     }
@@ -1193,7 +1233,7 @@ bool SyncJournalDb::updateFileRecordChecksum(const QString &filename,
 }
 
 bool SyncJournalDb::updateLocalMetadata(const QString &filename,
-    qint64 modtime, quint64 size, quint64 inode)
+    qint64 modtime, qint64 size, quint64 inode)
 
 {
     QMutexLocker locker(&_mutex);
@@ -1221,27 +1261,38 @@ bool SyncJournalDb::updateLocalMetadata(const QString &filename,
     return _setFileRecordLocalMetadataQuery.exec();
 }
 
-bool SyncJournalDb::setFileRecordMetadata(const SyncJournalFileRecord &record)
+Optional<SyncJournalDb::HasHydratedDehydrated> SyncJournalDb::hasHydratedOrDehydratedFiles(const QByteArray &filename)
 {
-    SyncJournalFileRecord existing;
-    if (!getFileRecord(record._path, &existing))
-        return false;
+    QMutexLocker locker(&_mutex);
+    if (!checkConnect())
+        return {};
 
-    // If there's no existing record, just insert the new one.
-    if (!existing.isValid()) {
-        return setFileRecord(record);
+    auto &query = _countDehydratedFilesQuery;
+    if (!query.initOrReset(QByteArrayLiteral(
+            "SELECT DISTINCT type FROM metadata"
+            " WHERE (" IS_PREFIX_PATH_OR_EQUAL("?1", "path") " OR ?1 == '');"), _db)) {
+        return {};
     }
 
-    // Update the metadata on the existing record.
-    existing._inode = record._inode;
-    existing._modtime = record._modtime;
-    existing._type = record._type;
-    existing._etag = record._etag;
-    existing._fileId = record._fileId;
-    existing._remotePerm = record._remotePerm;
-    existing._fileSize = record._fileSize;
-    existing._serverHasIgnoredFiles = record._serverHasIgnoredFiles;
-    return setFileRecord(existing);
+    query.bindValue(1, filename);
+    if (!query.exec())
+        return {};
+
+    HasHydratedDehydrated result;
+    forever {
+        auto next = query.next();
+        if (!next.ok)
+            return {};
+        if (!next.hasData)
+            break;
+        auto type = static_cast<ItemType>(query.intValue(0));
+        if (type == ItemTypeFile || type == ItemTypeVirtualFileDehydration)
+            result.hasHydrated = true;
+        if (type == ItemTypeVirtualFile || type == ItemTypeVirtualFileDownload)
+            result.hasDehydrated = true;
+    }
+
+    return result;
 }
 
 static void toDownloadInfo(SqlQuery &query, SyncJournalDb::DownloadInfo *res)
@@ -1290,10 +1341,8 @@ SyncJournalDb::DownloadInfo SyncJournalDb::getDownloadInfo(const QString &file)
             return res;
         }
 
-        if (_getDownloadInfoQuery.next()) {
+        if (_getDownloadInfoQuery.next().hasData) {
             toDownloadInfo(_getDownloadInfoQuery, &res);
-        } else {
-            res._valid = false;
         }
     }
     return res;
@@ -1347,7 +1396,7 @@ QVector<SyncJournalDb::DownloadInfo> SyncJournalDb::getAndDeleteStaleDownloadInf
     QStringList superfluousPaths;
     QVector<SyncJournalDb::DownloadInfo> deleted_entries;
 
-    while (query.next()) {
+    while (query.next().hasData) {
         const QString file = query.stringValue(3); // path
         if (!keep.contains(file)) {
             superfluousPaths.append(file);
@@ -1374,7 +1423,7 @@ int SyncJournalDb::downloadInfoCount()
         if (!query.exec()) {
             sqlFail("Count number of downloadinfo entries failed", query);
         }
-        if (query.next()) {
+        if (query.next().hasData) {
             re = query.intValue(0);
         }
     }
@@ -1399,7 +1448,7 @@ SyncJournalDb::UploadInfo SyncJournalDb::getUploadInfo(const QString &file)
             return res;
         }
 
-        if (_getUploadInfoQuery.next()) {
+        if (_getUploadInfoQuery.next().hasData) {
             bool ok = true;
             res._chunk = _getUploadInfoQuery.intValue(0);
             res._transferid = _getUploadInfoQuery.intValue(1);
@@ -1468,7 +1517,7 @@ QVector<uint> SyncJournalDb::deleteStaleUploadInfos(const QSet<QString> &keep)
 
     QStringList superfluousPaths;
 
-    while (query.next()) {
+    while (query.next().hasData) {
         const QString file = query.stringValue(0);
         if (!keep.contains(file)) {
             superfluousPaths.append(file);
@@ -1492,7 +1541,7 @@ SyncJournalErrorBlacklistRecord SyncJournalDb::errorBlacklistEntry(const QString
         _getErrorBlacklistQuery.reset_and_clear_bindings();
         _getErrorBlacklistQuery.bindValue(1, file);
         if (_getErrorBlacklistQuery.exec()) {
-            if (_getErrorBlacklistQuery.next()) {
+            if (_getErrorBlacklistQuery.next().hasData) {
                 entry._lastTryEtag = _getErrorBlacklistQuery.baValue(0);
                 entry._lastTryModtime = _getErrorBlacklistQuery.int64Value(1);
                 entry._retryCount = _getErrorBlacklistQuery.intValue(2);
@@ -1528,7 +1577,7 @@ bool SyncJournalDb::deleteStaleErrorBlacklistEntries(const QSet<QString> &keep)
 
     QStringList superfluousPaths;
 
-    while (query.next()) {
+    while (query.next().hasData) {
         const QString file = query.stringValue(0);
         if (!keep.contains(file)) {
             superfluousPaths.append(file);
@@ -1540,6 +1589,16 @@ bool SyncJournalDb::deleteStaleErrorBlacklistEntries(const QSet<QString> &keep)
     return deleteBatch(delQuery, superfluousPaths, "blacklist");
 }
 
+void SyncJournalDb::deleteStaleFlagsEntries()
+{
+    QMutexLocker locker(&_mutex);
+    if (!checkConnect())
+        return;
+
+    SqlQuery delQuery("DELETE FROM flags WHERE path != '' AND path NOT IN (SELECT path from metadata);", _db);
+    delQuery.exec();
+}
+
 int SyncJournalDb::errorBlackListEntryCount()
 {
     int re = 0;
@@ -1551,7 +1610,7 @@ int SyncJournalDb::errorBlackListEntryCount()
         if (!query.exec()) {
             sqlFail("Count number of blacklist entries failed", query);
         }
-        if (query.next()) {
+        if (query.next().hasData) {
             re = query.intValue(0);
         }
     }
@@ -1649,17 +1708,18 @@ QVector<SyncJournalDb::PollInfo> SyncJournalDb::getPollInfos()
     if (!checkConnect())
         return res;
 
-    SqlQuery query("SELECT path, modtime, pollpath FROM poll", _db);
+    SqlQuery query("SELECT path, modtime, filesize, pollpath FROM async_poll", _db);
 
     if (!query.exec()) {
         return res;
     }
 
-    while (query.next()) {
+    while (query.next().hasData) {
         PollInfo info;
         info._file = query.stringValue(0);
         info._modtime = query.int64Value(1);
-        info._url = query.stringValue(2);
+        info._fileSize = query.int64Value(2);
+        info._url = query.stringValue(3);
         res.append(info);
     }
 
@@ -1676,14 +1736,15 @@ void SyncJournalDb::setPollInfo(const SyncJournalDb::PollInfo &info)
 
     if (info._url.isEmpty()) {
         qCDebug(lcDb) << "Deleting Poll job" << info._file;
-        SqlQuery query("DELETE FROM poll WHERE path=?", _db);
+        SqlQuery query("DELETE FROM async_poll WHERE path=?", _db);
         query.bindValue(1, info._file);
         query.exec();
     } else {
-        SqlQuery query("INSERT OR REPLACE INTO poll (path, modtime, pollpath) VALUES( ? , ? , ? )", _db);
+        SqlQuery query("INSERT OR REPLACE INTO async_poll (path, modtime, filesize, pollpath) VALUES( ? , ? , ? , ? )", _db);
         query.bindValue(1, info._file);
         query.bindValue(2, info._modtime);
-        query.bindValue(3, info._url);
+        query.bindValue(3, info._fileSize);
+        query.bindValue(4, info._url);
         query.exec();
     }
 }
@@ -1709,7 +1770,15 @@ QStringList SyncJournalDb::getSelectiveSyncList(SyncJournalDb::SelectiveSyncList
         *ok = false;
         return result;
     }
-    while (_getSelectiveSyncListQuery.next()) {
+    forever {
+        auto next = _getSelectiveSyncListQuery.next();
+        if (!next.ok) {
+            *ok = false;
+            return result;
+        }
+        if (!next.hasData)
+            break;
+
         auto entry = _getSelectiveSyncListQuery.stringValue(0);
         if (!entry.endsWith(QLatin1Char('/'))) {
             entry.append(QLatin1Char('/'));
@@ -1765,10 +1834,10 @@ void SyncJournalDb::avoidRenamesOnNextSync(const QByteArray &path)
 
     // We also need to remove the ETags so the update phase refreshes the directory paths
     // on the next sync
-    avoidReadFromDbOnNextSync(path);
+    schedulePathForRemoteDiscovery(path);
 }
 
-void SyncJournalDb::avoidReadFromDbOnNextSync(const QByteArray &fileName)
+void SyncJournalDb::schedulePathForRemoteDiscovery(const QByteArray &fileName)
 {
     QMutexLocker locker(&_mutex);
 
@@ -1832,12 +1901,12 @@ QByteArray SyncJournalDb::getChecksumType(int checksumTypeId)
         return {};
     query.bindValue(1, checksumTypeId);
     if (!query.exec()) {
-        return 0;
+        return QByteArray();
     }
 
-    if (!query.next()) {
+    if (!query.next().hasData) {
         qCWarning(lcDb) << "No checksum type mapping found for" << checksumTypeId;
-        return 0;
+        return QByteArray();
     }
     return query.baValue(0);
 }
@@ -1848,6 +1917,10 @@ int SyncJournalDb::mapChecksumType(const QByteArray &checksumType)
         return 0;
     }
 
+    auto it =  _checksymTypeCache.find(checksumType);
+    if (it != _checksymTypeCache.end())
+        return *it;
+
     // Ensure the checksum type is in the db
     if (!_insertChecksumTypeQuery.initOrReset(QByteArrayLiteral("INSERT OR IGNORE INTO checksumtype (name) VALUES (?1)"), _db))
         return 0;
@@ -1864,11 +1937,13 @@ int SyncJournalDb::mapChecksumType(const QByteArray &checksumType)
         return 0;
     }
 
-    if (!_getChecksumTypeIdQuery.next()) {
+    if (!_getChecksumTypeIdQuery.next().hasData) {
         qCWarning(lcDb) << "No checksum type mapping found for" << checksumType;
         return 0;
     }
-    return _getChecksumTypeIdQuery.intValue(0);
+    auto value = _getChecksumTypeIdQuery.intValue(0);
+    _checksymTypeCache[checksumType] = value;
+    return value;
 }
 
 QByteArray SyncJournalDb::dataFingerprint()
@@ -1885,7 +1960,7 @@ QByteArray SyncJournalDb::dataFingerprint()
         return QByteArray();
     }
 
-    if (!_getDataFingerprintQuery.next()) {
+    if (!_getDataFingerprintQuery.next().hasData) {
         return QByteArray();
     }
     return _getDataFingerprintQuery.baValue(0);
@@ -1940,7 +2015,7 @@ ConflictRecord SyncJournalDb::conflictRecord(const QByteArray &path)
     ASSERT(query.initOrReset(QByteArrayLiteral("SELECT baseFileId, baseModtime, baseEtag, basePath FROM conflicts WHERE path=?1;"), _db));
     query.bindValue(1, path);
     ASSERT(query.exec());
-    if (!query.next())
+    if (!query.next().hasData)
         return entry;
 
     entry.path = path;
@@ -1973,12 +2048,28 @@ QByteArrayList SyncJournalDb::conflictRecordPaths()
     ASSERT(query.exec());
 
     QByteArrayList paths;
-    while (query.next())
+    while (query.next().hasData)
         paths.append(query.baValue(0));
 
     return paths;
 }
 
+QByteArray SyncJournalDb::conflictFileBaseName(const QByteArray &conflictName)
+{
+    auto conflict = conflictRecord(conflictName);
+    QByteArray result;
+    if (conflict.isValid()) {
+        getFileRecordsByFileId(conflict.baseFileId, [&result](const SyncJournalFileRecord &record) {
+            if (!record._path.isEmpty())
+                result = record._path;
+        });
+    }
+
+    if (result.isEmpty())
+        result = Utility::conflictFileBaseNameFromPattern(conflictName);
+    return result;
+}
+
 void SyncJournalDb::clearFileTable()
 {
     QMutexLocker lock(&_mutex);
@@ -1994,18 +2085,172 @@ void SyncJournalDb::markVirtualFileForDownloadRecursively(const QByteArray &path
         return;
 
     static_assert(ItemTypeVirtualFile == 4 && ItemTypeVirtualFileDownload == 5, "");
-    SqlQuery query("UPDATE metadata SET type=5 WHERE " IS_PREFIX_PATH_OF("?1", "path") " AND type=4;", _db);
+    SqlQuery query("UPDATE metadata SET type=5 WHERE "
+                   "(" IS_PREFIX_PATH_OF("?1", "path") " OR ?1 == '') "
+                   "AND type=4;", _db);
     query.bindValue(1, path);
     query.exec();
 
-    // We also must make sure we do not read the files from the database (same logic as in avoidReadFromDbOnNextSync)
+    // We also must make sure we do not read the files from the database (same logic as in schedulePathForRemoteDiscovery)
     // This includes all the parents up to the root, but also all the directory within the selected dir.
     static_assert(ItemTypeDirectory == 2, "");
-    query.prepare("UPDATE metadata SET md5='_invalid_' WHERE (" IS_PREFIX_PATH_OF("?1", "path") " OR " IS_PREFIX_PATH_OR_EQUAL("path", "?1") ") AND type == 2;");
+    query.prepare("UPDATE metadata SET md5='_invalid_' WHERE "
+                  "(" IS_PREFIX_PATH_OF("?1", "path") " OR ?1 == '' OR " IS_PREFIX_PATH_OR_EQUAL("path", "?1") ") AND type == 2;");
     query.bindValue(1, path);
     query.exec();
 }
 
+Optional<PinState> SyncJournalDb::PinStateInterface::rawForPath(const QByteArray &path)
+{
+    QMutexLocker lock(&_db->_mutex);
+    if (!_db->checkConnect())
+        return {};
+
+    auto &query = _db->_getRawPinStateQuery;
+    ASSERT(query.initOrReset(QByteArrayLiteral(
+            "SELECT pinState FROM flags WHERE path == ?1;"),
+        _db->_db));
+    query.bindValue(1, path);
+    query.exec();
+
+    auto next = query.next();
+    if (!next.ok)
+        return {};
+    // no-entry means Inherited
+    if (!next.hasData)
+        return PinState::Inherited;
+
+    return static_cast<PinState>(query.intValue(0));
+}
+
+Optional<PinState> SyncJournalDb::PinStateInterface::effectiveForPath(const QByteArray &path)
+{
+    QMutexLocker lock(&_db->_mutex);
+    if (!_db->checkConnect())
+        return {};
+
+    auto &query = _db->_getEffectivePinStateQuery;
+    ASSERT(query.initOrReset(QByteArrayLiteral(
+            "SELECT pinState FROM flags WHERE"
+            // explicitly allow "" to represent the root path
+            // (it'd be great if paths started with a / and "/" could be the root)
+            " (" IS_PREFIX_PATH_OR_EQUAL("path", "?1") " OR path == '')"
+            " AND pinState is not null AND pinState != 0"
+            " ORDER BY length(path) DESC LIMIT 1;"),
+        _db->_db));
+    query.bindValue(1, path);
+    query.exec();
+
+    auto next = query.next();
+    if (!next.ok)
+        return {};
+    // If the root path has no setting, assume AlwaysLocal
+    if (!next.hasData)
+        return PinState::AlwaysLocal;
+
+    return static_cast<PinState>(query.intValue(0));
+}
+
+Optional<PinState> SyncJournalDb::PinStateInterface::effectiveForPathRecursive(const QByteArray &path)
+{
+    // Get the item's effective pin state. We'll compare subitem's pin states
+    // against this.
+    const auto basePin = effectiveForPath(path);
+    if (!basePin)
+        return {};
+
+    QMutexLocker lock(&_db->_mutex);
+    if (!_db->checkConnect())
+        return {};
+
+    // Find all the non-inherited pin states below the item
+    auto &query = _db->_getSubPinsQuery;
+    ASSERT(query.initOrReset(QByteArrayLiteral(
+            "SELECT DISTINCT pinState FROM flags WHERE"
+            " (" IS_PREFIX_PATH_OF("?1", "path") " OR ?1 == '')"
+            " AND pinState is not null and pinState != 0;"),
+        _db->_db));
+    query.bindValue(1, path);
+    query.exec();
+
+    // Check if they are all identical
+    forever {
+        auto next = query.next();
+        if (!next.ok)
+            return {};
+        if (!next.hasData)
+            break;
+        const auto subPin = static_cast<PinState>(query.intValue(0));
+        if (subPin != *basePin)
+            return PinState::Inherited;
+    }
+
+    return *basePin;
+}
+
+void SyncJournalDb::PinStateInterface::setForPath(const QByteArray &path, PinState state)
+{
+    QMutexLocker lock(&_db->_mutex);
+    if (!_db->checkConnect())
+        return;
+
+    auto &query = _db->_setPinStateQuery;
+    ASSERT(query.initOrReset(QByteArrayLiteral(
+            // If we had sqlite >=3.24.0 everywhere this could be an upsert,
+            // making further flags columns easy
+            //"INSERT INTO flags(path, pinState) VALUES(?1, ?2)"
+            //" ON CONFLICT(path) DO UPDATE SET pinState=?2;"),
+            // Simple version that doesn't work nicely with multiple columns:
+            "INSERT OR REPLACE INTO flags(path, pinState) VALUES(?1, ?2);"),
+        _db->_db));
+    query.bindValue(1, path);
+    query.bindValue(2, static_cast<int>(state));
+    query.exec();
+}
+
+void SyncJournalDb::PinStateInterface::wipeForPathAndBelow(const QByteArray &path)
+{
+    QMutexLocker lock(&_db->_mutex);
+    if (!_db->checkConnect())
+        return;
+
+    auto &query = _db->_wipePinStateQuery;
+    ASSERT(query.initOrReset(QByteArrayLiteral(
+            "DELETE FROM flags WHERE "
+            // Allow "" to delete everything
+            " (" IS_PREFIX_PATH_OR_EQUAL("?1", "path") " OR ?1 == '');"),
+        _db->_db));
+    query.bindValue(1, path);
+    query.exec();
+}
+
+Optional<QVector<QPair<QByteArray, PinState>>>
+SyncJournalDb::PinStateInterface::rawList()
+{
+    QMutexLocker lock(&_db->_mutex);
+    if (!_db->checkConnect())
+        return {};
+
+    SqlQuery query("SELECT path, pinState FROM flags;", _db->_db);
+    query.exec();
+
+    QVector<QPair<QByteArray, PinState>> result;
+    forever {
+        auto next = query.next();
+        if (!next.ok)
+            return {};
+        if (!next.hasData)
+            break;
+        result.append({ query.baValue(0), static_cast<PinState>(query.intValue(1)) });
+    }
+    return result;
+}
+
+SyncJournalDb::PinStateInterface SyncJournalDb::internalPinStates()
+{
+    return {this};
+}
+
 void SyncJournalDb::commit(const QString &context, bool startTrans)
 {
     QMutexLocker lock(&_mutex);
@@ -2022,6 +2267,17 @@ void SyncJournalDb::commitIfNeededAndStartNewTransaction(const QString &context)
     }
 }
 
+bool SyncJournalDb::open()
+{
+    QMutexLocker lock(&_mutex);
+    return checkConnect();
+}
+
+bool SyncJournalDb::isOpen()
+{
+    QMutexLocker lock(&_mutex);
+    return _db.isOpen();
+}
 
 void SyncJournalDb::commitInternal(const QString &context, bool startTrans)
 {
@@ -2038,11 +2294,6 @@ SyncJournalDb::~SyncJournalDb()
     close();
 }
 
-bool SyncJournalDb::isConnected()
-{
-    QMutexLocker lock(&_mutex);
-    return checkConnect();
-}
 
 bool operator==(const SyncJournalDb::DownloadInfo &lhs,
     const SyncJournalDb::DownloadInfo &rhs)
