@@ -43,7 +43,8 @@ enum csync_exclude_type_e {
   CSYNC_FILE_EXCLUDE_HIDDEN,
   CSYNC_FILE_EXCLUDE_STAT_FAILED,
   CSYNC_FILE_EXCLUDE_CONFLICT,
-  CSYNC_FILE_EXCLUDE_CANNOT_ENCODE
+  CSYNC_FILE_EXCLUDE_CANNOT_ENCODE,
+  CSYNC_FILE_EXCLUDE_SERVER_BLACKLISTED,
 };
 typedef enum csync_exclude_type_e CSYNC_EXCLUDE_TYPE;
 
@@ -102,7 +103,7 @@ public:
      * Primarily used in tests. Patterns added this way are preserved when
      * reloadExcludeFiles() is called.
      */
-    void addManualExclude(const QByteArray &expr);
+    void addManualExclude(const QString &expr);
 
     /**
      * Removes all manually added exclude patterns.
@@ -122,14 +123,21 @@ public:
     void setClientVersion(Version version);
 
     /**
-     * Generate a hook for traversal exclude pattern matching
-     * that csync can use.
+     * @brief Check if the given path should be excluded in a traversal situation.
      *
-     * Careful: The function will only be valid for as long as this
-     * ExcludedFiles instance stays alive.
+     * It does only part of the work that full() does because it's assumed
+     * that all leading directories have been run through traversal()
+     * before. This can be significantly faster.
+     *
+     * That means for 'foo/bar/file' only ('foo/bar/file', 'file') is checked
+     * against the exclude patterns.
+     *
+     * @param Path is folder-relative, should not start with a /.
+     *
+     * Note that this only matches patterns. It does not check whether the file
+     * or directory pointed to is hidden (or whether it even exists).
      */
-    auto csyncTraversalMatchFun() const
-        -> std::function<CSYNC_EXCLUDE_TYPE(const char *path, ItemType filetype)>;
+    CSYNC_EXCLUDE_TYPE traversalPatternMatch(const QString &path, ItemType filetype) const;
 
 public slots:
     /**
@@ -163,24 +171,7 @@ private:
      * Note that this only matches patterns. It does not check whether the file
      * or directory pointed to is hidden (or whether it even exists).
      */
-    CSYNC_EXCLUDE_TYPE fullPatternMatch(const char *path, ItemType filetype) const;
-
-    /**
-     * @brief Check if the given path should be excluded in a traversal situation.
-     *
-     * It does only part of the work that full() does because it's assumed
-     * that all leading directories have been run through traversal()
-     * before. This can be significantly faster.
-     *
-     * That means for 'foo/bar/file' only ('foo/bar/file', 'file') is checked
-     * against the exclude patterns.
-     *
-     * @param Path is folder-relative, should not start with a /.
-     *
-     * Note that this only matches patterns. It does not check whether the file
-     * or directory pointed to is hidden (or whether it even exists).
-     */
-    CSYNC_EXCLUDE_TYPE traversalPatternMatch(const char *path, ItemType filetype) const;
+    CSYNC_EXCLUDE_TYPE fullPatternMatch(const QString &path, ItemType filetype) const;
 
     /**
      * Generate optimized regular expressions for the exclude patterns.
@@ -213,14 +204,17 @@ private:
      */
     void prepare();
 
+    static QString extractBnameTrigger(const QString &exclude, bool wildcardsMatchSlash);
+    static QString convertToRegexpSyntax(QString exclude, bool wildcardsMatchSlash);
+
     /// Files to load excludes from
     QSet<QString> _excludeFiles;
 
     /// Exclude patterns added with addManualExclude()
-    QList<QByteArray> _manualExcludes;
+    QStringList _manualExcludes;
 
     /// List of all active exclude patterns
-    QList<QByteArray> _allExcludes;
+    QStringList _allExcludes;
 
     /// see prepare()
     QRegularExpression _bnameTraversalRegexFile;
@@ -246,7 +240,7 @@ private:
      */
     Version _clientVersion;
 
-    friend class ExcludedFilesTest;
+    friend class TestExcludedFiles;
 };
 
 #endif /* _CSYNC_EXCLUDE_H */
