@@ -20,12 +20,14 @@
 #include <random>
 
 #include "config.h"
+#include "common/asserts.h"
 #include "account.h"
 #include "accountstate.h"
 #include "connectionvalidator.h"
 #include "folder.h"
 #include "folderman.h"
 #include "logger.h"
+#include "logbrowser.h"
 #include "configfile.h"
 #include "socketapi.h"
 #include "sslerrordialog.h"
@@ -38,6 +40,7 @@
 #include "owncloudsetupwizard.h"
 #include "version.h"
 #include "csync_exclude.h"
+#include "common/vfs.h"
 
 #include "config.h"
 
@@ -65,8 +68,11 @@ namespace {
     static const char optionsC[] =
         "Options:\n"
         "  -h --help            : show this help screen.\n"
+        "  -s --showsettings    : show the settings dialog while starting.\n"
+        "  -q --quit            : quit the running instance\n"
         "  --logwindow          : open a window to show log output.\n"
         "  --logfile <filename> : write log output to file <filename>.\n"
+        "  --logfile -          : write log output to stdout.\n"
         "  --logdir <name>      : write each sync log output in a new file\n"
         "                         in folder <name>.\n"
         "  --logexpire <hours>  : removes logs older than <hours> hours.\n"
@@ -168,7 +174,9 @@ Application::Application(int &argc, char **argv)
     , _userTriggeredConnect(false)
     , _debugMode(false)
 {
+#ifdef Q_OS_LINUX
     _startedAt.start();
+#endif
 
     qsrand(std::random_device()());
 
@@ -183,7 +191,6 @@ Application::Application(int &argc, char **argv)
     setOrganizationDomain(QLatin1String(APPLICATION_REV_DOMAIN));
     setApplicationName(_theme->appName());
     setWindowIcon(_theme->applicationIcon());
-    setAttribute(Qt::AA_UseHighDpiPixmaps, true);
 
     if (!ConfigFile().exists()) {
         // Migrate from version <= 2.4
@@ -231,6 +238,11 @@ Application::Application(int &argc, char **argv)
     if (_helpOnly || _versionOnly)
         return;
 
+    if (_quitInstance) {
+        QTimer::singleShot(0, qApp, &QApplication::quit);
+        return;
+    }
+
     if (isRunning())
         return;
 
@@ -251,6 +263,28 @@ Application::Application(int &argc, char **argv)
     if (!AbstractNetworkJob::httpTimeout)
         AbstractNetworkJob::httpTimeout = cfg.timeout();
 
+#ifdef PLUGINDIR
+    // Setup extra plugin search path
+    QString extraPluginPath = QStringLiteral(PLUGINDIR);
+    if (!extraPluginPath.isEmpty()) {
+        if (QDir::isRelativePath(extraPluginPath))
+            extraPluginPath = QDir(QApplication::applicationDirPath()).filePath(extraPluginPath);
+        qCInfo(lcApplication) << "Adding extra plugin search path:" << extraPluginPath;
+        QStringList pluginPath = libraryPaths();
+        pluginPath.prepend(extraPluginPath);
+        setLibraryPaths(pluginPath);
+    }
+#endif
+
+    // Check vfs plugins
+    if (Theme::instance()->showVirtualFilesOption() && bestAvailableVfsMode() == Vfs::Off) {
+        qCWarning(lcApplication) << "Theme wants to show vfs mode, but no vfs plugins are available";
+    }
+    if (isVfsPluginAvailable(Vfs::WindowsCfApi))
+        qCInfo(lcApplication) << "VFS windows plugin is available";
+    if (isVfsPluginAvailable(Vfs::WithSuffix))
+        qCInfo(lcApplication) << "VFS suffix plugin is available";
+
     _folderManager.reset(new FolderMan);
 
     connect(this, &SharedTools::QtSingleApplication::messageReceived, this, &Application::slotParseMessage);
@@ -287,6 +321,9 @@ Application::Application(int &argc, char **argv)
     if (_showLogWindow) {
         _gui->slotToggleLogBrowser(); // _showLogWindow is set in parseOptions.
     }
+    if (_showSettings) {
+        _gui->slotShowSettings();
+    }
 
     FolderMan::instance()->setupFolders();
     _proxy.setupQtProxyFromConfig(); // folders have to be defined first, than we set up the Qt proxy.
@@ -423,6 +460,18 @@ void Application::slotCrash()
     Utility::crash();
 }
 
+void Application::slotCrashEnforce()
+{
+    ENFORCE(1==0);
+}
+
+
+void Application::slotCrashFatal()
+{
+    qFatal("la Qt fatale");
+}
+
+
 void Application::slotownCloudWizardDone(int res)
 {
     AccountManager *accountMan = AccountManager::instance();
@@ -460,12 +509,11 @@ void Application::setupLogging()
     logger->setLogExpire(_logExpire);
     logger->setLogFlush(_logFlush);
     logger->setLogDebug(_logDebug);
-    if (!logger->isLoggingToFile() && ConfigFile().automaticLogDir()) {
-        logger->setupTemporaryFolderLogDir();
-    }
-
     logger->enterNextLogFile();
 
+    // Possibly configure logging from config file
+    LogBrowser::setupLoggingFromConfig();
+
     qCInfo(lcApplication) << QString::fromLatin1("################## %1 locale:[%2] ui_lang:[%3] version:[%4] os:[%5]").arg(_theme->appName()).arg(QLocale::system().name()).arg(property("ui_lang").toString()).arg(_theme->version()).arg(Utility::platformName());
 }
 
@@ -479,15 +527,29 @@ void Application::slotParseMessage(const QString &msg, QObject *)
     if (msg.startsWith(QLatin1String("MSG_PARSEOPTIONS:"))) {
         const int lengthOfMsgPrefix = 17;
         QStringList options = msg.mid(lengthOfMsgPrefix).split(QLatin1Char('|'));
+        _showLogWindow = false;
+        _showSettings = false;
         parseOptions(options);
         setupLogging();
+        if (_showLogWindow) {
+            _gui->slotToggleLogBrowser(); // _showLogWindow is set in parseOptions.
+        }
+        if (_showSettings) {
+            _gui->slotShowSettings();
+        }
+        if (_quitInstance) {
+            qApp->quit();
+        }
+
     } else if (msg.startsWith(QLatin1String("MSG_SHOWSETTINGS"))) {
+#ifdef Q_OS_LINUX
         qCInfo(lcApplication) << "Running for" << _startedAt.elapsed() / 1000.0 << "sec";
         if (_startedAt.elapsed() < 10 * 1000) {
             // This call is mirrored with the one in int main()
             qCWarning(lcApplication) << "Ignoring MSG_SHOWSETTINGS, possibly double-invocation of client via session restore and auto start";
             return;
         }
+#endif
         showSettingsDialog();
     }
 }
@@ -505,6 +567,10 @@ void Application::parseOptions(const QStringList &options)
         if (option == QLatin1String("--help") || option == QLatin1String("-h")) {
             setHelp();
             break;
+        } else if (option == QLatin1String("--showsettings") || option == QLatin1String("-s")) {
+            _showSettings = true;
+        } else if (option == QLatin1String("--quit") || option == QLatin1String("-q")) {
+            _quitInstance = true;
         } else if (option == QLatin1String("--logwindow") || option == QLatin1String("-l")) {
             _showLogWindow = true;
         } else if (option == QLatin1String("--logfile")) {
@@ -521,7 +587,7 @@ void Application::parseOptions(const QStringList &options)
             }
         } else if (option == QLatin1String("--logexpire")) {
             if (it.hasNext() && !it.peekNext().startsWith(QLatin1String("--"))) {
-                _logExpire = it.next().toInt();
+                _logExpire = std::chrono::hours(it.next().toInt());
             } else {
                 showHint("Log expiration not specified");
             }
@@ -717,7 +783,7 @@ void Application::openVirtualFile(const QString &filename)
         // TODO: show a QMessageBox for errors
         return;
     }
-    folder->downloadVirtualFile(relativePath);
+    folder->implicitlyHydrateFile(relativePath);
     QString normalName = filename.left(filename.size() - virtualFileExt.size());
     auto con = QSharedPointer<QMetaObject::Connection>::create();
     *con = QObject::connect(folder, &Folder::syncFinished, [con, normalName] {
@@ -728,6 +794,13 @@ void Application::openVirtualFile(const QString &filename)
     });
 }
 
+void Application::tryTrayAgain()
+{
+    qCInfo(lcApplication) << "Trying tray icon, tray available:" << QSystemTrayIcon::isSystemTrayAvailable();
+    if (!_gui->contextMenuVisible())
+        _gui->hideAndShowTray();
+}
+
 bool Application::event(QEvent *event)
 {
 #ifdef Q_OS_MAC
