@@ -107,7 +107,7 @@ void OAuth::start()
                     QUrl messageUrl = json["message_url"].toString();
 
                     if (reply->error() != QNetworkReply::NoError || jsonParseError.error != QJsonParseError::NoError
-                        || json.isEmpty() || refreshToken.isEmpty() || accessToken.isEmpty()
+                        || jsonData.isEmpty() || json.isEmpty() || refreshToken.isEmpty() || accessToken.isEmpty()
                         || json["token_type"].toString() != QLatin1String("Bearer")) {
                         QString errorReason;
                         QString errorFromJson = json["error"].toString();
@@ -117,6 +117,12 @@ void OAuth::start()
                         } else if (reply->error() != QNetworkReply::NoError) {
                             errorReason = tr("There was an error accessing the 'token' endpoint: <br><em>%1</em>")
                                               .arg(reply->errorString().toHtmlEscaped());
+                        } else if (jsonData.isEmpty()) {
+                            // Can happen if a funky load balancer strips away POST data, e.g. BigIP APM my.policy
+                            errorReason = tr("Empty JSON from OAuth2 redirect");
+                            // We explicitly have this as error case since the json qcWarning output below is misleading,
+                            // it will show a fake json will null values that actually never was received like this as
+                            // soon as you access json["whatever"] the debug output json will claim to have "whatever":null
                         } else if (jsonParseError.error != QJsonParseError::NoError) {
                             errorReason = tr("Could not parse the JSON returned from the server: <br><em>%1</em>")
                                               .arg(jsonParseError.errorString());
@@ -165,6 +171,7 @@ QUrl OAuth::authorisationLink() const
     if (!_expectedUser.isNull())
         query.addQueryItem("user", _expectedUser);
     QUrl url = Utility::concatUrlPath(_account->url(), QLatin1String("/index.php/apps/oauth2/authorize"), query);
+    url.setQuery(url.query(QUrl::FullyEncoded).replace('+', QLatin1String("%2B"))); // Issue #7762
     return url;
 }
 
