@@ -31,7 +31,8 @@
 #include "theme.h"
 #include "filesystem.h"
 #include "localdiscoverytracker.h"
-
+#include "csync_exclude.h"
+#include "common/vfs.h"
 #include "creds/abstractcredentials.h"
 
 #include <QTimer>
@@ -41,6 +42,7 @@
 
 #include <QMessageBox>
 #include <QPushButton>
+#include <QApplication>
 
 static const char versionC[] = "version";
 
@@ -49,17 +51,17 @@ namespace OCC {
 Q_LOGGING_CATEGORY(lcFolder, "gui.folder", QtInfoMsg)
 
 Folder::Folder(const FolderDefinition &definition,
-    AccountState *accountState,
+    AccountState *accountState, std::unique_ptr<Vfs> vfs,
     QObject *parent)
     : QObject(parent)
     , _accountState(accountState)
     , _definition(definition)
-    , _csyncUnavail(false)
     , _lastSyncDuration(0)
     , _consecutiveFailingSyncs(0)
     , _consecutiveFollowUpSyncs(0)
     , _journal(_definition.absoluteJournalPath())
     , _fileLog(new SyncRunFileLog)
+    , _vfs(vfs.release())
 {
     _timeSinceLastSyncStart.start();
     _timeSinceLastSyncDone.start();
@@ -88,13 +90,10 @@ Folder::Folder(const FolderDefinition &definition,
 
     connect(_engine.data(), &SyncEngine::started, this, &Folder::slotSyncStarted, Qt::QueuedConnection);
     connect(_engine.data(), &SyncEngine::finished, this, &Folder::slotSyncFinished, Qt::QueuedConnection);
-    connect(_engine.data(), &SyncEngine::csyncUnavailable, this, &Folder::slotCsyncUnavailable, Qt::QueuedConnection);
 
     //direct connection so the message box is blocking the sync.
     connect(_engine.data(), &SyncEngine::aboutToRemoveAllFiles,
         this, &Folder::slotAboutToRemoveAllFiles);
-    connect(_engine.data(), &SyncEngine::aboutToRestoreBackup,
-        this, &Folder::slotAboutToRestoreBackup);
     connect(_engine.data(), &SyncEngine::transmissionProgress, this, &Folder::slotTransmissionProgress);
     connect(_engine.data(), &SyncEngine::itemCompleted,
         this, &Folder::slotItemCompleted);
@@ -118,10 +117,33 @@ Folder::Folder(const FolderDefinition &definition,
         _localDiscoveryTracker.data(), &LocalDiscoveryTracker::slotSyncFinished);
     connect(_engine.data(), &SyncEngine::itemCompleted,
         _localDiscoveryTracker.data(), &LocalDiscoveryTracker::slotItemCompleted);
+
+    // Potentially upgrade suffix vfs to windows vfs
+    ENFORCE(_vfs);
+    if (_definition.virtualFilesMode == Vfs::WithSuffix
+        && _definition.upgradeVfsMode
+        && isVfsPluginAvailable(Vfs::WindowsCfApi)) {
+        if (auto winvfs = createVfsFromPlugin(Vfs::WindowsCfApi)) {
+            // Wipe the existing suffix files from fs and journal
+            SyncEngine::wipeVirtualFiles(path(), _journal, *_vfs);
+
+            // Then switch to winvfs mode
+            _vfs.reset(winvfs.release());
+            _definition.virtualFilesMode = Vfs::WindowsCfApi;
+            saveToSettings();
+        }
+    }
+
+    // Initialize the vfs plugin
+    startVfs();
 }
 
 Folder::~Folder()
 {
+    // If wipeForRemoval() was called the vfs has already shut down.
+    if (_vfs)
+        _vfs->stop();
+
     // Reset then engine first as it will abort and try to access members of the Folder
     _engine.reset();
 }
@@ -221,7 +243,12 @@ QString Folder::cleanPath() const
 
 bool Folder::isBusy() const
 {
-    return _engine->isSyncRunning();
+    return isSyncRunning();
+}
+
+bool Folder::isSyncRunning() const
+{
+    return _engine->isSyncRunning() || _vfs->isHydrating();
 }
 
 QString Folder::remotePath() const
@@ -229,6 +256,14 @@ QString Folder::remotePath() const
     return _definition.targetPath;
 }
 
+QString Folder::remotePathTrailingSlash() const
+{
+    QString result = remotePath();
+    if (!result.endsWith('/'))
+        result.append('/');
+    return result;
+}
+
 QUrl Folder::remoteUrl() const
 {
     return Utility::concatUrlPath(_accountState->account()->davUrl(), remotePath());
@@ -331,13 +366,13 @@ void Folder::etagRetreivedFromSyncEngine(const QString &etag)
 void Folder::showSyncResultPopup()
 {
     if (_syncResult.firstItemNew()) {
-        createGuiLog(_syncResult.firstItemNew()->_file, LogStatusNew, _syncResult.numNewItems());
+        createGuiLog(_syncResult.firstItemNew()->destination(), LogStatusNew, _syncResult.numNewItems());
     }
     if (_syncResult.firstItemDeleted()) {
-        createGuiLog(_syncResult.firstItemDeleted()->_file, LogStatusRemove, _syncResult.numRemovedItems());
+        createGuiLog(_syncResult.firstItemDeleted()->destination(), LogStatusRemove, _syncResult.numRemovedItems());
     }
     if (_syncResult.firstItemUpdated()) {
-        createGuiLog(_syncResult.firstItemUpdated()->_file, LogStatusUpdated, _syncResult.numUpdatedItems());
+        createGuiLog(_syncResult.firstItemUpdated()->destination(), LogStatusUpdated, _syncResult.numUpdatedItems());
     }
 
     if (_syncResult.firstItemRenamed()) {
@@ -348,12 +383,12 @@ void Folder::showSyncResultPopup()
         if (renTarget != renSource) {
             status = LogStatusMove;
         }
-        createGuiLog(_syncResult.firstItemRenamed()->_originalFile, status,
+        createGuiLog(_syncResult.firstItemRenamed()->_file, status,
             _syncResult.numRenamedItems(), _syncResult.firstItemRenamed()->_renameTarget);
     }
 
     if (_syncResult.firstNewConflictItem()) {
-        createGuiLog(_syncResult.firstNewConflictItem()->_file, LogStatusConflict, _syncResult.numNewConflictItems());
+        createGuiLog(_syncResult.firstNewConflictItem()->destination(), LogStatusConflict, _syncResult.numNewConflictItems());
     }
     if (int errorCount = _syncResult.numErrorItems()) {
         createGuiLog(_syncResult.firstItemError()->_file, LogStatusError, errorCount);
@@ -381,9 +416,9 @@ void Folder::createGuiLog(const QString &filename, LogStatus status, int count,
             break;
         case LogStatusNew:
             if (count > 1) {
-                text = tr("%1 and %n other file(s) have been downloaded.", "", count - 1).arg(file);
+                text = tr("%1 and %n other file(s) have been added.", "", count - 1).arg(file);
             } else {
-                text = tr("%1 has been downloaded.", "%1 names a file.").arg(file);
+                text = tr("%1 has been added.", "%1 names a file.").arg(file);
             }
             break;
         case LogStatusUpdated:
@@ -429,6 +464,35 @@ void Folder::createGuiLog(const QString &filename, LogStatus status, int count,
     }
 }
 
+void Folder::startVfs()
+{
+    ENFORCE(_vfs);
+    ENFORCE(_vfs->mode() == _definition.virtualFilesMode);
+
+    VfsSetupParams vfsParams;
+    vfsParams.filesystemPath = path();
+    vfsParams.remotePath = remotePathTrailingSlash();
+    vfsParams.account = _accountState->account();
+    vfsParams.journal = &_journal;
+    vfsParams.providerName = Theme::instance()->appNameGUI();
+    vfsParams.providerVersion = Theme::instance()->version();
+
+    connect(_vfs.data(), &Vfs::beginHydrating, this, &Folder::slotHydrationStarts);
+    connect(_vfs.data(), &Vfs::doneHydrating, this, &Folder::slotHydrationDone);
+
+    connect(&_engine->syncFileStatusTracker(), &SyncFileStatusTracker::fileStatusChanged,
+            _vfs.data(), &Vfs::fileStatusChanged);
+
+    _vfs->start(vfsParams);
+
+    // Immediately mark the sqlite temporaries as excluded. They get recreated
+    // on db-open and need to get marked again every time.
+    QString stateDbFile = _journal.databaseFilePath();
+    _journal.open();
+    _vfs->fileStatusChanged(stateDbFile + "-wal", SyncFileStatus::StatusExcluded);
+    _vfs->fileStatusChanged(stateDbFile + "-shm", SyncFileStatus::StatusExcluded);
+}
+
 int Folder::slotDiscardDownloadProgress()
 {
     // Delete from journal and from filesystem.
@@ -459,7 +523,7 @@ int Folder::slotWipeErrorBlacklist()
     return _journal.wipeErrorBlacklist();
 }
 
-void Folder::slotWatchedPathChanged(const QString &path)
+void Folder::slotWatchedPathChanged(const QString &path, ChangeReason reason)
 {
     if (!path.startsWith(this->path())) {
         qCDebug(lcFolder) << "Changed path is not contained in folder, ignoring:" << path;
@@ -490,15 +554,29 @@ void Folder::slotWatchedPathChanged(const QString &path)
     }
 #endif
 
-    // Check that the mtime actually changed.
-    SyncJournalFileRecord record;
-    if (_journal.getFileRecord(relativePathBytes, &record)
-        && record.isValid()
-        && !FileSystem::fileChanged(path, record._fileSize, record._modtime)) {
-        qCInfo(lcFolder) << "Ignoring spurious notification for file" << relativePath;
-        return; // probably a spurious notification
-    }
 
+    SyncJournalFileRecord record;
+    _journal.getFileRecord(relativePathBytes, &record);
+    if (reason != ChangeReason::UnLock) {
+        // Check that the mtime/size actually changed or there was
+        // an attribute change (pin state) that caused the notification
+        bool spurious = false;
+        if (record.isValid()
+            && !FileSystem::fileChanged(path, record._fileSize, record._modtime)) {
+            spurious = true;
+
+            if (auto pinState = _vfs->pinState(relativePath.toString())) {
+                if (*pinState == PinState::AlwaysLocal && record.isVirtualFile())
+                    spurious = false;
+                if (*pinState == PinState::OnlineOnly && record.isFile())
+                    spurious = false;
+            }
+        }
+        if (spurious) {
+            qCInfo(lcFolder) << "Ignoring spurious notification for file" << relativePath;
+            return; // probably a spurious notification
+        }
+    }
     warnOnNewExcludedItem(record, relativePath);
 
     emit watchedFileChangedExternally(path);
@@ -508,38 +586,85 @@ void Folder::slotWatchedPathChanged(const QString &path)
     scheduleThisFolderSoon();
 }
 
-void Folder::downloadVirtualFile(const QString &_relativepath)
+void Folder::implicitlyHydrateFile(const QString &relativepath)
 {
-    qCInfo(lcFolder) << "Download virtual file: " << _relativepath;
-    auto relativepath = _relativepath.toUtf8();
+    qCInfo(lcFolder) << "Implicitly hydrate virtual file:" << relativepath;
 
     // Set in the database that we should download the file
     SyncJournalFileRecord record;
-    _journal.getFileRecord(relativepath, &record);
-    if (!record.isValid())
+    _journal.getFileRecord(relativepath.toUtf8(), &record);
+    if (!record.isValid()) {
+        qCInfo(lcFolder) << "Did not find file in db";
         return;
-    if (record._type == ItemTypeVirtualFile) {
-        record._type = ItemTypeVirtualFileDownload;
-        _journal.setFileRecord(record);
-        // Make sure we go over that file during the discovery
-        _journal.avoidReadFromDbOnNextSync(relativepath);
-    } else if (record._type == ItemTypeDirectory) {
-        _journal.markVirtualFileForDownloadRecursively(relativepath);
-    } else {
-        qCWarning(lcFolder) << "Invalid existing record " << record._type << " for file " << _relativepath;
+    }
+    if (!record.isVirtualFile()) {
+        qCInfo(lcFolder) << "The file is not virtual";
+        return;
+    }
+    record._type = ItemTypeVirtualFileDownload;
+    _journal.setFileRecord(record);
+
+    // Change the file's pin state if it's contradictory to being hydrated
+    // (suffix-virtual file's pin state is stored at the hydrated path)
+    const auto pin = _vfs->pinState(relativepath);
+    if (pin && *pin == PinState::OnlineOnly) {
+        _vfs->setPinState(relativepath, PinState::Unspecified);
     }
 
-    // Schedule a sync (Folder man will start the sync in a few ms)
+    // Add to local discovery
+    schedulePathForLocalDiscovery(relativepath);
     slotScheduleThisFolder();
 }
 
-
-void Folder::setUseVirtualFiles(bool enabled)
+void Folder::setSupportsVirtualFiles(bool enabled)
 {
-    _definition.useVirtualFiles = enabled;
-    if (enabled)
-        _saveInFoldersWithPlaceholders = true;
-    saveToSettings();
+    Vfs::Mode newMode = _definition.virtualFilesMode;
+    if (enabled && _definition.virtualFilesMode == Vfs::Off) {
+        newMode = bestAvailableVfsMode();
+    } else if (!enabled && _definition.virtualFilesMode != Vfs::Off) {
+        newMode = Vfs::Off;
+    }
+
+    if (newMode != _definition.virtualFilesMode) {
+        // TODO: Must wait for current sync to finish!
+        SyncEngine::wipeVirtualFiles(path(), _journal, *_vfs);
+
+        _vfs->stop();
+        _vfs->unregisterFolder();
+
+        disconnect(_vfs.data(), 0, this, 0);
+        disconnect(&_engine->syncFileStatusTracker(), 0, _vfs.data(), 0);
+
+        _vfs.reset(createVfsFromPlugin(newMode).release());
+
+        _definition.virtualFilesMode = newMode;
+        startVfs();
+        if (newMode != Vfs::Off)
+            _saveInFoldersWithPlaceholders = true;
+        saveToSettings();
+    }
+}
+
+bool Folder::newFilesAreVirtual() const
+{
+    if (!supportsVirtualFiles())
+        return false;
+    auto pinState = _vfs->pinState(QString());
+    return pinState && *pinState == PinState::OnlineOnly;
+}
+
+void Folder::setRootPinState(PinState state)
+{
+    _vfs->setPinState(QString(), state);
+
+    // We don't actually need discovery, but it's important to recurse
+    // into all folders, so the changes can be applied.
+    slotNextSyncFullLocalDiscovery();
+}
+
+bool Folder::supportsSelectiveSync() const
+{
+    return !supportsVirtualFiles() && !isVfsOnOffSwitchPending();
 }
 
 void Folder::saveToSettings() const
@@ -559,7 +684,7 @@ void Folder::saveToSettings() const
         }
     }
 
-    if (_definition.useVirtualFiles || _saveInFoldersWithPlaceholders) {
+    if (supportsVirtualFiles() || _saveInFoldersWithPlaceholders) {
         // If virtual files are enabled or even were enabled at some point,
         // save the folder to a group that will not be read by older (<2.5.0) clients.
         // The name is from when virtual files were called placeholders.
@@ -578,8 +703,12 @@ void Folder::saveToSettings() const
     settings->beginGroup(settingsGroup);
     // Note: Each of these groups might have a "version" tag, but that's
     //       currently unused.
+    settings->beginGroup(FolderMan::escapeAlias(_definition.alias));
     FolderDefinition::save(*settings, _definition);
 
+    // Technically redundant, just for older clients
+    settings->setValue(QLatin1String("usePlaceholders"), newFilesAreVirtual());
+
     settings->sync();
     qCInfo(lcFolder) << "Saved folder" << _definition.alias << "to settings, status" << settings->status();
 }
@@ -618,20 +747,18 @@ void Folder::slotTerminateSync()
     }
 }
 
-// This removes the csync File database
-// This is needed to provide a clean startup again in case another
-// local folder is synced to the same ownCloud.
-void Folder::wipe()
+void Folder::wipeForRemoval()
 {
-    QString stateDbFile = _engine->journal()->databaseFilePath();
-
     // Delete files that have been partially downloaded.
     slotDiscardDownloadProgress();
 
-    //Unregister the socket API so it does not keep the ._sync_journal file open
+    // Unregister the socket API so it does not keep the .sync_journal file open
     FolderMan::instance()->socketApi()->slotUnregisterPath(alias());
     _journal.close(); // close the sync journal
 
+    // Remove db and temporaries
+    QString stateDbFile = _engine->journal()->databaseFilePath();
+
     QFile file(stateDbFile);
     if (file.exists()) {
         if (!file.remove()) {
@@ -649,8 +776,9 @@ void Folder::wipe()
     QFile::remove(stateDbFile + "-wal");
     QFile::remove(stateDbFile + "-journal");
 
-    if (canSync())
-        FolderMan::instance()->socketApi()->slotRegisterPath(alias());
+    _vfs->stop();
+    _vfs->unregisterFolder();
+    _vfs.reset(nullptr); // warning: folder now in an invalid state
 }
 
 bool Folder::reloadExcludes()
@@ -666,13 +794,12 @@ void Folder::startSync(const QStringList &pathList)
         qCCritical(lcFolder) << "ERROR csync is still running and new sync requested.";
         return;
     }
-    _csyncUnavail = false;
 
     _timeSinceLastSyncStart.start();
     _syncResult.setStatus(SyncResult::SyncPrepare);
     emit syncStateChange();
 
-    qCInfo(lcFolder) << "*** Start syncing " << remoteUrl().toString() << " - client version"
+    qCInfo(lcFolder) << "*** Start syncing " << remoteUrl().toString() << " -" << APPLICATION_NAME << "client version"
                      << qPrintable(Theme::instance()->version());
 
     _fileLog->start(path());
@@ -728,41 +855,19 @@ void Folder::setSyncOptions()
     opt._newBigFolderSizeLimit = newFolderLimit.first ? newFolderLimit.second * 1000LL * 1000LL : -1; // convert from MB to B
     opt._confirmExternalStorage = cfgFile.confirmExternalStorage();
     opt._moveFilesToTrash = cfgFile.moveToTrash();
-    opt._newFilesAreVirtual = _definition.useVirtualFiles;
-    opt._virtualFileSuffix = QStringLiteral(APPLICATION_DOTVIRTUALFILE_SUFFIX);
+    opt._vfs = _vfs;
+    opt._parallelNetworkJobs = _accountState->account()->isHttp2Supported() ? 20 : 6;
 
-    QByteArray chunkSizeEnv = qgetenv("OWNCLOUD_CHUNK_SIZE");
-    if (!chunkSizeEnv.isEmpty()) {
-        opt._initialChunkSize = chunkSizeEnv.toUInt();
-    } else {
-        opt._initialChunkSize = cfgFile.chunkSize();
-    }
-    QByteArray minChunkSizeEnv = qgetenv("OWNCLOUD_MIN_CHUNK_SIZE");
-    if (!minChunkSizeEnv.isEmpty()) {
-        opt._minChunkSize = minChunkSizeEnv.toUInt();
-    } else {
-        opt._minChunkSize = cfgFile.minChunkSize();
-    }
-    QByteArray maxChunkSizeEnv = qgetenv("OWNCLOUD_MAX_CHUNK_SIZE");
-    if (!maxChunkSizeEnv.isEmpty()) {
-        opt._maxChunkSize = maxChunkSizeEnv.toUInt();
-    } else {
-        opt._maxChunkSize = cfgFile.maxChunkSize();
-    }
+    opt._initialChunkSize = cfgFile.chunkSize();
+    opt._minChunkSize = cfgFile.minChunkSize();
+    opt._maxChunkSize = cfgFile.maxChunkSize();
+    opt._targetChunkUploadDuration = cfgFile.targetChunkUploadDuration();
 
-    // Previously min/max chunk size values didn't exist, so users might
-    // have setups where the chunk size exceeds the new min/max default
-    // values. To cope with this, adjust min/max to always include the
-    // initial chunk size value.
-    opt._minChunkSize = qMin(opt._minChunkSize, opt._initialChunkSize);
-    opt._maxChunkSize = qMax(opt._maxChunkSize, opt._initialChunkSize);
+    opt._deltaSyncEnabled = cfgFile.deltaSyncEnabled();
+    opt._deltaSyncMinFileSize = cfgFile.deltaSyncMinFileSize();
 
-    QByteArray targetChunkUploadDurationEnv = qgetenv("OWNCLOUD_TARGET_CHUNK_UPLOAD_DURATION");
-    if (!targetChunkUploadDurationEnv.isEmpty()) {
-        opt._targetChunkUploadDuration = std::chrono::milliseconds(targetChunkUploadDurationEnv.toUInt());
-    } else {
-        opt._targetChunkUploadDuration = cfgFile.targetChunkUploadDuration();
-    }
+    opt.fillFromEnvironmentVariables();
+    opt.verifyChunkSizes();
 
     _engine->setSyncOptions(opt);
 }
@@ -802,11 +907,6 @@ void Folder::slotSyncStarted()
     emit syncStateChange();
 }
 
-void Folder::slotCsyncUnavailable()
-{
-    _csyncUnavail = true;
-}
-
 void Folder::slotSyncFinished(bool success)
 {
     qCInfo(lcFolder) << "Client version" << qPrintable(Theme::instance()->version())
@@ -827,9 +927,6 @@ void Folder::slotSyncFinished(bool success)
 
     if (syncError) {
         _syncResult.setStatus(SyncResult::Error);
-    } else if (_csyncUnavail) {
-        _syncResult.setStatus(SyncResult::Error);
-        qCWarning(lcFolder) << "csync not available.";
     } else if (_syncResult.foundFilesNotSynced()) {
         _syncResult.setStatus(SyncResult::Problem);
     } else if (_definition.paused) {
@@ -925,16 +1022,6 @@ void Folder::slotItemCompleted(const SyncFileItemPtr &item)
         return;
     }
 
-    // add new directories or remove gone away dirs to the watcher
-    if (item->isDirectory() && item->_instruction == CSYNC_INSTRUCTION_NEW) {
-        if (_folderWatcher)
-            _folderWatcher->addPath(path() + item->_file);
-    }
-    if (item->isDirectory() && item->_instruction == CSYNC_INSTRUCTION_REMOVE) {
-        if (_folderWatcher)
-            _folderWatcher->removePath(path() + item->_file);
-    }
-
     _syncResult.processCompletedItem(item);
 
     _fileLog->logItem(*item);
@@ -992,6 +1079,11 @@ void Folder::slotNextSyncFullLocalDiscovery()
     _timeSinceLastFullLocalDiscovery.invalidate();
 }
 
+void Folder::schedulePathForLocalDiscovery(const QString &relativePath)
+{
+    _localDiscoveryTracker->addTouchedPath(relativePath.toUtf8());
+}
+
 void Folder::slotFolderConflicts(const QString &folder, const QStringList &conflictPaths)
 {
     if (folder != _definition.alias)
@@ -1049,6 +1141,30 @@ void Folder::slotWatcherUnreliable(const QString &message)
     Logger::instance()->postGuiLog(Theme::instance()->appNameGUI(), fullMessage);
 }
 
+void Folder::slotHydrationStarts()
+{
+    // Abort any running full sync run and reschedule
+    if (_engine->isSyncRunning()) {
+        slotTerminateSync();
+        scheduleThisFolderSoon();
+        // TODO: This sets the sync state to AbortRequested on done, we don't want that
+    }
+
+    // Let everyone know we're syncing
+    _syncResult.reset();
+    _syncResult.setStatus(SyncResult::SyncRunning);
+    emit syncStarted();
+    emit syncStateChange();
+}
+
+void Folder::slotHydrationDone()
+{
+    // emit signal to update ui and reschedule normal syncs if necessary
+    _syncResult.setStatus(SyncResult::Success);
+    emit syncFinished(_syncResult);
+    emit syncStateChange();
+}
+
 void Folder::scheduleThisFolderSoon()
 {
     if (!_scheduleSelfTimer.isActive()) {
@@ -1070,12 +1186,18 @@ void Folder::registerFolderWatcher()
 
     _folderWatcher.reset(new FolderWatcher(this));
     connect(_folderWatcher.data(), &FolderWatcher::pathChanged,
-        this, &Folder::slotWatchedPathChanged);
+        this, [this](const QString &path) { slotWatchedPathChanged(path, Folder::ChangeReason::Other); });
     connect(_folderWatcher.data(), &FolderWatcher::lostChanges,
         this, &Folder::slotNextSyncFullLocalDiscovery);
     connect(_folderWatcher.data(), &FolderWatcher::becameUnreliable,
         this, &Folder::slotWatcherUnreliable);
     _folderWatcher->init(path());
+    _folderWatcher->startNotificatonTest(path() + QLatin1String(".owncloudsync.log"));
+}
+
+bool Folder::supportsVirtualFiles() const
+{
+    return _definition.virtualFilesMode != Vfs::Off && !isVfsOnOffSwitchPending();
 }
 
 void Folder::slotAboutToRemoveAllFiles(SyncFileItem::Direction dir, bool *cancel)
@@ -1111,51 +1233,33 @@ void Folder::slotAboutToRemoveAllFiles(SyncFileItem::Direction dir, bool *cancel
     }
 }
 
-void Folder::slotAboutToRestoreBackup(bool *restore)
-{
-    QString msg =
-        tr("This sync would reset the files to an earlier time in the sync folder '%1'.\n"
-           "This might be because a backup was restored on the server.\n"
-           "Continuing the sync as normal will cause all your files to be overwritten by an older "
-           "file in an earlier state. "
-           "Do you want to keep your local most recent files as conflict files?");
-    QMessageBox msgBox(QMessageBox::Warning, tr("Backup detected"),
-        msg.arg(shortGuiLocalPath()));
-    msgBox.setWindowFlags(msgBox.windowFlags() | Qt::WindowStaysOnTopHint);
-    msgBox.addButton(tr("Normal Synchronisation"), QMessageBox::DestructiveRole);
-    QPushButton *keepBtn = msgBox.addButton(tr("Keep Local Files as Conflict"), QMessageBox::AcceptRole);
-
-    if (msgBox.exec() == -1) {
-        *restore = true;
-        return;
-    }
-    *restore = msgBox.clickedButton() == keepBtn;
-}
-
-
 void FolderDefinition::save(QSettings &settings, const FolderDefinition &folder)
 {
-    settings.beginGroup(FolderMan::escapeAlias(folder.alias));
     settings.setValue(QLatin1String("localPath"), folder.localPath);
     settings.setValue(QLatin1String("journalPath"), folder.journalPath);
     settings.setValue(QLatin1String("targetPath"), folder.targetPath);
     settings.setValue(QLatin1String("paused"), folder.paused);
     settings.setValue(QLatin1String("ignoreHiddenFiles"), folder.ignoreHiddenFiles);
-    settings.setValue(QLatin1String("usePlaceholders"), folder.useVirtualFiles);
-    settings.setValue(QLatin1String(versionC), 1);
+
+    settings.setValue(QStringLiteral("virtualFilesMode"), Vfs::modeToString(folder.virtualFilesMode));
+
+    // Ensure new vfs modes won't be attempted by older clients
+    if (folder.virtualFilesMode == Vfs::WindowsCfApi) {
+        settings.setValue(QLatin1String(versionC), 3);
+    } else {
+        settings.setValue(QLatin1String(versionC), 2);
+    }
 
     // Happens only on Windows when the explorer integration is enabled.
     if (!folder.navigationPaneClsid.isNull())
         settings.setValue(QLatin1String("navigationPaneClsid"), folder.navigationPaneClsid);
     else
         settings.remove(QLatin1String("navigationPaneClsid"));
-    settings.endGroup();
 }
 
 bool FolderDefinition::load(QSettings &settings, const QString &alias,
     FolderDefinition *folder)
 {
-    settings.beginGroup(alias);
     folder->alias = FolderMan::unescapeAlias(alias);
     folder->localPath = settings.value(QLatin1String("localPath")).toString();
     folder->journalPath = settings.value(QLatin1String("journalPath")).toString();
@@ -1163,11 +1267,24 @@ bool FolderDefinition::load(QSettings &settings, const QString &alias,
     folder->paused = settings.value(QLatin1String("paused")).toBool();
     folder->ignoreHiddenFiles = settings.value(QLatin1String("ignoreHiddenFiles"), QVariant(true)).toBool();
     folder->navigationPaneClsid = settings.value(QLatin1String("navigationPaneClsid")).toUuid();
-    folder->useVirtualFiles = settings.value(QLatin1String("usePlaceholders")).toBool();
-    settings.endGroup();
+
+    folder->virtualFilesMode = Vfs::Off;
+    QString vfsModeString = settings.value(QStringLiteral("virtualFilesMode")).toString();
+    if (!vfsModeString.isEmpty()) {
+        if (auto mode = Vfs::modeFromString(vfsModeString)) {
+            folder->virtualFilesMode = *mode;
+        } else {
+            qCWarning(lcFolder) << "Unknown virtualFilesMode:" << vfsModeString << "assuming 'off'";
+        }
+    } else {
+        if (settings.value(QLatin1String("usePlaceholders")).toBool()) {
+            folder->virtualFilesMode = Vfs::WithSuffix;
+            folder->upgradeVfsMode = true; // maybe winvfs is available?
+        }
+    }
 
     // Old settings can contain paths with native separators. In the rest of the
-    // code we assum /, so clean it up now.
+    // code we assume /, so clean it up now.
     folder->localPath = prepareLocalPath(folder->localPath);
 
     // Target paths also have a convention
