@@ -87,7 +87,8 @@ public:
      */
     static void backwardMigrationSettingsKeys(QStringList *deleteKeys, QStringList *ignoreKeys);
 
-    OCC::Folder::Map map();
+    OCC::Folder::Map map() const;
+    QList<Folder *> list() const;
 
     /** Adds a folder for an account, ensures the journal is gone and saves it in the settings.
       */
@@ -182,9 +183,22 @@ public:
 
     /**
      * Access to the currently syncing folder.
+     *
+     * Note: This is only the folder that's currently syncing *as-scheduled*. There
+     * may be externally-managed syncs such as from placeholder hydrations.
+     *
+     * See also isAnySyncRunning()
      */
     Folder *currentSyncFolder() const;
 
+    /**
+     * Returns true if any folder is currently syncing.
+     *
+     * This might be a FolderMan-scheduled sync, or a externally
+     * managed sync like a placeholder hydration.
+     */
+    bool isAnySyncRunning() const;
+
     /** Removes all folders */
     int unloadAndDeleteAllFolders();
 
@@ -206,13 +220,6 @@ public:
     void setDirtyProxy();
     void setDirtyNetworkLimits();
 
-    /**
-     * Terminates the current folder sync.
-     *
-     * It does not switch the folder to paused state.
-     */
-    void terminateSyncProcess();
-
 signals:
     /**
       * signal to indicate a folder has changed its sync state.
@@ -297,7 +304,7 @@ private:
      *  does not set an account on the new folder.
       */
     Folder *addFolderInternal(FolderDefinition folderDefinition,
-        AccountState *accountState);
+        AccountState *accountState, std::unique_ptr<Vfs> vfs);
 
     /* unloads a folder object, does not delete it */
     void unloadFolder(Folder *);
