@@ -18,6 +18,7 @@
 #include "issueswidget.h"
 #include "configfile.h"
 #include "syncresult.h"
+#include "syncengine.h"
 #include "logger.h"
 #include "theme.h"
 #include "folderman.h"
@@ -32,6 +33,7 @@
 #include "common/syncjournalfilerecord.h"
 #include "elidedlabel.h"
 
+
 #include "ui_issueswidget.h"
 
 #include <climits>
@@ -44,6 +46,11 @@ namespace OCC {
  */
 static const int maxIssueCount = 50000;
 
+static QPair<QString, QString> pathsWithIssuesKey(const ProtocolItem::ExtraData &data)
+{
+    return qMakePair(data.folderName, data.path);
+}
+
 IssuesWidget::IssuesWidget(QWidget *parent)
     : QWidget(parent)
     , _ui(new Ui::IssuesWidget)
@@ -60,6 +67,9 @@ IssuesWidget::IssuesWidget(QWidget *parent)
     connect(_ui->_treeWidget, &QTreeWidget::itemActivated, this, &IssuesWidget::slotOpenFile);
     connect(_ui->copyIssuesButton, &QAbstractButton::clicked, this, &IssuesWidget::copyToClipboard);
 
+    _ui->_treeWidget->setContextMenuPolicy(Qt::CustomContextMenu);
+    connect(_ui->_treeWidget, &QTreeWidget::customContextMenuRequested, this, &IssuesWidget::slotItemContextMenu);
+
     connect(_ui->showIgnores, &QAbstractButton::toggled, this, &IssuesWidget::slotRefreshIssues);
     connect(_ui->showWarnings, &QAbstractButton::toggled, this, &IssuesWidget::slotRefreshIssues);
     connect(_ui->filterAccount, static_cast<void (QComboBox::*)(int)>(&QComboBox::currentIndexChanged), this, &IssuesWidget::slotRefreshIssues);
@@ -91,7 +101,7 @@ IssuesWidget::IssuesWidget(QWidget *parent)
     _ui->_treeWidget->setHeaderLabels(header);
     int timestampColumnWidth =
         ActivityItemDelegate::rowHeight() // icon
-        + _ui->_treeWidget->fontMetrics().width(ProtocolWidget::timeString(QDateTime::currentDateTime()))
+        + _ui->_treeWidget->fontMetrics().boundingRect(ProtocolItem::timeString(QDateTime::currentDateTime())).width()
         + timestampColumnExtra;
     _ui->_treeWidget->setColumnWidth(0, timestampColumnWidth);
     _ui->_treeWidget->setColumnWidth(1, 180);
@@ -110,6 +120,11 @@ IssuesWidget::IssuesWidget(QWidget *parent)
     _ui->_tooManyIssuesWarning->hide();
     connect(this, &IssuesWidget::issueCountUpdated, this,
         [this](int count) { _ui->_tooManyIssuesWarning->setVisible(count >= maxIssueCount); });
+
+    _ui->_conflictHelp->hide();
+    _ui->_conflictHelp->setText(
+        tr("There were conflicts. <a href=\"%1\">Check the documentation on how to resolve them.</a>")
+            .arg(Theme::instance()->conflictHelpUrl()));
 }
 
 IssuesWidget::~IssuesWidget()
@@ -141,7 +156,14 @@ void IssuesWidget::hideEvent(QHideEvent *ev)
     QWidget::hideEvent(ev);
 }
 
-void IssuesWidget::cleanItems(const QString &folder)
+static bool persistsUntilLocalDiscovery(QTreeWidgetItem *item)
+{
+    const auto data = ProtocolItem::extraData(item);
+    return data.status == SyncFileItem::Conflict
+        || (data.status == SyncFileItem::FileIgnored && data.direction == SyncFileItem::Up);
+}
+
+void IssuesWidget::cleanItems(const std::function<bool(QTreeWidgetItem *)> &shouldDelete)
 {
     _ui->_treeWidget->setSortingEnabled(false);
 
@@ -150,8 +172,8 @@ void IssuesWidget::cleanItems(const QString &folder)
     int itemCnt = _ui->_treeWidget->topLevelItemCount();
     for (int cnt = itemCnt - 1; cnt >= 0; cnt--) {
         QTreeWidgetItem *item = _ui->_treeWidget->topLevelItem(cnt);
-        QString itemFolder = item->data(2, Qt::UserRole).toString();
-        if (itemFolder == folder) {
+        if (shouldDelete(item)) {
+            _pathsWithIssues.remove(pathsWithIssuesKey(ProtocolItem::extraData(item)));
             delete item;
         }
     }
@@ -168,8 +190,10 @@ void IssuesWidget::addItem(QTreeWidgetItem *item)
         return;
 
     int count = _ui->_treeWidget->topLevelItemCount();
-    if (count >= maxIssueCount)
+    if (count >= maxIssueCount) {
+        delete item;
         return;
+    }
 
     _ui->_treeWidget->setSortingEnabled(false);
     _reenableSorting.start();
@@ -186,18 +210,29 @@ void IssuesWidget::addItem(QTreeWidgetItem *item)
         }
     }
 
+    // Wipe any existing message for the same folder and path
+    auto newData = ProtocolItem::extraData(item);
+    if (_pathsWithIssues.contains(pathsWithIssuesKey(newData))) {
+        for (int i = 0; i < count; ++i) {
+            auto otherItem = _ui->_treeWidget->topLevelItem(i);
+            auto otherData = ProtocolItem::extraData(otherItem);
+            if (otherData.path == newData.path && otherData.folderName == newData.folderName) {
+                delete otherItem;
+                break;
+            }
+        }
+    }
+
     _ui->_treeWidget->insertTopLevelItem(insertLoc, item);
+    _pathsWithIssues.insert(pathsWithIssuesKey(newData));
     item->setHidden(!shouldBeVisible(item, currentAccountFilter(), currentFolderFilter()));
     emit issueCountUpdated(_ui->_treeWidget->topLevelItemCount());
 }
 
 void IssuesWidget::slotOpenFile(QTreeWidgetItem *item, int)
 {
-    QString folderName = item->data(2, Qt::UserRole).toString();
     QString fileName = item->text(1);
-
-    Folder *folder = FolderMan::instance()->folder(folderName);
-    if (folder) {
+    if (Folder *folder = ProtocolItem::folder(item)) {
         // folder->path() always comes back with trailing path
         QString fullPath = folder->path() + fileName;
         if (QFile(fullPath).exists()) {
@@ -208,9 +243,49 @@ void IssuesWidget::slotOpenFile(QTreeWidgetItem *item, int)
 
 void IssuesWidget::slotProgressInfo(const QString &folder, const ProgressInfo &progress)
 {
-    if (progress.status() == ProgressInfo::Starting) {
-        // The sync is restarting, clean the old items
-        cleanItems(folder);
+    if (progress.status() == ProgressInfo::Reconcile) {
+        // Wipe all non-persistent entries - as well as the persistent ones
+        // in cases where a local discovery was done.
+        auto f = FolderMan::instance()->folder(folder);
+        if (!f)
+            return;
+        const auto &engine = f->syncEngine();
+        const auto style = engine.lastLocalDiscoveryStyle();
+        cleanItems([&](QTreeWidgetItem *item) {
+            if (ProtocolItem::extraData(item).folderName != folder)
+                return false;
+            if (style == LocalDiscoveryStyle::FilesystemOnly)
+                return true;
+            if (!persistsUntilLocalDiscovery(item))
+                return true;
+
+            // Definitely wipe the entry if the file no longer exists
+            if (!QFileInfo(f->path() + ProtocolItem::extraData(item).path).exists())
+                return true;
+
+            auto path = QFileInfo(ProtocolItem::extraData(item).path).dir().path();
+            if (path == ".")
+                path.clear();
+
+            return engine.shouldDiscoverLocally(path);
+        });
+    }
+    if (progress.status() == ProgressInfo::Done) {
+        // We keep track very well of pending conflicts.
+        // Inform other components about them.
+        QStringList conflicts;
+        auto tree = _ui->_treeWidget;
+        for (int i = 0; i < tree->topLevelItemCount(); ++i) {
+            auto item = tree->topLevelItem(i);
+            auto data = ProtocolItem::extraData(item);
+            if (data.folderName == folder
+                && data.status == SyncFileItem::Conflict) {
+                conflicts.append(data.path);
+            }
+        }
+        emit ProgressDispatcher::instance()->folderConflicts(folder, conflicts);
+
+        _ui->_conflictHelp->setHidden(Theme::instance()->conflictHelpUrl().isEmpty() || conflicts.isEmpty());
     }
 }
 
@@ -218,7 +293,7 @@ void IssuesWidget::slotItemCompleted(const QString &folder, const SyncFileItemPt
 {
     if (!item->showInIssuesTab())
         return;
-    QTreeWidgetItem *line = ProtocolWidget::createCompletedTreewidgetItem(folder, *item);
+    QTreeWidgetItem *line = ProtocolItem::create(folder, *item);
     if (!line)
         return;
     addItem(line);
@@ -253,6 +328,15 @@ void IssuesWidget::slotAccountRemoved(AccountState *account)
     updateAccountChoiceVisibility();
 }
 
+void IssuesWidget::slotItemContextMenu(const QPoint &pos)
+{
+    auto item = _ui->_treeWidget->itemAt(pos);
+    if (!item)
+        return;
+    auto globalPos = _ui->_treeWidget->viewport()->mapToGlobal(pos);
+    ProtocolItem::openContextMenu(globalPos, item, this);
+}
+
 void IssuesWidget::updateAccountChoiceVisibility()
 {
     bool visible = _ui->filterAccount->count() > 2;
@@ -275,13 +359,14 @@ bool IssuesWidget::shouldBeVisible(QTreeWidgetItem *item, AccountState *filterAc
     const QString &filterFolderAlias) const
 {
     bool visible = true;
-    auto status = item->data(3, Qt::UserRole);
+    auto data = ProtocolItem::extraData(item);
+    auto status = data.status;
     visible &= (_ui->showIgnores->isChecked() || status != SyncFileItem::FileIgnored);
     visible &= (_ui->showWarnings->isChecked()
         || (status != SyncFileItem::SoftError
                && status != SyncFileItem::Restoration));
 
-    auto folderalias = item->data(2, Qt::UserRole).toString();
+    const auto &folderalias = data.folderName;
     if (filterAccount) {
         auto folder = FolderMan::instance()->folder(folderalias);
         visible &= folder && folder->accountState() == filterAccount;
@@ -393,8 +478,8 @@ void IssuesWidget::addError(const QString &folderAlias, const QString &message,
 
     QStringList columns;
     QDateTime timestamp = QDateTime::currentDateTime();
-    const QString timeStr = ProtocolWidget::timeString(timestamp);
-    const QString longTimeStr = ProtocolWidget::timeString(timestamp, QLocale::LongFormat);
+    const QString timeStr = ProtocolItem::timeString(timestamp);
+    const QString longTimeStr = ProtocolItem::timeString(timestamp, QLocale::LongFormat);
 
     columns << timeStr;
     columns << ""; // no "File" entry
@@ -403,14 +488,16 @@ void IssuesWidget::addError(const QString &folderAlias, const QString &message,
 
     QIcon icon = Theme::instance()->syncStateIcon(SyncResult::Error);
 
-    QTreeWidgetItem *twitem = new SortedTreeWidgetItem(columns);
+    QTreeWidgetItem *twitem = new ProtocolItem(columns);
     twitem->setData(0, Qt::SizeHintRole, QSize(0, ActivityItemDelegate::rowHeight()));
-    twitem->setData(0, Qt::UserRole, timestamp);
     twitem->setIcon(0, icon);
     twitem->setToolTip(0, longTimeStr);
-    twitem->setData(2, Qt::UserRole, folderAlias);
     twitem->setToolTip(3, message);
-    twitem->setData(3, Qt::UserRole, SyncFileItem::NormalError);
+    ProtocolItem::ExtraData data;
+    data.timestamp = timestamp;
+    data.folderName = folderAlias;
+    data.status = SyncFileItem::NormalError;
+    ProtocolItem::setExtraData(twitem, data);
 
     addItem(twitem);
     addErrorWidget(twitem, message, category);
@@ -430,7 +517,7 @@ void IssuesWidget::addErrorWidget(QTreeWidgetItem *item, const QString &message,
 
         auto button = new QPushButton("Retry all uploads", widget);
         button->setSizePolicy(QSizePolicy::Maximum, QSizePolicy::Expanding);
-        auto folderAlias = item->data(2, Qt::UserRole).toString();
+        auto folderAlias = ProtocolItem::extraData(item).folderName;
         connect(button, &QPushButton::clicked,
             this, [this, folderAlias]() { retryInsufficentRemoteStorageErrors(folderAlias); });
         layout->addWidget(button);
