@@ -31,7 +31,6 @@ namespace OCC {
 class Folder;
 
 class SettingsDialog;
-class SettingsDialogMac;
 class ShareDialog;
 class Application;
 class LogBrowser;
@@ -61,6 +60,8 @@ public:
     /// Whether the tray menu is visible
     bool contextMenuVisible() const;
 
+    void hideAndShowTray();
+
 signals:
     void setupProxy();
 
@@ -116,11 +117,7 @@ private:
     void addAccountContextMenu(AccountStatePtr accountState, QMenu *menu, bool separateMenu);
 
     QPointer<Systray> _tray;
-#if defined(Q_OS_MAC)
-    QPointer<SettingsDialogMac> _settingsDialog;
-#else
     QPointer<SettingsDialog> _settingsDialog;
-#endif
     QPointer<LogBrowser> _logBrowser;
     // tray's menu
     QScopedPointer<QMenu> _contextMenu;
@@ -152,6 +149,9 @@ private:
     QAction *_actionAbout;
     QAction *_actionQuit;
     QAction *_actionCrash;
+    QAction *_actionCrashEnforce;
+    QAction *_actionCrashFatal;
+
 
     QList<QAction *> _recentItemsActions;
     Application *_app;
