@@ -44,8 +44,7 @@ ShareLinkWidget::ShareLinkWidget(AccountPtr account,
     , _account(account)
     , _sharePath(sharePath)
     , _localPath(localPath)
-    , _manager(0)
-    , _passwordRequired(false)
+    , _manager(nullptr)
     , _expiryRequired(false)
     , _namesSupported(true)
 {
@@ -68,7 +67,7 @@ ShareLinkWidget::ShareLinkWidget(AccountPtr account,
     _pi_password = new QProgressIndicator();
     _pi_date = new QProgressIndicator();
     _pi_editing = new QProgressIndicator();
-    _ui->horizontalLayout_create->addWidget(_pi_create);
+    _ui->horizontalLayout_create->insertWidget(2, _pi_create);
     _ui->horizontalLayout_password->addWidget(_pi_password);
     _ui->layout_editing->addWidget(_pi_editing, 0, 2);
     _ui->horizontalLayout_expire->insertWidget(_ui->horizontalLayout_expire->count() - 1, _pi_date);
@@ -108,6 +107,7 @@ ShareLinkWidget::ShareLinkWidget(AccountPtr account,
         _ui->createShareButton->setText(tr("Create public link share"));
         _ui->nameLineEdit->hide();
         _ui->nameLineEdit->clear(); // so we don't send a name
+        _ui->nameLabel->hide();
     }
 
     _ui->shareProperties->setEnabled(false);
@@ -135,26 +135,25 @@ ShareLinkWidget::ShareLinkWidget(AccountPtr account,
 
     // Parse capabilities
 
-    // If password is enforced then don't allow users to disable it
-    if (_account->capabilities().sharePublicLinkEnforcePassword()) {
-        _ui->checkBox_password->setEnabled(false);
-        _passwordRequired = true;
+    if (!_account->capabilities().sharePublicLinkAllowUpload()) {
+        _ui->radio_readWrite->setEnabled(false);
+        _ui->radio_uploadOnly->setEnabled(false);
     }
 
     // If expiredate is enforced do not allow disable and set max days
     if (_account->capabilities().sharePublicLinkEnforceExpireDate()) {
         _ui->checkBox_expire->setEnabled(false);
-        _ui->calendar->setMaximumDate(QDate::currentDate().addDays(
-            _account->capabilities().sharePublicLinkExpireDateDays()));
+        _ui->calendar->setMaximumDate(capabilityDefaultExpireDate());
         _expiryRequired = true;
     }
 
-    // File can't have public upload set; we also hide it if the capability isn't there
-    _ui->widget_editing->setVisible(
-        !_isFile && _account->capabilities().sharePublicLinkAllowUpload());
-    _ui->radio_uploadOnly->setVisible(
-        _account->capabilities().sharePublicLinkSupportsUploadOnly());
-
+    // Hide permissions that are unavailable for files or disabled by capability.
+    bool rwVisible = !_isFile && _account->capabilities().sharePublicLinkAllowUpload();
+    bool uploadOnlyVisible = rwVisible && _account->capabilities().sharePublicLinkSupportsUploadOnly();
+    _ui->radio_readWrite->setVisible(rwVisible);
+    _ui->label_readWrite->setVisible(rwVisible);
+    _ui->radio_uploadOnly->setVisible(uploadOnlyVisible);
+    _ui->label_uploadOnly->setVisible(uploadOnlyVisible);
 
     // Prepare sharing menu
 
@@ -175,9 +174,22 @@ ShareLinkWidget::ShareLinkWidget(AccountPtr account,
         _manager = new ShareManager(_account, this);
         connect(_manager, &ShareManager::sharesFetched, this, &ShareLinkWidget::slotSharesFetched);
         connect(_manager, &ShareManager::linkShareCreated, this, &ShareLinkWidget::slotCreateShareFetched);
-        connect(_manager, &ShareManager::linkShareRequiresPassword, this, &ShareLinkWidget::slotCreateShareRequiresPassword);
+        connect(_manager, &ShareManager::linkShareCreationForbidden, this, &ShareLinkWidget::slotCreateShareForbidden);
         connect(_manager, &ShareManager::serverError, this, &ShareLinkWidget::slotServerError);
     }
+
+    auto retainSizeWhenHidden = [](QWidget *w) {
+        auto sp = w->sizePolicy();
+        sp.setRetainSizeWhenHidden(true);
+        w->setSizePolicy(sp);
+    };
+    retainSizeWhenHidden(_ui->pushButton_setPassword);
+    retainSizeWhenHidden(_ui->create);
+
+    // If this starts out empty the first call to slotShareSelectionChanged()
+    // will not properly initialize the ui state if "Create new..." is the
+    // only option. So pre-fill with an invalid share id.
+    _selectedShareId = QStringLiteral("!&no-share-selected");
 }
 
 ShareLinkWidget::~ShareLinkWidget()
@@ -216,7 +228,7 @@ void ShareLinkWidget::slotSharesFetched(const QList<QSharedPointer<Share>> &shar
     connect(table, &QTableWidget::itemSelectionChanged, this, &ShareLinkWidget::slotShareSelectionChanged);
 
     auto deleteIcon = QIcon::fromTheme(QLatin1String("user-trash"),
-        QIcon(QLatin1String(":/client/resources/delete.png")));
+        Utility::createColorAwareIcon(QLatin1String(":/client/resources/delete.png")));
 
     foreach (auto share, shares) {
         if (share->getShareType() != Share::TypeLink) {
@@ -264,6 +276,23 @@ void ShareLinkWidget::slotSharesFetched(const QList<QSharedPointer<Share>> &shar
         }
     }
 
+    // Add create-new entry
+    if (_namesSupported || table->rowCount() == 0) {
+        auto row = table->rowCount();
+        table->insertRow(row);
+        auto createItem = new QTableWidgetItem;
+        createItem->setFlags(createItem->flags() & ~Qt::ItemIsEditable);
+        createItem->setText(tr("Create new..."));
+        auto font = createItem->font();
+        font.setItalic(true);
+        createItem->setFont(font);
+        table->setItem(row, 0, createItem);
+        auto dummyItem = new QTableWidgetItem;
+        dummyItem->setFlags(dummyItem->flags() & ~Qt::ItemIsEditable);
+        table->setItem(row, 1, dummyItem->clone());
+        table->setItem(row, 2, dummyItem);
+    }
+
     if (!selectedShare()) {
         if (table->rowCount() != 0) {
             // Select the first share by default
@@ -274,10 +303,6 @@ void ShareLinkWidget::slotSharesFetched(const QList<QSharedPointer<Share>> &shar
             slotShareSelectionChanged();
         }
     }
-
-    if (!_namesSupported) {
-        _ui->createShareButton->setEnabled(table->rowCount() == 0);
-    }
 }
 
 void ShareLinkWidget::slotShareSelectionChanged()
@@ -291,33 +316,54 @@ void ShareLinkWidget::slotShareSelectionChanged()
     _ui->errorLabel->hide();
 
     auto share = selectedShare();
-    if (!share) {
+    bool selectionUnchanged = false;
+    bool createNew = !share;
+    if (share) {
+        selectionUnchanged = _selectedShareId == share->getId();
+        _selectedShareId = share->getId();
+    } else {
+        selectionUnchanged = _selectedShareId.isEmpty();
         _selectedShareId.clear();
-        _ui->shareProperties->setEnabled(false);
-        _ui->radio_readOnly->setChecked(false);
-        _ui->radio_readWrite->setChecked(false);
-        _ui->radio_uploadOnly->setChecked(false);
-        _ui->checkBox_expire->setChecked(false);
-        _ui->checkBox_password->setChecked(false);
-        return;
     }
-    bool selectionUnchanged = _selectedShareId == share->getId();
-    _selectedShareId = share->getId();
 
     _ui->shareProperties->setEnabled(true);
 
-    _ui->checkBox_password->setEnabled(!_passwordRequired);
-    _ui->checkBox_expire->setEnabled(!_expiryRequired);
-    _ui->widget_editing->setEnabled(true);
-    if (!_account->capabilities().sharePublicLinkAllowUpload()) {
-        _ui->radio_readWrite->setEnabled(false);
-        _ui->radio_uploadOnly->setEnabled(false);
+    // Public upload state (files can only be read-only, box is hidden for them)
+    _ui->widget_editing->setEnabled(!_isFile);
+    if (!selectionUnchanged) {
+        if (share && share->getPublicUpload()) {
+            if (share->getShowFileListing()) {
+                _ui->radio_readWrite->setChecked(true);
+            } else {
+                _ui->radio_uploadOnly->setChecked(true);
+            }
+        } else {
+            _ui->radio_readOnly->setChecked(true);
+        }
     }
 
+    const auto capabilities = _account->capabilities();
+    const bool passwordRequired =
+        (_ui->radio_readOnly->isChecked() && capabilities.sharePublicLinkEnforcePasswordForReadOnly())
+        || (_ui->radio_readWrite->isChecked() && capabilities.sharePublicLinkEnforcePasswordForReadWrite())
+        || (_ui->radio_uploadOnly->isChecked() && capabilities.sharePublicLinkEnforcePasswordForUploadOnly());
+
     // Password state
+    _ui->pushButton_setPassword->setVisible(!createNew);
     _ui->checkBox_password->setText(tr("P&assword protect"));
-    if (!selectionUnchanged) {
-        if (share->isPasswordSet()) {
+    if (createNew) {
+        if (passwordRequired) {
+            _ui->checkBox_password->setChecked(true);
+            _ui->lineEdit_password->setPlaceholderText(tr("Please Set Password"));
+            _ui->lineEdit_password->setEnabled(true);
+        } else if (!selectionUnchanged || _ui->lineEdit_password->text().isEmpty()) {
+            // force to off if no password was entered yet
+            _ui->checkBox_password->setChecked(false);
+            _ui->lineEdit_password->setPlaceholderText(QString());
+            _ui->lineEdit_password->setEnabled(false);
+        }
+    } else if (!selectionUnchanged) {
+        if (share && share->isPasswordSet()) {
             _ui->checkBox_password->setChecked(true);
             _ui->lineEdit_password->setPlaceholderText("********");
             _ui->lineEdit_password->setEnabled(true);
@@ -329,30 +375,34 @@ void ShareLinkWidget::slotShareSelectionChanged()
         _ui->lineEdit_password->setText(QString());
         _ui->pushButton_setPassword->setEnabled(false);
     }
+    // The following is done to work with old shares when the pw requirement
+    // is enabled on the server later: users can add pws to old shares.
+    _ui->checkBox_password->setEnabled(!_ui->checkBox_password->isChecked() || !passwordRequired);
 
     // Expiry state
     _ui->calendar->setMinimumDate(QDate::currentDate().addDays(1));
-    if (share->getExpireDate().isValid()) {
-        _ui->checkBox_expire->setChecked(true);
-        _ui->calendar->setDate(share->getExpireDate());
-        _ui->calendar->setEnabled(true);
-    } else {
-        _ui->checkBox_expire->setChecked(false);
-        _ui->calendar->setEnabled(false);
-    }
-
-    // Public upload state (box is hidden for files)
-    if (!_isFile) {
-        if (share->getPublicUpload()) {
-            if (share->getShowFileListing()) {
-                _ui->radio_readWrite->setChecked(true);
-            } else {
-                _ui->radio_uploadOnly->setChecked(true);
-            }
+    if (!selectionUnchanged) {
+        if (share && share->getExpireDate().isValid()) {
+            _ui->checkBox_expire->setChecked(true);
+            _ui->calendar->setDate(share->getExpireDate());
+            _ui->calendar->setEnabled(true);
+        } else if (createNew) {
+            const QDate defaultExpire = capabilityDefaultExpireDate();
+            if (defaultExpire.isValid())
+                _ui->calendar->setDate(defaultExpire);
+            const bool enabled = _expiryRequired || defaultExpire.isValid();
+            _ui->checkBox_expire->setChecked(enabled);
+            _ui->calendar->setEnabled(enabled);
         } else {
-            _ui->radio_readOnly->setChecked(true);
+            _ui->checkBox_expire->setChecked(false);
+            _ui->calendar->setEnabled(false);
         }
     }
+    // Allows checking expiry on old shares created before it was required.
+    _ui->checkBox_expire->setEnabled(!_ui->checkBox_expire->isChecked() || !_expiryRequired);
+
+    // Name and create button
+    _ui->create->setVisible(createNew);
 }
 
 void ShareLinkWidget::setExpireDate(const QDate &date)
@@ -380,22 +430,10 @@ void ShareLinkWidget::slotExpireDateChanged(const QDate &date)
 
 void ShareLinkWidget::slotPasswordReturnPressed()
 {
-    if (!_manager) {
-        return;
-    }
-    if (!selectedShare()) {
-        // If share creation requires a password, we'll be in this case
-        if (_ui->lineEdit_password->text().isEmpty()) {
-            _ui->lineEdit_password->setFocus();
-            return;
-        }
-
-        _pi_create->startAnimation();
-        _manager->createLinkShare(_sharePath, _ui->nameLineEdit->text(), _ui->lineEdit_password->text());
-    } else {
+    if (selectedShare()) {
         setPassword(_ui->lineEdit_password->text());
+        _ui->lineEdit_password->clearFocus();
     }
-    _ui->lineEdit_password->clearFocus();
 }
 
 void ShareLinkWidget::slotPasswordChanged(const QString &newText)
@@ -463,7 +501,12 @@ void ShareLinkWidget::slotShareNameEntered()
         return;
     }
     _pi_create->startAnimation();
-    _manager->createLinkShare(_sharePath, _ui->nameLineEdit->text(), QString());
+    _manager->createLinkShare(
+            _sharePath,
+            _ui->nameLineEdit->text(),
+            _ui->checkBox_password->isChecked() ? _ui->lineEdit_password->text() : QString(),
+            _ui->checkBox_expire->isChecked() ? _ui->calendar->date() : QDate(),
+            uiPermissionState());
 }
 
 void ShareLinkWidget::slotDeleteShareFetched()
@@ -481,28 +524,14 @@ void ShareLinkWidget::slotCreateShareFetched(const QSharedPointer<LinkShare> &sh
     getShares();
 }
 
-void ShareLinkWidget::slotCreateShareRequiresPassword(const QString &message)
+void ShareLinkWidget::slotCreateShareForbidden(const QString &message)
 {
-    // Deselect existing shares
-    _ui->linkShares->clearSelection();
-
-    // Prepare password entry
+    // Show the message, so users can adjust and try again
     _pi_create->stopAnimation();
-    _pi_password->stopAnimation();
-    _ui->shareProperties->setEnabled(true);
-    _ui->checkBox_password->setChecked(true);
-    _ui->checkBox_password->setEnabled(false);
-    _ui->checkBox_password->setText(tr("Public sh&aring requires a password"));
-    _ui->checkBox_expire->setEnabled(false);
-    _ui->widget_editing->setEnabled(false);
     if (!message.isEmpty()) {
         _ui->errorLabel->setText(message);
         _ui->errorLabel->show();
     }
-
-    _passwordRequired = true;
-
-    slotCheckBoxPasswordClicked();
 }
 
 void ShareLinkWidget::slotCheckBoxPasswordClicked()
@@ -524,10 +553,13 @@ void ShareLinkWidget::slotCheckBoxPasswordClicked()
 void ShareLinkWidget::slotCheckBoxExpireClicked()
 {
     if (_ui->checkBox_expire->checkState() == Qt::Checked) {
-        const QDate date = QDate::currentDate().addDays(1);
-        setExpireDate(date);
-        _ui->calendar->setDate(date);
-        _ui->calendar->setMinimumDate(date);
+        const QDate tomorrow = QDate::currentDate().addDays(1);
+        QDate defaultDate = capabilityDefaultExpireDate();
+        if (!defaultDate.isValid())
+            defaultDate = tomorrow;
+        setExpireDate(defaultDate);
+        _ui->calendar->setDate(defaultDate);
+        _ui->calendar->setMinimumDate(tomorrow);
         _ui->calendar->setEnabled(true);
     } else {
         setExpireDate(QDate());
@@ -617,6 +649,17 @@ void ShareLinkWidget::slotDeleteShareClicked()
     confirmAndDeleteShare(share);
 }
 
+SharePermissions ShareLinkWidget::uiPermissionState() const
+{
+    if (_ui->radio_readWrite->isChecked()) {
+        return SharePermissionRead | SharePermissionCreate
+            | SharePermissionUpdate | SharePermissionDelete;
+    } else if (_ui->radio_uploadOnly->isChecked()) {
+        return SharePermissionCreate;
+    }
+    return SharePermissionRead;
+}
+
 void ShareLinkWidget::slotPermissionsClicked()
 {
     if (auto current = selectedShare()) {
@@ -624,14 +667,10 @@ void ShareLinkWidget::slotPermissionsClicked()
         _pi_editing->startAnimation();
         _ui->errorLabel->hide();
 
-        SharePermissions perm = SharePermissionRead;
-        if (_ui->radio_readWrite->isChecked()) {
-            perm = SharePermissionRead | SharePermissionCreate
-                | SharePermissionUpdate | SharePermissionDelete;
-        } else if (_ui->radio_uploadOnly->isChecked()) {
-            perm = SharePermissionCreate;
-        }
-        current->setPermissions(perm);
+        current->setPermissions(uiPermissionState());
+    } else {
+        // Password may now be required, update ui.
+        slotShareSelectionChanged();
     }
 }
 
@@ -645,6 +684,14 @@ QSharedPointer<LinkShare> ShareLinkWidget::selectedShare() const
     return items.first()->data(Qt::UserRole).value<QSharedPointer<LinkShare>>();
 }
 
+QDate ShareLinkWidget::capabilityDefaultExpireDate() const
+{
+    if (!_account->capabilities().sharePublicLinkDefaultExpire())
+        return QDate();
+    return QDate::currentDate().addDays(
+            _account->capabilities().sharePublicLinkDefaultExpireDateDays());
+}
+
 void ShareLinkWidget::slotPermissionsSet()
 {
     if (sender() == selectedShare().data()) {
@@ -659,6 +706,10 @@ void ShareLinkWidget::slotServerError(int code, const QString &message)
     _pi_password->stopAnimation();
     _pi_editing->stopAnimation();
 
+    // Reset UI state
+    _selectedShareId.clear();
+    slotShareSelectionChanged();
+
     qCWarning(lcSharing) << "Error from server" << code << message;
     displayError(message);
 }
@@ -667,7 +718,6 @@ void ShareLinkWidget::slotPasswordSetError(int code, const QString &message)
 {
     slotServerError(code, message);
 
-    _ui->checkBox_password->setEnabled(!_passwordRequired);
     _ui->lineEdit_password->setEnabled(true);
     _ui->lineEdit_password->setFocus();
 }
