@@ -3,7 +3,8 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; version 2 of the License.
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
@@ -12,11 +13,12 @@
  */
 
 #include "creds/abstractcredentials.h"
-#include "creds/credentialsfactory.h"
+#include "creds/dummycredentials.h"
 #include "account.h"
 #include "wizard/owncloudwizard.h"
 
 #include "wizard/abstractcredswizardpage.h"
+#include <accountmanager.h>
 
 namespace OCC {
 
@@ -24,19 +26,10 @@ void AbstractCredentialsWizardPage::cleanupPage()
 {
     // Reset the credentials when the 'Back' button is used.
 
-    // Unfortunately this code is also run when the Wizard finishes
-    // prematurely with 'Skip Folder Configuration'. Therefore we need to
-    // avoid resetting credentials on active accounts.
-    AccountPtr account = static_cast<OwncloudWizard*>(wizard())->account();
-    if (account == AccountManager::instance()->account())
-        return;
-
+    AccountPtr account = static_cast<OwncloudWizard *>(wizard())->account();
     AbstractCredentials *creds = account->credentials();
-    if (creds) {
-        if (!creds->inherits("DummyCredentials")) {
-            account->setCredentials(CredentialsFactory::create("dummy"));
-        }
+    if (creds && !qobject_cast<DummyCredentials *>(creds)) {
+        account->setCredentials(new DummyCredentials);
     }
 }
-
 }
