@@ -3,7 +3,8 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; version 2 of the License.
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
@@ -16,8 +17,7 @@
 
 #include <QWizardPage>
 
-namespace OCC
-{
+namespace OCC {
 
 class AbstractCredentials;
 
@@ -29,7 +29,7 @@ class AbstractCredentialsWizardPage : public QWizardPage
 {
 public:
     void cleanupPage() Q_DECL_OVERRIDE;
-    virtual AbstractCredentials* getCredentials() const = 0;
+    virtual AbstractCredentials *getCredentials() const = 0;
 };
 
 } // namespace OCC
