@@ -4,7 +4,8 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; version 2 of the License.
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
@@ -19,10 +20,16 @@
 
 #include "ui_owncloudconnectionmethoddialog.h"
 
+namespace OCC {
+
 namespace Ui {
-class OwncloudConnectionMethodDialog;
+    class OwncloudConnectionMethodDialog;
 }
 
+/**
+ * @brief The OwncloudConnectionMethodDialog class
+ * @ingroup gui
+ */
 class OwncloudConnectionMethodDialog : public QDialog
 {
     Q_OBJECT
@@ -31,11 +38,12 @@ public:
     explicit OwncloudConnectionMethodDialog(QWidget *parent = 0);
     ~OwncloudConnectionMethodDialog();
     enum {
+        Closed = 0,
         No_TLS,
         Client_Side_TLS,
         Back
     };
-    
+
     // The URL that was tried
     void setUrl(const QUrl &);
 
@@ -47,5 +55,6 @@ public slots:
 private:
     Ui::OwncloudConnectionMethodDialog *ui;
 };
+}
 
 #endif // OWNCLOUDCONNECTIONMETHODDIALOG_H
