@@ -22,17 +22,17 @@
 #include "wizard/owncloudsetuppage.h"
 #include "wizard/owncloudhttpcredspage.h"
 #include "wizard/owncloudoauthcredspage.h"
-#ifndef NO_SHIBBOLETH
-#include "wizard/owncloudshibbolethcredspage.h"
-#endif
 #include "wizard/owncloudadvancedsetuppage.h"
 #include "wizard/owncloudwizardresultpage.h"
 
+#include "common/vfs.h"
+
 #include "QProgressIndicator.h"
 
 #include <QtCore>
 #include <QtGui>
 #include <QMessageBox>
+#include <owncloudgui.h>
 
 #include <stdlib.h>
 
@@ -46,9 +46,6 @@ OwncloudWizard::OwncloudWizard(QWidget *parent)
     , _setupPage(new OwncloudSetupPage(this))
     , _httpCredsPage(new OwncloudHttpCredsPage(this))
     , _browserCredsPage(new OwncloudOAuthCredsPage)
-#ifndef NO_SHIBBOLETH
-    , _shibbolethCredsPage(new OwncloudShibbolethCredsPage)
-#endif
     , _advancedSetupPage(new OwncloudAdvancedSetupPage)
     , _resultPage(new OwncloudWizardResultPage)
     , _credentialsPage(0)
@@ -58,9 +55,6 @@ OwncloudWizard::OwncloudWizard(QWidget *parent)
     setPage(WizardCommon::Page_ServerSetup, _setupPage);
     setPage(WizardCommon::Page_HttpCreds, _httpCredsPage);
     setPage(WizardCommon::Page_OAuthCreds, _browserCredsPage);
-#ifndef NO_SHIBBOLETH
-    setPage(WizardCommon::Page_ShibbolethCreds, _shibbolethCredsPage);
-#endif
     setPage(WizardCommon::Page_AdvancedSetup, _advancedSetupPage);
     setPage(WizardCommon::Page_Result, _resultPage);
 
@@ -74,9 +68,6 @@ OwncloudWizard::OwncloudWizard(QWidget *parent)
     connect(_setupPage, &OwncloudSetupPage::determineAuthType, this, &OwncloudWizard::determineAuthType);
     connect(_httpCredsPage, &OwncloudHttpCredsPage::connectToOCUrl, this, &OwncloudWizard::connectToOCUrl);
     connect(_browserCredsPage, &OwncloudOAuthCredsPage::connectToOCUrl, this, &OwncloudWizard::connectToOCUrl);
-#ifndef NO_SHIBBOLETH
-    connect(_shibbolethCredsPage, &OwncloudShibbolethCredsPage::connectToOCUrl, this, &OwncloudWizard::connectToOCUrl);
-#endif
     connect(_advancedSetupPage, &OwncloudAdvancedSetupPage::createLocalAndRemoteFolders,
         this, &OwncloudWizard::createLocalAndRemoteFolders);
 
@@ -158,12 +149,6 @@ void OwncloudWizard::successfulStep()
         _browserCredsPage->setConnected();
         break;
 
-#ifndef NO_SHIBBOLETH
-    case WizardCommon::Page_ShibbolethCreds:
-        _shibbolethCredsPage->setConnected();
-        break;
-#endif
-
     case WizardCommon::Page_AdvancedSetup:
         _advancedSetupPage->directoriesCreated();
         break;
@@ -174,18 +159,14 @@ void OwncloudWizard::successfulStep()
         break;
     }
 
+    ownCloudGui::raiseDialog(this);
     next();
 }
 
 void OwncloudWizard::setAuthType(DetermineAuthTypeJob::AuthType type)
 {
     _setupPage->setAuthType(type);
-#ifndef NO_SHIBBOLETH
-    if (type == DetermineAuthTypeJob::Shibboleth) {
-        _credentialsPage = _shibbolethCredsPage;
-    } else
-#endif
-        if (type == DetermineAuthTypeJob::OAuth) {
+    if (type == DetermineAuthTypeJob::OAuth) {
         _credentialsPage = _browserCredsPage;
     } else { // try Basic auth even for "Unknown"
         _credentialsPage = _httpCredsPage;
@@ -254,22 +235,54 @@ AbstractCredentials *OwncloudWizard::getCredentials() const
     return 0;
 }
 
-void OwncloudWizard::askExperimentalVirtualFilesFeature(const std::function<void(bool enable)> &callback)
+void OwncloudWizard::askExperimentalVirtualFilesFeature(QWidget *receiver, const std::function<void(bool enable)> &callback)
 {
-    auto msgBox = new QMessageBox(
-        QMessageBox::Warning,
-        tr("Enable experimental feature?"),
-        tr("When the \"virtual files\" mode is enabled no files will be downloaded initially. "
-           "Instead, a tiny \"%1\" file will be created for each file that exists on the server. "
-           "The contents can be downloaded by running these files or by using their context menu."
-           "\n\n"
-           "This is a new, experimental mode. If you decide to use it, please report any "
-           "issues that come up.")
-            .arg(APPLICATION_DOTVIRTUALFILE_SUFFIX));
-    msgBox->addButton(tr("Enable experimental mode"), QMessageBox::AcceptRole);
-    msgBox->addButton(tr("Stay safe"), QMessageBox::RejectRole);
-    connect(msgBox, &QMessageBox::finished, msgBox, [callback, msgBox](int result) {
-        callback(result == QMessageBox::AcceptRole);
+    const auto bestVfsMode = bestAvailableVfsMode();
+    QMessageBox *msgBox = nullptr;
+    QPushButton *acceptButton = nullptr;
+    switch (bestVfsMode)
+    {
+    case Vfs::WindowsCfApi:
+        msgBox = new QMessageBox(
+            QMessageBox::Warning,
+            tr("Enable technical preview feature?"),
+            tr("When the \"virtual files\" mode is enabled no files will be downloaded initially. "
+               "Instead a virtual file will be created for each file that exists on the server. "
+               "When a file is opened its contents will be downloaded automatically. "
+               "Alternatively, files can be downloaded manually by using their context menu."
+               "\n\n"
+               "The virtual files mode is mutually exclusive with selective sync. "
+               "Currently unselected folders will be translated to online-only folders "
+               "and your selective sync settings will be reset."), QMessageBox::NoButton, receiver);
+        acceptButton = msgBox->addButton(tr("Enable virtual files"), QMessageBox::AcceptRole);
+        msgBox->addButton(tr("Continue to use selective sync"), QMessageBox::RejectRole);
+        break;
+    case Vfs::WithSuffix:
+        msgBox = new QMessageBox(
+            QMessageBox::Warning,
+            tr("Enable experimental feature?"),
+            tr("When the \"virtual files\" mode is enabled no files will be downloaded initially. "
+               "Instead, a tiny \"%1\" file will be created for each file that exists on the server. "
+               "The contents can be downloaded by running these files or by using their context menu."
+               "\n\n"
+               "The virtual files mode is mutually exclusive with selective sync. "
+               "Currently unselected folders will be translated to online-only folders "
+               "and your selective sync settings will be reset."
+               "\n\n"
+               "Switching to this mode will abort any currently running synchronization."
+               "\n\n"
+               "This is a new, experimental mode. If you decide to use it, please report any "
+               "issues that come up.")
+                .arg(APPLICATION_DOTVIRTUALFILE_SUFFIX), QMessageBox::NoButton, receiver);
+        acceptButton = msgBox->addButton(tr("Enable experimental placeholder mode"), QMessageBox::AcceptRole);
+        msgBox->addButton(tr("Stay safe"), QMessageBox::RejectRole);
+        break;
+    case Vfs::Off:
+        Q_UNREACHABLE();
+    }
+
+    connect(msgBox, &QMessageBox::accepted, receiver, [callback, msgBox, acceptButton] {
+        callback(msgBox->clickedButton() == acceptButton);
         msgBox->deleteLater();
     });
     msgBox->open();
