@@ -17,6 +17,7 @@
 #include <QDir>
 #include <QUrl>
 
+#include "guiutility.h"
 #include "wizard/owncloudwizardresultpage.h"
 #include "wizard/owncloudwizardcommon.h"
 #include "theme.h"
@@ -38,7 +39,7 @@ OwncloudWizardResultPage::OwncloudWizardResultPage()
     setSubTitle(QLatin1String(" "));
 
     _ui.pbOpenLocal->setText(tr("Open Local Folder"));
-    _ui.pbOpenLocal->setIcon(QIcon(QLatin1String(":/client/resources/folder-sync.png")));
+    _ui.pbOpenLocal->setIcon(Utility::createColorAwareIcon(QLatin1String(":/client/resources/folder-sync.png")));
     _ui.pbOpenLocal->setIconSize(QSize(48, 48));
     _ui.pbOpenLocal->setToolButtonStyle(Qt::ToolButtonTextUnderIcon);
     connect(_ui.pbOpenLocal, &QAbstractButton::clicked, this, &OwncloudWizardResultPage::slotOpenLocal);
@@ -70,7 +71,7 @@ bool OwncloudWizardResultPage::isComplete() const
 
 void OwncloudWizardResultPage::initializePage()
 {
-    _ui.localFolderLabel->setText(QString::null);
+    _ui.localFolderLabel->setText(QString());
 }
 
 void OwncloudWizardResultPage::setRemoteFolder(const QString &remoteFolder)
@@ -81,7 +82,7 @@ void OwncloudWizardResultPage::setRemoteFolder(const QString &remoteFolder)
 void OwncloudWizardResultPage::setupCustomization()
 {
     // set defaults for the customize labels.
-    _ui.topLabel->setText(QString::null);
+    _ui.topLabel->setText(QString());
     _ui.topLabel->hide();
 
     QVariant variant = Theme::instance()->customMedia(Theme::oCSetupResultTop);
