@@ -22,10 +22,16 @@
 #include <QtGlobal>
 #include <qmetaobject.h>
 
+#include <iostream>
+
 #ifdef ZLIB_FOUND
 #include <zlib.h>
 #endif
 
+#ifdef Q_OS_WIN
+#include <io.h> // for stdout
+#endif
+
 namespace OCC {
 
 static void mirallLogCatcher(QtMsgType type, const QMessageLogContext &ctx, const QString &message)
@@ -34,13 +40,19 @@ static void mirallLogCatcher(QtMsgType type, const QMessageLogContext &ctx, cons
     if (!logger->isNoop()) {
         logger->doLog(qFormatLogMessage(type, ctx, message));
     }
+    if(type == QtCriticalMsg || type == QtFatalMsg) {
+        std::cerr << qPrintable(qFormatLogMessage(type, ctx, message)) << std::endl;
+    }
 
+    if(type == QtFatalMsg) {
+        if (!logger->isNoop()) {
+            logger->close();
+        }
 #if defined(Q_OS_WIN)
     // Make application terminate in a way that can be caught by the crash reporter
-    if(type == QtFatalMsg) {
         Utility::crash();
-    }
 #endif
+    }
 }
 
 
@@ -53,7 +65,6 @@ Logger *Logger::instance()
 Logger::Logger(QObject *parent)
     : QObject(parent)
     , _showTime(true)
-    , _logWindowActivated(false)
     , _doFileFlush(false)
     , _logExpire(0)
     , _logDebug(false)
@@ -110,7 +121,7 @@ void Logger::log(Log log)
 bool Logger::isNoop() const
 {
     QMutexLocker lock(&_mutex);
-    return !_logstream && !_logWindowActivated;
+    return !_logstream;
 }
 
 bool Logger::isLoggingToFile() const
@@ -132,6 +143,17 @@ void Logger::doLog(const QString &msg)
     emit logWindowLog(msg);
 }
 
+void Logger::close()
+{
+    QMutexLocker lock(&_mutex);
+    if (_logstream)
+    {
+        _logstream->flush();
+        _logFile.close();
+        _logstream.reset();
+    }
+}
+
 void Logger::mirallLog(const QString &message)
 {
     Log log_;
@@ -141,12 +163,6 @@ void Logger::mirallLog(const QString &message)
     Logger::instance()->log(log_);
 }
 
-void Logger::setLogWindowActivated(bool activated)
-{
-    QMutexLocker locker(&_mutex);
-    _logWindowActivated = activated;
-}
-
 void Logger::setLogFile(const QString &name)
 {
     QMutexLocker locker(&_mutex);
@@ -161,7 +177,7 @@ void Logger::setLogFile(const QString &name)
 
     bool openSucceeded = false;
     if (name == QLatin1String("-")) {
-        openSucceeded = _logFile.open(1, QIODevice::WriteOnly);
+        openSucceeded = _logFile.open(stdout, QIODevice::WriteOnly);
     } else {
         _logFile.setFileName(name);
         openSucceeded = _logFile.open(QIODevice::WriteOnly);
@@ -179,7 +195,7 @@ void Logger::setLogFile(const QString &name)
     _logstream.reset(new QTextStream(&_logFile));
 }
 
-void Logger::setLogExpire(int expire)
+void Logger::setLogExpire(std::chrono::hours expire)
 {
     _logExpire = expire;
 }
@@ -212,7 +228,6 @@ void Logger::setupTemporaryFolderLogDir()
     if (!QDir().mkpath(dir))
         return;
     setLogDebug(true);
-    setLogExpire(4 /*hours*/);
     setLogDir(dir);
     _temporaryFolderLogDir = true;
 }
@@ -270,13 +285,14 @@ void Logger::enterNextLogFile()
 
         // Expire old log files and deal with conflicts
         QStringList files = dir.entryList(QStringList("*owncloud.log.*"),
-            QDir::Files);
+            QDir::Files, QDir::Name);
         QRegExp rx(R"(.*owncloud\.log\.(\d+).*)");
         int maxNumber = -1;
         foreach (const QString &s, files) {
-            if (_logExpire > 0) {
+            if (_logExpire.count() > 0) {
+                std::chrono::seconds expireSeconds(_logExpire);
                 QFileInfo fileInfo(dir.absoluteFilePath(s));
-                if (fileInfo.lastModified().addSecs(60 * 60 * _logExpire) < now) {
+                if (fileInfo.lastModified().addSecs(expireSeconds.count()) < now) {
                     dir.remove(s);
                 }
             }
@@ -289,10 +305,15 @@ void Logger::enterNextLogFile()
         auto previousLog = _logFile.fileName();
         setLogFile(dir.filePath(newLogName));
 
-        if (!previousLog.isEmpty()) {
-            QString compressedName = previousLog + ".gz";
-            if (compressLog(previousLog, compressedName)) {
-                QFile::remove(previousLog);
+        // Compress the previous log file. On a restart this can be the most recent
+        // log file.
+        auto logToCompress = previousLog;
+        if (logToCompress.isEmpty() && files.size() > 0 && !files.last().endsWith(".gz"))
+            logToCompress = dir.absoluteFilePath(files.last());
+        if (!logToCompress.isEmpty()) {
+            QString compressedName = logToCompress + ".gz";
+            if (compressLog(logToCompress, compressedName)) {
+                QFile::remove(logToCompress);
             } else {
                 QFile::remove(compressedName);
             }
