@@ -30,7 +30,7 @@
 #include "common/syncjournaldb.h"
 #include "bandwidthmanager.h"
 #include "accountfwd.h"
-#include "discoveryphase.h"
+#include "syncoptions.h"
 
 namespace OCC {
 
@@ -49,6 +49,7 @@ qint64 freeSpaceLimit();
 
 class SyncJournalDb;
 class OwncloudPropagator;
+class PropagatorCompositeJob;
 
 /**
  * @brief the base class of propagator jobs
@@ -102,6 +103,14 @@ public:
      */
     virtual qint64 committedDiskSpace() const { return 0; }
 
+    /** Set the associated composite job
+     *
+     * Used only from PropagatorCompositeJob itself, when a job is added
+     * and from PropagateDirectory to associate the subJobs with the first
+     * job.
+     */
+    void setAssociatedComposite(PropagatorCompositeJob *job) { _associatedComposite = job; }
+
 public slots:
     /*
      * Asynchronous abort requires emit of abortFinished() signal,
@@ -128,6 +137,16 @@ signals:
     void abortFinished(SyncFileItem::Status status = SyncFileItem::NormalError);
 protected:
     OwncloudPropagator *propagator() const;
+
+    /** If this job gets added to a composite job, this will point to the parent.
+     *
+     * For the PropagateDirectory::_firstJob it will point to
+     * PropagateDirectory::_subJobs.
+     *
+     * That can be useful for jobs that want to spawn follow-up jobs without
+     * becoming composite jobs themselves.
+     */
+    PropagatorCompositeJob *_associatedComposite = nullptr;
 };
 
 /*
@@ -137,9 +156,7 @@ class PropagateItemJob : public PropagatorJob
 {
     Q_OBJECT
 protected:
-    void done(SyncFileItem::Status status, const QString &errorString = QString());
-
-    bool checkForProblemsWithShared(int httpStatusCode, const QString &msg);
+    virtual void done(SyncFileItem::Status status, const QString &errorString = QString());
 
     /*
      * set a custom restore job message that is used if the restore job succeeded.
@@ -175,7 +192,7 @@ public:
             return false;
         }
         const char *instruction_str = csync_instruction_str(_item->_instruction);
-        qCInfo(lcPropagator) << "Starting" << instruction_str << "propagation of" << _item->_file << "by" << this;
+        qCInfo(lcPropagator) << "Starting" << instruction_str << "propagation of" << _item->destination() << "by" << this;
 
         _state = Running;
         QMetaObject::invokeMethod(this, "start"); // We could be in a different thread (neon jobs)
@@ -215,10 +232,7 @@ public:
         // deleting something that has already been deleted by a shared parent.
     }
 
-    void appendJob(PropagatorJob *job)
-    {
-        _jobsToDo.append(job);
-    }
+    void appendJob(PropagatorJob *job);
     void appendTask(const SyncFileItemPtr &item)
     {
         _tasksToDo.append(item);
@@ -278,7 +292,7 @@ public:
 
     PropagatorCompositeJob _subJobs;
 
-    explicit PropagateDirectory(OwncloudPropagator *propagator, const SyncFileItemPtr &item = SyncFileItemPtr(new SyncFileItem));
+    explicit PropagateDirectory(OwncloudPropagator *propagator, const SyncFileItemPtr &item);
 
     void appendJob(PropagatorJob *job)
     {
@@ -319,10 +333,35 @@ public:
 private slots:
 
     void slotFirstJobFinished(SyncFileItem::Status status);
-    void slotSubJobsFinished(SyncFileItem::Status status);
+    virtual void slotSubJobsFinished(SyncFileItem::Status status);
 
 };
 
+/**
+ * @brief Propagate the root directory, and all its sub entries.
+ * @ingroup libsync
+ *
+ * Primary difference to PropagateDirectory is that it keeps track of directory
+ * deletions that must happen at the very end.
+ */
+class OWNCLOUDSYNC_EXPORT PropagateRootDirectory : public PropagateDirectory
+{
+    Q_OBJECT
+public:
+    PropagatorCompositeJob _dirDeletionJobs;
+
+    explicit PropagateRootDirectory(OwncloudPropagator *propagator);
+
+    bool scheduleSelfOrChild() override;
+    JobParallelism parallelism() override;
+    void abort(PropagatorJob::AbortType abortType) override;
+
+    qint64 committedDiskSpace() const override;
+
+private slots:
+    void slotSubJobsFinished(SyncFileItem::Status status) override;
+    void slotDirDeletionJobsFinished(SyncFileItem::Status status);
+};
 
 /**
  * @brief Dummy job that just mark it as completed and ignored
@@ -339,11 +378,19 @@ public:
     void start() Q_DECL_OVERRIDE
     {
         SyncFileItem::Status status = _item->_status;
-        done(status == SyncFileItem::NoStatus ? SyncFileItem::FileIgnored : status, _item->_errorString);
+        if (status == SyncFileItem::NoStatus) {
+            if (_item->_instruction == CSYNC_INSTRUCTION_ERROR) {
+                status = SyncFileItem::NormalError;
+            } else {
+                status = SyncFileItem::FileIgnored;
+                ASSERT(_item->_instruction == CSYNC_INSTRUCTION_IGNORE);
+            }
+        }
+        done(status, _item->_errorString);
     }
 };
 
-class OwncloudPropagator : public QObject
+class OWNCLOUDSYNC_EXPORT OwncloudPropagator : public QObject
 {
     Q_OBJECT
 public:
@@ -403,7 +450,7 @@ public:
      *
      * This allows skipping of uploads that have a very high likelihood of failure.
      */
-    QHash<QString, quint64> _folderQuota;
+    QHash<QString, qint64> _folderQuota;
 
     /* the maximum number of jobs using bandwidth (uploads or downloads, in parallel) */
     int maximumActiveTransferJob();
@@ -414,14 +461,12 @@ public:
      * if Capabilities::desiredChunkUploadDuration has a target
      * chunk-upload duration set.
      */
-    quint64 _chunkSize;
-    quint64 smallFileSize();
+    qint64 _chunkSize;
+    qint64 smallFileSize();
 
     /* The maximum number of active jobs in parallel  */
     int hardMaximumActiveJob();
 
-    bool isInSharedDirectory(const QString &file);
-
     /** Check whether a download would clash with an existing file
      * in filesystems that are only case-preserving.
      */
@@ -440,9 +485,13 @@ public:
 
     QString getFilePath(const QString &tmp_file_name) const;
 
+    /** Creates the job for an item.
+     */
     PropagateItemJob *createJob(const SyncFileItemPtr &item);
+
     void scheduleNextJob();
-    void reportProgress(const SyncFileItem &, quint64 bytes);
+    void reportProgress(const SyncFileItem &, qint64 bytes);
+    void reportFileTotal(const SyncFileItem &item, qint64 newSize);
 
     void abort()
     {
@@ -465,9 +514,6 @@ public:
         }
     }
 
-    // timeout in seconds
-    static int httpTimeout();
-
     AccountPtr account() const;
 
     enum DiskSpaceResult {
@@ -481,6 +527,32 @@ public:
      */
     DiskSpaceResult diskSpaceCheck() const;
 
+    /** Handles a conflict by renaming the file 'item'.
+     *
+     * Sets up conflict records.
+     *
+     * It also creates a new upload job in composite if the item that's
+     * moved away is a file and conflict uploads are requested.
+     *
+     * Returns true on success, false and error on error.
+     */
+    bool createConflict(const SyncFileItemPtr &item,
+        PropagatorCompositeJob *composite, QString *error);
+
+    // Map original path (as in the DB) to target final path
+    QMap<QString, QString> _renamedDirectories;
+    QString adjustRenamedPath(const QString &original) const;
+
+    /** Update the database for an item.
+     *
+     * Typically after a sync operation succeeded. Updates the inode from
+     * the filesystem.
+     *
+     * Will also trigger a Vfs::convertToPlaceholder.
+     */
+    static bool updateMetadata(const SyncFileItem &item, const QString &localFolderPath, SyncJournalDb &journal, Vfs &vfs);
+    bool updateMetadata(const SyncFileItem &item); // convenience for the above
+
 private slots:
 
     void abortTimeout()
@@ -501,8 +573,10 @@ private slots:
     void scheduleNextJobImpl();
 
 signals:
+    void newItem(const SyncFileItemPtr &);
     void itemCompleted(const SyncFileItemPtr &);
-    void progress(const SyncFileItem &, quint64 bytes);
+    void progress(const SyncFileItem &, qint64 bytes);
+    void updateFileTotal(const SyncFileItem &, qint64 newSize);
     void finished(bool success);
 
     /** Emitted when propagation has problems with a locked file. */
@@ -520,8 +594,9 @@ signals:
 
 private:
     AccountPtr _account;
-    QScopedPointer<PropagateDirectory> _rootJob;
+    QScopedPointer<PropagateRootDirectory> _rootJob;
     SyncOptions _syncOptions;
+    bool _jobScheduled = false;
 };
 
 
@@ -536,15 +611,17 @@ class CleanupPollsJob : public QObject
     AccountPtr _account;
     SyncJournalDb *_journal;
     QString _localPath;
+    QSharedPointer<Vfs> _vfs;
 
 public:
     explicit CleanupPollsJob(const QVector<SyncJournalDb::PollInfo> &pollInfos, AccountPtr account,
-        SyncJournalDb *journal, const QString &localPath, QObject *parent = 0)
+        SyncJournalDb *journal, const QString &localPath, const QSharedPointer<Vfs> &vfs, QObject *parent = 0)
         : QObject(parent)
         , _pollInfos(pollInfos)
         , _account(account)
         , _journal(journal)
         , _localPath(localPath)
+        , _vfs(vfs)
     {
     }
 
