@@ -24,6 +24,7 @@
 #include "propagatorjobs.h"
 #include "common/checksums.h"
 #include "common/asserts.h"
+#include "common/vfs.h"
 
 #include <QLoggingCategory>
 #include <QNetworkAccessManager>
@@ -66,45 +67,35 @@ QString OWNCLOUDSYNC_EXPORT createDownloadTmpFileName(const QString &previous)
 }
 
 // DOES NOT take ownership of the device.
-GETFileJob::GETFileJob(AccountPtr account, const QString &path, QFile *device,
+GETFileJob::GETFileJob(AccountPtr account, const QString &path, QIODevice *device,
     const QMap<QByteArray, QByteArray> &headers, const QByteArray &expectedEtagForResume,
-    quint64 resumeStart, QObject *parent)
-    : AbstractNetworkJob(account, path, parent)
+    qint64 resumeStart, QObject *parent)
+    : GETJob(account, path, parent)
     , _device(device)
     , _headers(headers)
     , _expectedEtagForResume(expectedEtagForResume)
+    , _expectedContentLength(-1)
+    , _contentLength(-1)
     , _resumeStart(resumeStart)
-    , _errorStatus(SyncFileItem::NoStatus)
-    , _bandwidthLimited(false)
-    , _bandwidthChoked(false)
-    , _bandwidthQuota(0)
-    , _bandwidthManager(0)
     , _hasEmittedFinishedSignal(false)
-    , _lastModified()
 {
 }
 
-GETFileJob::GETFileJob(AccountPtr account, const QUrl &url, QFile *device,
+GETFileJob::GETFileJob(AccountPtr account, const QUrl &url, QIODevice *device,
     const QMap<QByteArray, QByteArray> &headers, const QByteArray &expectedEtagForResume,
-    quint64 resumeStart, QObject *parent)
-
-    : AbstractNetworkJob(account, url.toEncoded(), parent)
+    qint64 resumeStart, QObject *parent)
+    : GETJob(account, url.toEncoded(), parent)
     , _device(device)
     , _headers(headers)
     , _expectedEtagForResume(expectedEtagForResume)
+    , _expectedContentLength(-1)
+    , _contentLength(-1)
     , _resumeStart(resumeStart)
-    , _errorStatus(SyncFileItem::NoStatus)
     , _directDownloadUrl(url)
-    , _bandwidthLimited(false)
-    , _bandwidthChoked(false)
-    , _bandwidthQuota(0)
-    , _bandwidthManager(0)
     , _hasEmittedFinishedSignal(false)
-    , _lastModified()
 {
 }
 
-
 void GETFileJob::start()
 {
     if (_resumeStart > 0) {
@@ -132,10 +123,6 @@ void GETFileJob::start()
         _bandwidthManager->registerDownloadJob(this);
     }
 
-    if (reply()->error() != QNetworkReply::NoError) {
-        qCWarning(lcGetJob) << " Network error: " << errorString();
-    }
-
     connect(this, &AbstractNetworkJob::networkActivity, account().data(), &Account::propagatorNetworkActivity);
 
     AbstractNetworkJob::start();
@@ -147,6 +134,7 @@ void GETFileJob::newReplyHook(QNetworkReply *reply)
 
     connect(reply, &QNetworkReply::metaDataChanged, this, &GETFileJob::slotMetaDataChanged);
     connect(reply, &QIODevice::readyRead, this, &GETFileJob::slotReadyRead);
+    connect(reply, &QNetworkReply::finished, this, &GETFileJob::slotReadyRead);
     connect(reply, &QNetworkReply::downloadProgress, this, &GETFileJob::downloadProgress);
 }
 
@@ -158,14 +146,23 @@ void GETFileJob::slotMetaDataChanged()
 
     int httpStatus = reply()->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt();
 
-    // Ignore redirects
-    if (httpStatus == 301 || httpStatus == 302 || httpStatus == 303 || httpStatus == 307 || httpStatus == 308)
+    if (httpStatus == 301 || httpStatus == 302 || httpStatus == 303 || httpStatus == 307
+        || httpStatus == 308 || httpStatus == 401) {
+        // Redirects and auth failures (oauth token renew) are handled by AbstractNetworkJob and
+        // will end up restarting the job. We do not want to process further data from the initial
+        // request. newReplyHook() will reestablish signal connections for the follow-up request.
+        bool ok = disconnect(reply(), &QNetworkReply::finished, this, &GETFileJob::slotReadyRead)
+            && disconnect(reply(), &QNetworkReply::readyRead, this, &GETFileJob::slotReadyRead);
+        ASSERT(ok);
         return;
+    }
 
     // If the status code isn't 2xx, don't write the reply body to the file.
     // For any error: handle it when the job is finished, not here.
     if (httpStatus / 100 != 2) {
-        _device->close();
+        // Disable the buffer limit, as we don't limit the bandwidth for error messages.
+        // (We are only going to do a readAll() at the end.)
+        reply()->setReadBufferSize(0);
         return;
     }
     if (reply()->error() != QNetworkReply::NoError) {
@@ -193,12 +190,22 @@ void GETFileJob::slotMetaDataChanged()
         return;
     }
 
-    quint64 start = 0;
+    _contentLength = reply()->header(QNetworkRequest::ContentLengthHeader).toLongLong();
+    if (_expectedContentLength != -1 && _contentLength != _expectedContentLength) {
+        qCWarning(lcGetJob) << "We received a different content length than expected!"
+                            << _expectedContentLength << "vs" << _contentLength;
+        _errorString = tr("We received an unexpected download Content-Length.");
+        _errorStatus = SyncFileItem::NormalError;
+        reply()->abort();
+        return;
+    }
+
+    qint64 start = 0;
     QByteArray ranges = reply()->rawHeader("Content-Range");
     if (!ranges.isEmpty()) {
         QRegExp rx("bytes (\\d+)-");
         if (rx.indexIn(ranges) >= 0) {
-            start = rx.cap(1).toULongLong();
+            start = rx.cap(1).toLongLong();
         }
     }
     if (start != _resumeStart) {
@@ -229,24 +236,24 @@ void GETFileJob::slotMetaDataChanged()
     _saveBodyToFile = true;
 }
 
-void GETFileJob::setBandwidthManager(BandwidthManager *bwm)
+void GETJob::setBandwidthManager(BandwidthManager *bwm)
 {
     _bandwidthManager = bwm;
 }
 
-void GETFileJob::setChoked(bool c)
+void GETJob::setChoked(bool c)
 {
     _bandwidthChoked = c;
     QMetaObject::invokeMethod(this, "slotReadyRead", Qt::QueuedConnection);
 }
 
-void GETFileJob::setBandwidthLimited(bool b)
+void GETJob::setBandwidthLimited(bool b)
 {
     _bandwidthLimited = b;
     QMetaObject::invokeMethod(this, "slotReadyRead", Qt::QueuedConnection);
 }
 
-void GETFileJob::giveBandwidthQuota(qint64 q)
+void GETJob::giveBandwidthQuota(qint64 q)
 {
     _bandwidthQuota = q;
     qCDebug(lcGetJob) << "Got" << q << "bytes";
@@ -268,7 +275,7 @@ void GETFileJob::slotReadyRead()
     int bufferSize = qMin(1024 * 8ll, reply()->bytesAvailable());
     QByteArray buffer(bufferSize, Qt::Uninitialized);
 
-    while (reply()->bytesAvailable() > 0) {
+    while (reply()->bytesAvailable() > 0 && _saveBodyToFile) {
         if (_bandwidthChoked) {
             qCWarning(lcGetJob) << "Download choked";
             break;
@@ -292,27 +299,24 @@ void GETFileJob::slotReadyRead()
             return;
         }
 
-        if (_device->isOpen() && _saveBodyToFile) {
-            qint64 w = _device->write(buffer.constData(), r);
-            if (w != r) {
-                _errorString = _device->errorString();
-                _errorStatus = SyncFileItem::NormalError;
-                qCWarning(lcGetJob) << "Error while writing to file" << w << r << _errorString;
-                reply()->abort();
-                return;
-            }
+        qint64 w = _device->write(buffer.constData(), r);
+        if (w != r) {
+            _errorString = _device->errorString();
+            _errorStatus = SyncFileItem::NormalError;
+            qCWarning(lcGetJob) << "Error while writing to file" << w << r << _errorString;
+            reply()->abort();
+            return;
         }
     }
 
-    if (reply()->isFinished() && reply()->bytesAvailable() == 0) {
+    if (reply()->isFinished() && (reply()->bytesAvailable() == 0 || !_saveBodyToFile)) {
         qCDebug(lcGetJob) << "Actually finished!";
         if (_bandwidthManager) {
             _bandwidthManager->unregisterDownloadJob(this);
         }
         if (!_hasEmittedFinishedSignal) {
             qCInfo(lcGetJob) << "GET of" << reply()->request().url().toString() << "FINISHED WITH STATUS"
-                             << reply()->error()
-                             << (reply()->error() == QNetworkReply::NoError ? QLatin1String("") : errorString())
+                             << replyStatusString()
                              << reply()->rawHeader("Content-Range") << reply()->rawHeader("Content-Length");
 
             emit finishedSignal();
@@ -322,7 +326,7 @@ void GETFileJob::slotReadyRead()
     }
 }
 
-void GETFileJob::onTimedOut()
+void GETJob::onTimedOut()
 {
     qCWarning(lcGetJob) << "Timeout" << (reply() ? reply()->request().url() : path());
     if (!reply())
@@ -332,7 +336,7 @@ void GETFileJob::onTimedOut()
     reply()->abort();
 }
 
-QString GETFileJob::errorString() const
+QString GETJob::errorString() const
 {
     if (!_errorString.isEmpty()) {
         return _errorString;
@@ -348,6 +352,43 @@ void PropagateDownloadFile::start()
     qCDebug(lcPropagateDownload) << _item->_file << propagator()->_activeJobList.count();
     _stopwatch.start();
 
+    auto &syncOptions = propagator()->syncOptions();
+    auto &vfs = syncOptions._vfs;
+
+    // For virtual files just dehydrate or create the file and be done
+    if (_item->_type == ItemTypeVirtualFileDehydration) {
+        QString fsPath = propagator()->getFilePath(_item->_file);
+        if (!FileSystem::verifyFileUnchanged(fsPath, _item->_previousSize, _item->_previousModtime)) {
+            propagator()->_anotherSyncNeeded = true;
+            done(SyncFileItem::SoftError, tr("File has changed since discovery"));
+            return;
+        }
+
+        qCDebug(lcPropagateDownload) << "dehydrating file" << _item->_file;
+        auto r = vfs->dehydratePlaceholder(*_item);
+        if (!r) {
+            done(SyncFileItem::NormalError, r.error());
+            return;
+        }
+        propagator()->_journal->deleteFileRecord(_item->_originalFile);
+        updateMetadata(false);
+        return;
+    }
+    if (vfs->mode() == Vfs::Off && _item->_type == ItemTypeVirtualFile) {
+        qCWarning(lcPropagateDownload) << "ignored virtual file type of" << _item->_file;
+        _item->_type = ItemTypeFile;
+    }
+    if (_item->_type == ItemTypeVirtualFile) {
+        qCDebug(lcPropagateDownload) << "creating virtual file" << _item->_file;
+        auto r = vfs->createPlaceholder(*_item);
+        if (!r) {
+            done(SyncFileItem::NormalError, r.error());
+            return;
+        }
+        updateMetadata(false);
+        return;
+    }
+
     if (_deleteExisting) {
         deleteExistingFolder();
 
@@ -372,6 +413,7 @@ void PropagateDownloadFile::start()
         computeChecksum->setChecksumType(parseChecksumHeaderType(_item->_checksumHeader));
         connect(computeChecksum, &ComputeChecksum::done,
             this, &PropagateDownloadFile::conflictChecksumComputed);
+        propagator()->_activeJobList.append(this);
         computeChecksum->start(propagator()->getFilePath(_item->_file));
         return;
     }
@@ -381,6 +423,7 @@ void PropagateDownloadFile::start()
 
 void PropagateDownloadFile::conflictChecksumComputed(const QByteArray &checksumType, const QByteArray &checksum)
 {
+    propagator()->_activeJobList.removeOne(this);
     if (makeChecksumHeader(checksumType, checksum) == _item->_checksumHeader) {
         // No download necessary, just update fs and journal metadata
         qCDebug(lcPropagateDownload) << _item->_file << "remote and local checksum match";
@@ -413,7 +456,6 @@ void PropagateDownloadFile::startDownload()
     propagator()->reportProgress(*_item, 0);
 
     QString tmpFileName;
-    QByteArray expectedEtagForResume;
     const SyncJournalDb::DownloadInfo progressInfo = propagator()->_journal->getDownloadInfo(_item->_file);
     if (progressInfo._valid) {
         // if the etag has changed meanwhile, remove the already downloaded part.
@@ -422,31 +464,33 @@ void PropagateDownloadFile::startDownload()
             propagator()->_journal->setDownloadInfo(_item->_file, SyncJournalDb::DownloadInfo());
         } else {
             tmpFileName = progressInfo._tmpfile;
-            expectedEtagForResume = progressInfo._etag;
+            _expectedEtagForResume = progressInfo._etag;
         }
     }
 
     if (tmpFileName.isEmpty()) {
         tmpFileName = createDownloadTmpFileName(_item->_file);
     }
-
     _tmpFile.setFileName(propagator()->getFilePath(tmpFileName));
-    if (!_tmpFile.open(QIODevice::Append | QIODevice::Unbuffered)) {
-        done(SyncFileItem::NormalError, _tmpFile.errorString());
+
+    _resumeStart = _tmpFile.size();
+    if (_resumeStart > 0 && _resumeStart == _item->_size) {
+        qCInfo(lcPropagateDownload) << "File is already complete, no need to download";
+        downloadFinished();
         return;
     }
 
-    FileSystem::setFileHidden(_tmpFile.fileName(), true);
-
-    _resumeStart = _tmpFile.size();
-    if (_resumeStart > 0) {
-        if (_resumeStart == _item->_size) {
-            qCInfo(lcPropagateDownload) << "File is already complete, no need to download";
-            _tmpFile.close();
-            downloadFinished();
-            return;
-        }
+    // Can't open(Append) read-only files, make sure to make
+    // file writable if it exists.
+    if (_tmpFile.exists())
+        FileSystem::setFileReadOnly(_tmpFile.fileName(), false);
+    if (!_tmpFile.open(QIODevice::Append | QIODevice::Unbuffered)) {
+        qCWarning(lcPropagateDownload) << "could not open temporary file" << _tmpFile.fileName();
+        done(SyncFileItem::NormalError, _tmpFile.errorString());
+        return;
     }
+    // Hide temporary after creation
+    FileSystem::setFileHidden(_tmpFile.fileName(), true);
 
     // If there's not enough space to fully download this file, stop.
     const auto diskSpaceResult = propagator()->diskSpaceCheck();
@@ -480,13 +524,60 @@ void PropagateDownloadFile::startDownload()
         propagator()->_journal->commit("download file start");
     }
 
+    if (_item->_remotePerm.hasPermission(RemotePermissions::HasZSyncMetadata) && isZsyncPropagationEnabled(propagator(), _item)) {
+        if (_item->_previousSize) {
+            // Retrieve zsync metadata file from the server
+            qCInfo(lcZsyncGet) << "Retrieving zsync metadata for:" << _item->_file;
+            QNetworkRequest req;
+            req.setPriority(QNetworkRequest::LowPriority);
+            QUrl zsyncUrl = zsyncMetadataUrl(propagator(), _item->_file);
+            auto job = propagator()->account()->sendRequest("GET", zsyncUrl, req);
+            connect(job, &SimpleNetworkJob::finishedSignal, this, &PropagateDownloadFile::slotZsyncGetMetaFinished);
+            return;
+        }
+        qCInfo(lcZsyncGet) << "No local copy of:" << _item->_file;
+    }
+
+    startFullDownload();
+}
+
+
+void PropagateDownloadFile::slotZsyncGetMetaFinished(QNetworkReply *reply)
+{
+    int httpStatusCode = reply->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt();
+    if (httpStatusCode / 100 != 2) {
+        /* Fall back to full download */
+        qCWarning(lcZsyncGet) << "Failed to retrieve zsync metadata for:" << _item->_file;
+        startFullDownload();
+        return;
+    }
+
+    QByteArray zsyncData = reply->readAll();
+    _expectedEtagForResume = getEtagFromReply(reply);
+    qCInfo(lcZsyncGet) << "Retrieved zsync metadata for:" << _item->_file << "size:" << zsyncData.size()
+                       << "etag:" << _expectedEtagForResume;
+
+    QMap<QByteArray, QByteArray> headers;
+    _job = new GETFileZsyncJob(propagator(), _item, propagator()->_remoteFolder + _item->_file,
+        &_tmpFile, headers, _expectedEtagForResume, zsyncData, this);
+    connect(_job.data(), &GETJob::finishedSignal, this, &PropagateDownloadFile::slotGetFinished);
+    connect(qobject_cast<GETFileZsyncJob *>(_job.data()), &GETFileZsyncJob::overallDownloadProgress,
+        this, &PropagateDownloadFile::slotDownloadProgress);
+    _job->setBandwidthManager(&propagator()->_bandwidthManager);
+    propagator()->_activeJobList.append(this);
+    _job->start();
+    _isDeltaSyncDownload = true;
+}
+
+void PropagateDownloadFile::startFullDownload()
+{
     QMap<QByteArray, QByteArray> headers;
 
     if (_item->_directDownloadUrl.isEmpty()) {
         // Normal job, download from oC instance
         _job = new GETFileJob(propagator()->account(),
             propagator()->_remoteFolder + _item->_file,
-            &_tmpFile, headers, expectedEtagForResume, _resumeStart, this);
+            &_tmpFile, headers, _expectedEtagForResume, _resumeStart, this);
     } else {
         // We were provided a direct URL, use that one
         qCInfo(lcPropagateDownload) << "directDownloadUrl given for " << _item->_file << _item->_directDownloadUrl;
@@ -498,11 +589,12 @@ void PropagateDownloadFile::startDownload()
         QUrl url = QUrl::fromUserInput(_item->_directDownloadUrl);
         _job = new GETFileJob(propagator()->account(),
             url,
-            &_tmpFile, headers, expectedEtagForResume, _resumeStart, this);
+            &_tmpFile, headers, _expectedEtagForResume, _resumeStart, this);
     }
     _job->setBandwidthManager(&propagator()->_bandwidthManager);
-    connect(_job.data(), &GETFileJob::finishedSignal, this, &PropagateDownloadFile::slotGetFinished);
-    connect(_job.data(), &GETFileJob::downloadProgress, this, &PropagateDownloadFile::slotDownloadProgress);
+    connect(_job.data(), &GETJob::finishedSignal, this, &PropagateDownloadFile::slotGetFinished);
+    connect(qobject_cast<GETFileJob *>(_job.data()), &GETFileJob::downloadProgress,
+        this, &PropagateDownloadFile::slotDownloadProgress);
     propagator()->_activeJobList.append(this);
     _job->start();
 }
@@ -510,7 +602,7 @@ void PropagateDownloadFile::startDownload()
 qint64 PropagateDownloadFile::committedDiskSpace() const
 {
     if (_state == Running) {
-        return qBound(0ULL, _item->_size - _resumeStart - _downloadProgress, _item->_size);
+        return qBound(0LL, _item->_size - _resumeStart - _downloadProgress, _item->_size);
     }
     return 0;
 }
@@ -525,12 +617,37 @@ void PropagateDownloadFile::slotGetFinished()
 {
     propagator()->_activeJobList.removeOne(this);
 
-    GETFileJob *job = qobject_cast<GETFileJob *>(sender());
+    GETJob *job = _job;
     ASSERT(job);
 
+    SyncFileItem::Status status = job->errorStatus();
+
+    // Needed because GETFileZsyncJob may emit finishedSignal without any further network activity
+    if (!job->reply()) {
+        if (status == SyncFileItem::Success) {
+            _tmpFile.close();
+            _tmpFile.flush();
+            downloadFinished();
+            return;
+        }
+
+        FileSystem::remove(_tmpFile.fileName());
+        if (status != SyncFileItem::NoStatus) {
+            done(status, job->errorString());
+            return;
+        }
+
+        ASSERT(false, "Download slot finished, but there was no reply!");
+        done(SyncFileItem::FatalError, tr("Download slot finished, but there was no reply!"));
+        return;
+    }
+
+    _item->_httpErrorCode = job->reply()->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt();
+    _item->_responseTimeStamp = job->responseTimestamp();
+    _item->_requestId = job->requestId();
+
     QNetworkReply::NetworkError err = job->reply()->error();
     if (err != QNetworkReply::NoError) {
-        _item->_httpErrorCode = job->reply()->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt();
 
         // If we sent a 'Range' header and get 416 back, we want to retry
         // without the header.
@@ -548,7 +665,7 @@ void PropagateDownloadFile::slotGetFinished()
 
         // Don't keep the temporary file if it is empty or we
         // used a bad range header or the file's not on the server anymore.
-        if (_tmpFile.size() == 0 || badRangeHeader || fileNotFound) {
+        if (_tmpFile.exists() && (_tmpFile.size() == 0 || badRangeHeader || fileNotFound)) {
             _tmpFile.close();
             FileSystem::remove(_tmpFile.fileName());
             propagator()->_journal->setDownloadInfo(_item->_file, SyncJournalDb::DownloadInfo());
@@ -580,16 +697,19 @@ void PropagateDownloadFile::slotGetFinished()
             // As a precaution against bugs that cause our database and the
             // reality on the server to diverge, rediscover this folder on the
             // next sync run.
-            propagator()->_journal->avoidReadFromDbOnNextSync(_item->_file);
+            propagator()->_journal->schedulePathForRemoteDiscovery(_item->_file);
         }
 
+        QByteArray errorBody;
+        QString errorString = _item->_httpErrorCode >= 400 ? job->errorStringParsingBody(&errorBody)
+                                                           : job->errorString();
         SyncFileItem::Status status = job->errorStatus();
         if (status == SyncFileItem::NoStatus) {
             status = classifyError(err, _item->_httpErrorCode,
-                &propagator()->_anotherSyncNeeded);
+                &propagator()->_anotherSyncNeeded, errorBody);
         }
 
-        done(status, job->errorString());
+        done(status, errorString);
         return;
     }
 
@@ -603,7 +723,6 @@ void PropagateDownloadFile::slotGetFinished()
         // so make sure we have the up-to-date time
         _item->_modtime = job->lastModified();
     }
-    _item->_responseTimeStamp = job->responseTimestamp();
 
     _tmpFile.close();
     _tmpFile.flush();
@@ -613,9 +732,25 @@ void PropagateDownloadFile::slotGetFinished()
      * truncated, as described here: https://github.com/owncloud/mirall/issues/2528
      */
     const QByteArray sizeHeader("Content-Length");
-    quint64 bodySize = job->reply()->rawHeader(sizeHeader).toULongLong();
+    qint64 bodySize = job->reply()->rawHeader(sizeHeader).toLongLong();
+    bool hasSizeHeader = !job->reply()->rawHeader(sizeHeader).isEmpty();
 
-    if (!job->reply()->rawHeader(sizeHeader).isEmpty() && _tmpFile.size() > 0 && bodySize == 0) {
+    // Qt removes the content-length header for transparently decompressed HTTP1 replies
+    // but not for HTTP2 or SPDY replies. For these it remains and contains the size
+    // of the compressed data. See QTBUG-73364.
+    const auto contentEncoding = job->reply()->rawHeader("content-encoding").toLower();
+    if ((contentEncoding == "gzip" || contentEncoding == "deflate")
+        && (
+#if QT_VERSION >= QT_VERSION_CHECK(5, 9, 0)
+           job->reply()->attribute(QNetworkRequest::HTTP2WasUsedAttribute).toBool() ||
+#endif
+           job->reply()->attribute(QNetworkRequest::SpdyWasUsedAttribute).toBool()
+         )) {
+        bodySize = 0;
+        hasSizeHeader = false;
+    }
+
+    if (!_isDeltaSyncDownload && hasSizeHeader && _tmpFile.size() > 0 && bodySize == 0) {
         // Strange bug with broken webserver or webfirewall https://github.com/owncloud/client/issues/3373#issuecomment-122672322
         // This happened when trying to resume a file. The Content-Range header was files, Content-Length was == 0
         qCDebug(lcPropagateDownload) << bodySize << _item->_size << _tmpFile.size() << job->resumeStart();
@@ -624,7 +759,7 @@ void PropagateDownloadFile::slotGetFinished()
         return;
     }
 
-    if (bodySize > 0 && bodySize != _tmpFile.size() - job->resumeStart()) {
+    if (!_isDeltaSyncDownload && bodySize > 0 && bodySize != _tmpFile.size() - job->resumeStart()) {
         qCDebug(lcPropagateDownload) << bodySize << _tmpFile.size() << job->resumeStart();
         propagator()->_anotherSyncNeeded = true;
         done(SyncFileItem::SoftError, tr("The file could not be downloaded completely."));
@@ -639,6 +774,27 @@ void PropagateDownloadFile::slotGetFinished()
         return;
     }
 
+    // Did the file come with conflict headers? If so, store them now!
+    // If we download conflict files but the server doesn't send conflict
+    // headers, the record will be established by SyncEngine::conflictRecordMaintenance.
+    // (we can't reliably determine the file id of the base file here,
+    // it might still be downloaded in a parallel job and not exist in
+    // the database yet!)
+    if (job->reply()->rawHeader("OC-Conflict") == "1") {
+        _conflictRecord.path = _item->_file.toUtf8();
+        _conflictRecord.initialBasePath = job->reply()->rawHeader("OC-ConflictInitialBasePath");
+        _conflictRecord.baseFileId = job->reply()->rawHeader("OC-ConflictBaseFileId");
+        _conflictRecord.baseEtag = job->reply()->rawHeader("OC-ConflictBaseEtag");
+
+        auto mtimeHeader = job->reply()->rawHeader("OC-ConflictBaseMtime");
+        if (!mtimeHeader.isEmpty())
+            _conflictRecord.baseModtime = mtimeHeader.toLongLong();
+
+        // We don't set it yet. That will only be done when the download finished
+        // successfully, much further down. Here we just grab the headers because the
+        // job will be deleted later.
+    }
+
     // Do checksum validation for the download. If there is no checksum header, the validator
     // will also emit the validated() signal to continue the flow in slot transmissionChecksumValidated()
     // as this is (still) also correct.
@@ -677,14 +833,9 @@ void PropagateDownloadFile::deleteExistingFolder()
         // on error, just try to move it away...
     }
 
-    QString conflictDir = FileSystem::makeConflictFileName(
-        existingDir, Utility::qDateTimeFromTime_t(FileSystem::getModTime(existingDir)));
-
-    emit propagator()->touchedFile(existingDir);
-    emit propagator()->touchedFile(conflictDir);
-    QString renameError;
-    if (!FileSystem::rename(existingDir, conflictDir, &renameError)) {
-        done(SyncFileItem::NormalError, renameError);
+    QString error;
+    if (!propagator()->createConflict(_item, _associatedComposite, &error)) {
+        done(SyncFileItem::NormalError, error);
     }
 }
 
@@ -752,7 +903,9 @@ namespace { // Anonymous namespace for the recall feature
     static void preserveGroupOwnership(const QString &fileName, const QFileInfo &fi)
     {
 #ifdef Q_OS_UNIX
-        chown(fileName.toLocal8Bit().constData(), -1, fi.groupId());
+        if (chown(fileName.toLocal8Bit().constData(), -1, fi.groupId()) != 0) {
+            qCWarning(lcPropagateDownload) << "Unable to chown" << fileName << "to previous group owner" << strerror(errno);
+        }
 #else
         Q_UNUSED(fileName);
         Q_UNUSED(fi);
@@ -790,6 +943,7 @@ void PropagateDownloadFile::contentChecksumComputed(const QByteArray &checksumTy
 
 void PropagateDownloadFile::downloadFinished()
 {
+    ASSERT(!_tmpFile.isOpen());
     QString fn = propagator()->getFilePath(_item->_file);
 
     // In case of file name clash, report an error
@@ -799,35 +953,13 @@ void PropagateDownloadFile::downloadFinished()
         return;
     }
 
-    // In case of conflict, make a backup of the old file
-    // Ignore conflicts where both files are binary equal
-    bool isConflict = _item->_instruction == CSYNC_INSTRUCTION_CONFLICT
-        && !FileSystem::fileEquals(fn, _tmpFile.fileName());
-    if (isConflict) {
-        QString renameError;
-        QString conflictFileName = FileSystem::makeConflictFileName(
-            fn, Utility::qDateTimeFromTime_t(FileSystem::getModTime(fn)));
-        if (!FileSystem::rename(fn, conflictFileName, &renameError)) {
-            // If the rename fails, don't replace it.
-
-            // If the file is locked, we want to retry this sync when it
-            // becomes available again.
-            if (FileSystem::isFileLocked(fn)) {
-                emit propagator()->seenLockedFile(fn);
-            }
-
-            done(SyncFileItem::SoftError, renameError);
-            return;
-        }
-        qCInfo(lcPropagateDownload) << "Created conflict file" << fn << "->" << conflictFileName;
-    }
-
     FileSystem::setModTime(_tmpFile.fileName(), _item->_modtime);
     // We need to fetch the time again because some file systems such as FAT have worse than a second
     // Accuracy, and we really need the time from the file system. (#3103)
     _item->_modtime = FileSystem::getModTime(_tmpFile.fileName());
 
-    if (FileSystem::fileExists(fn)) {
+    bool previousFileExists = FileSystem::fileExists(fn);
+    if (previousFileExists) {
         // Preserve the existing file permissions.
         QFileInfo existingFile(fn);
         if (existingFile.permissions() != _tmpFile.permissions()) {
@@ -835,6 +967,25 @@ void PropagateDownloadFile::downloadFinished()
         }
         preserveGroupOwnership(_tmpFile.fileName(), existingFile);
 
+        // Make the file a hydrated placeholder if possible
+        propagator()->syncOptions()._vfs->convertToPlaceholder(_tmpFile.fileName(), *_item, fn);
+    }
+
+    // Apply the remote permissions
+    FileSystem::setFileReadOnlyWeak(_tmpFile.fileName(), !_item->_remotePerm.isNull() && !_item->_remotePerm.hasPermission(RemotePermissions::CanWrite));
+
+    bool isConflict = _item->_instruction == CSYNC_INSTRUCTION_CONFLICT
+        && (QFileInfo(fn).isDir() || !FileSystem::fileEquals(fn, _tmpFile.fileName()));
+    if (isConflict) {
+        QString error;
+        if (!propagator()->createConflict(_item, _associatedComposite, &error)) {
+            done(SyncFileItem::SoftError, error);
+            return;
+        }
+        previousFileExists = false;
+    }
+
+    if (previousFileExists) {
         // Check whether the existing file has changed since the discovery
         // phase by comparing size and mtime to the previous values. This
         // is necessary to avoid overwriting user changes that happened between
@@ -848,27 +999,11 @@ void PropagateDownloadFile::downloadFinished()
         }
     }
 
-    // Apply the remote permissions
-    FileSystem::setFileReadOnlyWeak(_tmpFile.fileName(), !_item->_remotePerm.isNull() && !_item->_remotePerm.hasPermission(RemotePermissions::CanWrite));
-
     QString error;
     emit propagator()->touchedFile(fn);
     // The fileChanged() check is done above to generate better error messages.
     if (!FileSystem::uncheckedRenameReplace(_tmpFile.fileName(), fn, &error)) {
         qCWarning(lcPropagateDownload) << QString("Rename failed: %1 => %2").arg(_tmpFile.fileName()).arg(fn);
-
-        // If we moved away the original file due to a conflict but can't
-        // put the downloaded file in its place, we are in a bad spot:
-        // If we do nothing the next sync run will assume the user deleted
-        // the file!
-        // To avoid that, the file is removed from the metadata table entirely
-        // which makes it look like we're just about to initially download
-        // it.
-        if (isConflict) {
-            propagator()->_journal->deleteFileRecord(fn);
-            propagator()->_journal->commit("download finished");
-        }
-
         // If the file is locked, we want to retry this sync when it
         // becomes available again, otherwise try again directly
         if (FileSystem::isFileLocked(fn)) {
@@ -880,12 +1015,43 @@ void PropagateDownloadFile::downloadFinished()
         done(SyncFileItem::SoftError, error);
         return;
     }
+
     FileSystem::setFileHidden(fn, false);
 
     // Maybe we downloaded a newer version of the file than we thought we would...
     // Get up to date information for the journal.
     _item->_size = FileSystem::getSize(fn);
 
+    // Maybe what we downloaded was a conflict file? If so, set a conflict record.
+    // (the data was prepared in slotGetFinished above)
+    if (_conflictRecord.isValid())
+        propagator()->_journal->setConflictRecord(_conflictRecord);
+
+    auto vfs = propagator()->syncOptions()._vfs;
+    if (vfs && vfs->mode() == Vfs::WithSuffix) {
+        // If the virtual file used to have a different name and db
+        // entry, remove it transfer its old pin state.
+        if (_item->_type == ItemTypeVirtualFileDownload) {
+            QString virtualFile = _item->_file + vfs->fileSuffix();
+            auto fn = propagator()->getFilePath(virtualFile);
+            qCDebug(lcPropagateDownload) << "Download of previous virtual file finished" << fn;
+            QFile::remove(fn);
+            propagator()->_journal->deleteFileRecord(virtualFile);
+
+            // Move the pin state to the new location
+            auto pin = propagator()->_journal->internalPinStates().rawForPath(virtualFile.toUtf8());
+            if (pin && *pin != PinState::Inherited) {
+                vfs->setPinState(_item->_file, *pin);
+                vfs->setPinState(virtualFile, PinState::Inherited);
+            }
+        }
+
+        // Ensure the pin state isn't contradictory
+        auto pin = vfs->pinState(_item->_file);
+        if (pin && *pin == PinState::OnlineOnly)
+            vfs->setPinState(_item->_file, PinState::Unspecified);
+    }
+
     updateMetadata(isConflict);
 }
 
@@ -893,12 +1059,13 @@ void PropagateDownloadFile::updateMetadata(bool isConflict)
 {
     QString fn = propagator()->getFilePath(_item->_file);
 
-    if (!propagator()->_journal->setFileRecord(_item->toSyncJournalFileRecordWithInode(fn))) {
+    if (!propagator()->updateMetadata(*_item)) {
         done(SyncFileItem::FatalError, tr("Error writing metadata to the database"));
         return;
     }
     propagator()->_journal->setDownloadInfo(_item->_file, SyncJournalDb::DownloadInfo());
     propagator()->_journal->commit("download file start2");
+
     done(isConflict ? SyncFileItem::Conflict : SyncFileItem::Success);
 
     // handle the special recall file
@@ -919,6 +1086,7 @@ void PropagateDownloadFile::slotDownloadProgress(qint64 received, qint64)
     if (!_job)
         return;
     _downloadProgress = received;
+
     propagator()->reportProgress(*_item, _resumeStart + received);
 }
 
