@@ -15,12 +15,12 @@
 
 #include "owncloudpropagator.h"
 #include "networkjobs.h"
+#include "propagatecommonzsync.h"
 
 #include <QBuffer>
 #include <QFile>
 #include <QElapsedTimer>
 
-
 namespace OCC {
 
 Q_DECLARE_LOGGING_CATEGORY(lcPutJob)
@@ -36,11 +36,11 @@ class UploadDevice : public QIODevice
 {
     Q_OBJECT
 public:
-    UploadDevice(BandwidthManager *bwm);
+    UploadDevice(const QString &fileName, qint64 start, qint64 size, BandwidthManager *bwm);
     ~UploadDevice();
 
-    /** Reads the data from the file and opens the device */
-    bool prepareAndOpen(const QString &fileName, qint64 start, qint64 size);
+    bool open(QIODevice::OpenMode mode) override;
+    void close() override;
 
     qint64 writeData(const char *, qint64) Q_DECL_OVERRIDE;
     qint64 readData(char *data, qint64 maxlen) Q_DECL_OVERRIDE;
@@ -59,10 +59,15 @@ public:
 signals:
 
 private:
-    // The file data
-    QByteArray _data;
-    // Position in the data
-    qint64 _read;
+    /// The local file to read data from
+    QFile _file;
+
+    /// Start of the file data to use
+    qint64 _start = 0;
+    /// Amount of file data after _start to use
+    qint64 _size = 0;
+    /// Position between _start and _start+_size
+    qint64 _read = 0;
 
     // Bandwidth manager related
     QPointer<BandwidthManager> _bandwidthManager;
@@ -116,16 +121,7 @@ public:
 
     virtual void start() Q_DECL_OVERRIDE;
 
-    virtual bool finished() Q_DECL_OVERRIDE
-    {
-        qCInfo(lcPutJob) << "PUT of" << reply()->request().url().toString() << "FINISHED WITH STATUS"
-                         << replyStatusString()
-                         << reply()->attribute(QNetworkRequest::HttpStatusCodeAttribute)
-                         << reply()->attribute(QNetworkRequest::HttpReasonPhraseAttribute);
-
-        emit finishedSignal();
-        return true;
-    }
+    virtual bool finished() Q_DECL_OVERRIDE;
 
     QIODevice *device()
     {
@@ -151,8 +147,8 @@ signals:
 /**
  * @brief This job implements the asynchronous PUT
  *
- * If the server replies to a PUT with a OC-Finish-Poll url, we will query this url until the server
- * replies with an etag. https://github.com/owncloud/core/issues/12097
+ * If the server replies to a PUT with a OC-JobStatus-Location path, we will query this url until the server
+ * replies with an etag.
  * @ingroup libsync
  */
 class PollJob : public AbstractNetworkJob
@@ -291,9 +287,9 @@ protected:
      *
      * See #6527, enterprise#2480
      */
-    static void adjustLastJobTimeout(AbstractNetworkJob *job, quint64 fileSize);
+    static void adjustLastJobTimeout(AbstractNetworkJob *job, qint64 fileSize);
 
-    // Bases headers that need to be sent with every chunk
+    /** Bases headers that need to be sent on the PUT, or in the MOVE for chunking-ng */
     QMap<QByteArray, QByteArray> headers();
 };
 
@@ -321,9 +317,9 @@ private:
      */
     int _currentChunk = 0;
     int _chunkCount = 0; /// Total number of chunks for this file
-    int _transferId = 0; /// transfer id (part of the url)
+    uint _transferId = 0; /// transfer id (part of the url)
 
-    quint64 chunkSize() const {
+    qint64 chunkSize() const {
         // Old chunking does not use dynamic chunking algorithm, and does not adjusts the chunk size respectively,
         // thus this value should be used as the one classifing item to be chunked
         return propagator()->syncOptions()._initialChunkSize;
@@ -354,38 +350,77 @@ class PropagateUploadFileNG : public PropagateUploadFileCommon
 {
     Q_OBJECT
 private:
-    quint64 _sent = 0; /// amount of data (bytes) that was already sent
+    /** Amount of data that was already sent in bytes.
+     *
+     * If this job is resuming an upload, this number includes bytes that were
+     * sent in previous jobs.
+     */
+    qint64 _sent = 0;
+
+    /** Amount of data that needs to be sent to the server in bytes.
+     *
+     * For normal uploads this will be the file size, for zsync uploads it can
+     * be less.
+     *
+     * This value is intended to be comparable to _sent: it's always the total
+     * amount of data that needs to be present at the server to finish the upload -
+     * regardless of whether previous jobs have already sent something.
+     */
+    qint64 _bytesToUpload;
+
     uint _transferId = 0; /// transfer id (part of the url)
-    int _currentChunk = 0; /// Id of the next chunk that will be sent
-    quint64 _currentChunkSize = 0; /// current chunk size
-    bool _removeJobError = false; /// If not null, there was an error removing the job
+    qint64 _currentChunkOffset = 0; /// byte offset of the next chunk data that will be sent
+    qint64 _currentChunkSize = 0; /// current chunk size
+    bool _removeJobError = false; /// if not null, there was an error removing the job
+    bool _zsyncSupported = false; /// if zsync is supported this will be set to true
+    bool _isZsyncMetadataUploadRunning = false; // flag to ensure that zsync metadata upload is complete before job is
 
     // Map chunk number with its size  from the PROPFIND on resume.
     // (Only used from slotPropfindIterate/slotPropfindFinished because the LsColJob use signals to report data.)
     struct ServerChunkInfo
     {
-        quint64 size;
+        qint64 size;
         QString originalName;
     };
-    QMap<int, ServerChunkInfo> _serverChunks;
+    QMap<qint64, ServerChunkInfo> _serverChunks;
+
+    // Vector with expected PUT ranges.
+    struct UploadRangeInfo
+    {
+        qint64 start;
+        qint64 size;
+        qint64 end() const { return start + size; }
+    };
+    QVector<UploadRangeInfo> _rangesToUpload;
 
     /**
      * Return the URL of a chunk.
-     * If chunk == -1, returns the URL of the parent folder containing the chunks
+     * If chunkOffset == -1, returns the URL of the parent folder containing the chunks
      */
-    QUrl chunkUrl(int chunk = -1);
+    QUrl chunkUrl(qint64 chunkOffset = -1);
+
+    /**
+     * Finds the range starting at 'start' in _rangesToUpload and removes the first
+     * 'size' bytes from it. If it becomes empty, remove the range.
+     *
+     * Retuns false if no matching range was found.
+     */
+    bool markRangeAsDone(qint64 start, qint64 size);
 
 public:
     PropagateUploadFileNG(OwncloudPropagator *propagator, const SyncFileItemPtr &item)
         : PropagateUploadFileCommon(propagator, item)
+        , _bytesToUpload(item->_size)
     {
     }
 
     void doStartUpload() Q_DECL_OVERRIDE;
 
 private:
+    void doStartUploadNext();
     void startNewUpload();
     void startNextChunk();
+    void doFinalMove();
 public slots:
     void abort(AbortType abortType) Q_DECL_OVERRIDE;
 private slots:
@@ -395,6 +430,12 @@ private slots:
     void slotDeleteJobFinished();
     void slotMkColFinished(QNetworkReply::NetworkError);
     void slotPutFinished();
+    void slotZsyncGetMetaFinished(QNetworkReply *reply);
+    void slotZsyncSeedFinished(void *zs);
+    void slotZsyncSeedFailed(const QString &errorString);
+    void slotZsyncGenerationFinished(const QString &fileName);
+    void slotZsyncGenerationFailed(const QString &errorString);
+    void slotZsyncMetadataUploadFinished();
     void slotMoveJobFinished();
     void slotUploadProgress(qint64, qint64);
 };
