@@ -45,7 +45,6 @@ QByteArray localFileIdFromFullId(const QByteArray &id)
 }
 
 /**
- * Code inspired from Qt5's QDir::removeRecursively
  * The code will update the database in case of error.
  * If everything goes well (no error, returns true), the caller is responsible for removing the entries
  * in the database.  But in case of error, we need to remove the entries from the database of the files
@@ -55,55 +54,34 @@ QByteArray localFileIdFromFullId(const QByteArray &id)
  */
 bool PropagateLocalRemove::removeRecursively(const QString &path)
 {
-    bool success = true;
-    QString absolute = propagator()->_localDir + _item->_file + path;
-    QDirIterator di(absolute, QDir::AllEntries | QDir::Hidden | QDir::System | QDir::NoDotAndDotDot);
+    auto folderDir = propagator()->_localDir;
+    QString absolute = folderDir + _item->_file + path;
+    QStringList errors;
+    QList<QPair<QString, bool>> deleted;
+    bool success = FileSystem::removeRecursively(
+        absolute,
+        [&deleted](const QString &path, bool isDir) {
+            // by prepending, a folder deletion may be followed by content deletions
+            deleted.prepend(qMakePair(path, isDir));
+        },
+        &errors);
 
-    QVector<QPair<QString, bool>> deleted;
+    if (!success) {
+        // We need to delete the entries from the database now from the deleted vector.
+        // Do it while avoiding redundant delete calls to the journal.
+        QString deletedDir;
+        foreach (const auto &it, deleted) {
+            if (!it.first.startsWith(folderDir))
+                continue;
+            if (!deletedDir.isEmpty() && it.first.startsWith(deletedDir))
+                continue;
+            if (it.second) {
+                deletedDir = it.first;
+            }
+            propagator()->_journal->deleteFileRecord(it.first.mid(folderDir.size()), it.second);
+        }
 
-    while (di.hasNext()) {
-        di.next();
-        const QFileInfo &fi = di.fileInfo();
-        bool ok;
-        // The use of isSymLink here is okay:
-        // we never want to go into this branch for .lnk files
-        bool isDir = fi.isDir() && !fi.isSymLink() && !FileSystem::isJunction(fi.absoluteFilePath());
-        if (isDir) {
-            ok = removeRecursively(path + QLatin1Char('/') + di.fileName()); // recursive
-        } else {
-            QString removeError;
-            ok = FileSystem::remove(di.filePath(), &removeError);
-            if (!ok) {
-                _error += PropagateLocalRemove::tr("Error removing '%1': %2;").arg(QDir::toNativeSeparators(di.filePath()), removeError) + " ";
-                qCWarning(lcPropagateLocalRemove) << "Error removing " << di.filePath() << ':' << removeError;
-            }
-        }
-        if (success && !ok) {
-            // We need to delete the entries from the database now from the deleted vector
-            foreach (const auto &it, deleted) {
-                propagator()->_journal->deleteFileRecord(_item->_originalFile + path + QLatin1Char('/') + it.first,
-                    it.second);
-            }
-            success = false;
-            deleted.clear();
-        }
-        if (success) {
-            deleted.append(qMakePair(di.fileName(), isDir));
-        }
-        if (!success && ok) {
-            // This succeeded, so we need to delete it from the database now because the caller won't
-            propagator()->_journal->deleteFileRecord(_item->_originalFile + path + QLatin1Char('/') + di.fileName(),
-                isDir);
-        }
-    }
-    if (success) {
-        success = QDir().rmdir(absolute);
-        if (!success) {
-            _error += PropagateLocalRemove::tr("Could not remove folder '%1'")
-                          .arg(QDir::toNativeSeparators(absolute))
-                + " ";
-            qCWarning(lcPropagateLocalRemove) << "Error removing folder" << absolute;
-        }
+        _error = errors.join(", ");
     }
     return success;
 }
@@ -196,9 +174,9 @@ void PropagateLocalMkdir::start()
     // Adding an entry with a dummy etag to the database still makes sense here
     // so the database is aware that this folder exists even if the sync is aborted
     // before the correct etag is stored.
-    SyncJournalFileRecord record = _item->toSyncJournalFileRecordWithInode(newDirStr);
-    record._etag = "_invalid_";
-    if (!propagator()->_journal->setFileRecord(record)) {
+    SyncFileItem newItem(*_item);
+    newItem._etag = "_invalid_";
+    if (!propagator()->updateMetadata(newItem)) {
         done(SyncFileItem::FatalError, tr("Error writing metadata to the database"));
         return;
     }
@@ -220,7 +198,7 @@ void PropagateLocalRename::start()
     if (propagator()->_abortRequested.fetchAndAddRelaxed(0))
         return;
 
-    QString existingFile = propagator()->getFilePath(_item->_file);
+    QString existingFile = propagator()->getFilePath(propagator()->adjustRenamedPath(_item->_file));
     QString targetFile = propagator()->getFilePath(_item->_renameTarget);
 
     // if the file is a file underneath a moved dir, the _item->file is equal
@@ -256,27 +234,33 @@ void PropagateLocalRename::start()
     propagator()->_journal->getFileRecord(_item->_originalFile, &oldRecord);
     propagator()->_journal->deleteFileRecord(_item->_originalFile);
 
-    // store the rename file name in the item.
-    const auto oldFile = _item->_file;
-    _item->_file = _item->_renameTarget;
+    auto &vfs = propagator()->syncOptions()._vfs;
+    auto pinState = vfs->pinState(_item->_originalFile);
+    vfs->setPinState(_item->_originalFile, PinState::Inherited);
 
-    SyncJournalFileRecord record = _item->toSyncJournalFileRecordWithInode(targetFile);
-    record._path = _item->_renameTarget.toUtf8();
-    if (oldRecord.isValid()) {
-        record._checksumHeader = oldRecord._checksumHeader;
-    }
+    const auto oldFile = _item->_file;
 
     if (!_item->isDirectory()) { // Directories are saved at the end
-        if (!propagator()->_journal->setFileRecord(record)) {
+        SyncFileItem newItem(*_item);
+        if (oldRecord.isValid()) {
+            newItem._checksumHeader = oldRecord._checksumHeader;
+        }
+        if (!propagator()->updateMetadata(newItem)) {
             done(SyncFileItem::FatalError, tr("Error writing metadata to the database"));
             return;
         }
     } else {
+        propagator()->_renamedDirectories.insert(oldFile, _item->_renameTarget);
         if (!PropagateRemoteMove::adjustSelectiveSync(propagator()->_journal, oldFile, _item->_renameTarget)) {
             done(SyncFileItem::FatalError, tr("Error writing metadata to the database"));
             return;
         }
     }
+    if (pinState && *pinState != PinState::Inherited
+        && !vfs->setPinState(_item->_renameTarget, *pinState)) {
+        done(SyncFileItem::NormalError, tr("Error setting pin state"));
+        return;
+    }
 
     propagator()->_journal->commit("localRename");
 
