@@ -16,10 +16,14 @@
 #include "config.h"
 #include "common/utility.h"
 #include "version.h"
+#include "configfile.h"
+#include "common/vfs.h"
 
 #include <QtCore>
 #ifndef TOKEN_AUTH_ONLY
 #include <QtGui>
+#include <QStyle>
+#include <QApplication>
 #endif
 #include <QSslSocket>
 
@@ -105,12 +109,16 @@ QString Theme::version() const
     return MIRALL_VERSION_STRING;
 }
 
+QString Theme::configFileName() const
+{
+    return QStringLiteral(APPLICATION_EXECUTABLE ".cfg");
+}
+
 #ifndef TOKEN_AUTH_ONLY
 
-QIcon Theme::trayFolderIcon(const QString &backend) const
+QIcon Theme::applicationIcon() const
 {
-    Q_UNUSED(backend)
-    return applicationIcon();
+    return themeIcon(QStringLiteral(APPLICATION_ICON_NAME "-icon"));
 }
 
 /*
@@ -135,7 +143,7 @@ QIcon Theme::themeIcon(const QString &name, bool sysTray, bool sysTrayMenuVisibl
         }
 
         QList<int> sizes;
-        sizes << 16 << 22 << 32 << 48 << 64 << 128 << 256;
+        sizes << 16 << 22 << 32 << 48 << 64 << 128 << 256 << 512 << 1024;
         foreach (int size, sizes) {
             QString pixmapName = QString::fromLatin1(":/client/theme/%1/%2-%3.png").arg(flavor).arg(name).arg(size);
             if (QFile::exists(pixmapName)) {
@@ -217,9 +225,24 @@ QString Theme::defaultServerFolder() const
     return QLatin1String("/");
 }
 
+QString Theme::helpUrl() const
+{
+    return QString::fromLatin1("https://doc.owncloud.org/desktop/%1.%2/").arg(MIRALL_VERSION_MAJOR).arg(MIRALL_VERSION_MINOR);
+}
+
+QString Theme::conflictHelpUrl() const
+{
+    auto baseUrl = helpUrl();
+    if (baseUrl.isEmpty())
+        return QString();
+    if (!baseUrl.endsWith('/'))
+        baseUrl.append('/');
+    return baseUrl + QStringLiteral("conflicts.html");
+}
+
 QString Theme::overrideServerUrl() const
 {
-    return QString::null;
+    return QString();
 }
 
 QString Theme::forceConfigAuthType() const
@@ -303,7 +326,7 @@ QString Theme::gitSHA1() const
                     .arg(gitSha1.left(6))
                     .arg(__DATE__)
                     .arg(__TIME__)
-                    .arg(QString::fromAscii(qVersion()))
+                    .arg(qVersion())
                     .arg(QSslSocket::sslLibraryVersionString());
 #endif
     return devString;
@@ -311,21 +334,35 @@ QString Theme::gitSHA1() const
 
 QString Theme::about() const
 {
-    QString re;
-    re = tr("<p>Version %1. For more information please visit <a href='%2'>%3</a>.</p>")
-             .arg(MIRALL_VERSION_STRING)
-             .arg("http://" MIRALL_STRINGIFY(APPLICATION_DOMAIN))
-             .arg(MIRALL_STRINGIFY(APPLICATION_DOMAIN));
+    QString vendor = APPLICATION_VENDOR;
+    // Ideally, the vendor should be "ownCloud GmbH", but it cannot be changed without
+    // changing the location of the settings and other registery keys.
+    if (vendor == "ownCloud") vendor = QLatin1String("ownCloud GmbH");
 
-    re += tr("<p>Copyright ownCloud GmbH</p>");
-    re += tr("<p>Distributed by %1 and licensed under the GNU General Public License (GPL) Version 2.0.<br/>"
-             "%2 and the %2 logo are registered trademarks of %1 in the "
-             "United States, other countries, or both.</p>")
-              .arg(APPLICATION_VENDOR)
-              .arg(APPLICATION_NAME);
+    QString devString;
+    devString = tr("<p>Version %2. For more information visit <a href=\"%3\">https://%4</a></p>"
+                       "<p>For known issues and help, please visit: <a href=\"https://central.owncloud.org/c/desktop-client\">https://central.owncloud.org</a></p>"
+                       "<p><small>By Klaas Freitag, Daniel Molkentin, Olivier Goffart, Markus Götz, "
+                       " Jan-Christoph Borchardt, Thomas Müller, Dominik Schmidt, Hannah von Reth, and others.</small></p>")
+                    .arg(Utility::escape(MIRALL_VERSION_STRING),
+                        Utility::escape("https://" MIRALL_STRINGIFY(APPLICATION_DOMAIN)),
+                        Utility::escape(MIRALL_STRINGIFY(APPLICATION_DOMAIN)));
+    devString += tr("<p>Copyright ownCloud GmbH</p>");
+    devString += tr("<p>Distributed by %1 and licensed under the GNU General Public License (GPL) Version 2.0.<br/>"
+                    "%2 and the %2 logo are registered trademarks of %1 in the "
+                    "United States, other countries, or both.</p>")
+               .arg(Utility::escape(vendor), Utility::escape(APPLICATION_NAME));
 
-    re += gitSHA1();
-    return re;
+    devString += gitSHA1();
+    devString += QString("<p><small>Using virtual files plugin: %1</small></p>")
+        .arg(Vfs::modeToString(bestAvailableVfsMode()));
+
+    return devString;
+}
+
+bool Theme::aboutShowCopyright() const
+{
+    return true;
 }
 
 #ifndef TOKEN_AUTH_ONLY
@@ -528,5 +565,19 @@ QString Theme::versionSwitchOutput() const
     return helpText;
 }
 
+bool Theme::showVirtualFilesOption() const
+{
+    return enableExperimentalFeatures();
+}
+
+bool Theme::enableExperimentalFeatures() const
+{
+    return ConfigFile().showExperimentalOptions();
+}
+
+bool Theme::noUnauthedRequests() const
+{
+    return false;
+}
 
 } // end namespace client
