@@ -455,7 +455,7 @@ sub traverse( $$;$ )
 
 	$isHere = 1 if( $acceptConflicts && !$isHere && $f =~ /conflicted copy/ );
 	$isHere = 1 if( $f =~ /\.csync/ );
-	$isHere = 1 if( $f =~ /\._sync_/ );
+	$isHere = 1 if( $f =~ /\.sync_/ );
 	assert( $isHere, "Filename local, but not remote: $f" );
     }
 
