@@ -12,6 +12,10 @@
 #include "filesystem.h"
 #include "syncengine.h"
 #include "common/syncjournaldb.h"
+#include "common/syncjournalfilerecord.h"
+#include "common/vfs.h"
+#include "csync_exclude.h"
+#include <cstring>
 
 #include <QDir>
 #include <QNetworkReply>
@@ -41,7 +45,7 @@ inline QString getFilePathFromUrl(const QUrl &url) {
 
 
 inline QString generateEtag() {
-    return QString::number(QDateTime::currentDateTimeUtc().toMSecsSinceEpoch(), 16);
+    return QString::number(QDateTime::currentDateTimeUtc().toMSecsSinceEpoch(), 16) + QByteArray::number(qrand(), 16);
 }
 inline QByteArray generateFileId() {
     return QByteArray::number(qrand(), 16);
@@ -56,7 +60,8 @@ public:
     PathComponents parentDirComponents() const {
         return PathComponents{mid(0, size() - 1)};
     }
-    PathComponents subComponents() const { return PathComponents{mid(1)}; }
+    PathComponents subComponents() const& { return PathComponents{mid(1)}; }
+    PathComponents subComponents() && { removeFirst(); return std::move(*this); }
     QString pathRoot() const { return first(); }
     QString fileName() const { return last(); }
 };
@@ -68,7 +73,8 @@ public:
     virtual void remove(const QString &relativePath) = 0;
     virtual void insert(const QString &relativePath, qint64 size = 64, char contentChar = 'W') = 0;
     virtual void setContents(const QString &relativePath, char contentChar) = 0;
-    virtual void appendByte(const QString &relativePath) = 0;
+    virtual void appendByte(const QString &relativePath, char contentChar = 0) = 0;
+    virtual void modifyByte(const QString &relativePath, quint64 offset, char contentChar) = 0;
     virtual void mkdir(const QString &relativePath) = 0;
     virtual void rename(const QString &relativePath, const QString &relativeDestinationDirectory) = 0;
     virtual void setModTime(const QString &relativePath, const QDateTime &modTime) = 0;
@@ -107,14 +113,29 @@ public:
         file.open(QFile::WriteOnly);
         file.write(QByteArray{}.fill(contentChar, size));
     }
-    void appendByte(const QString &relativePath) override {
+    void appendByte(const QString &relativePath, char contentChar) override
+    {
         QFile file{_rootDir.filePath(relativePath)};
         QVERIFY(file.exists());
         file.open(QFile::ReadWrite);
-        QByteArray contents = file.read(1);
+        QByteArray contents;
+        if (contentChar)
+            contents += contentChar;
+        else
+            contents = file.read(1);
         file.seek(file.size());
         file.write(contents);
     }
+    void modifyByte(const QString &relativePath, quint64 offset, char contentChar) override
+    {
+        QFile file{ _rootDir.filePath(relativePath) };
+        QVERIFY(file.exists());
+        file.open(QFile::ReadWrite);
+        file.seek(offset);
+        file.write(&contentChar, 1);
+        file.close();
+    }
+
     void mkdir(const QString &relativePath) override {
         _rootDir.mkpath(relativePath);
     }
@@ -190,12 +211,23 @@ public:
         file->contentChar = contentChar;
     }
 
-    void appendByte(const QString &relativePath) override {
+    void appendByte(const QString &relativePath, char contentChar = 0) override
+    {
+        Q_UNUSED(contentChar);
         FileInfo *file = findInvalidatingEtags(relativePath);
         Q_ASSERT(file);
         file->size += 1;
     }
 
+    void modifyByte(const QString &relativePath, quint64 offset, char contentChar) override
+    {
+        Q_UNUSED(offset);
+        Q_UNUSED(contentChar);
+        FileInfo *file = findInvalidatingEtags(relativePath);
+        Q_ASSERT(file);
+        Q_ASSERT(!"unimplemented");
+    }
+
     void mkdir(const QString &relativePath) override {
         createDir(relativePath);
     }
@@ -221,19 +253,21 @@ public:
         file->lastModified = modTime;
     }
 
-    FileInfo *find(const PathComponents &pathComponents, const bool invalidateEtags = false) {
+    FileInfo *find(PathComponents pathComponents, const bool invalidateEtags = false) {
         if (pathComponents.isEmpty()) {
-            if (invalidateEtags)
+            if (invalidateEtags) {
                 etag = generateEtag();
+            }
             return this;
         }
         QString childName = pathComponents.pathRoot();
         auto it = children.find(childName);
         if (it != children.end()) {
-            auto file = it->find(pathComponents.subComponents(), invalidateEtags);
-            if (file && invalidateEtags)
+            auto file = it->find(std::move(pathComponents).subComponents(), invalidateEtags);
+            if (file && invalidateEtags) {
                 // Update parents on the way back
-                etag = file->etag;
+                etag = generateEtag();
+            }
             return file;
         }
         return 0;
@@ -273,6 +307,10 @@ public:
             && children == other.children;
     }
 
+    bool operator!=(const FileInfo &other) const {
+        return !operator==(other);
+    }
+
     QString path() const {
         return (parentPath.isEmpty() ? QString() : (parentPath + '/')) + name;
     }
@@ -290,7 +328,7 @@ public:
     bool isDir = true;
     bool isShared = false;
     OCC::RemotePermissions permissions; // When uset, defaults to everything
-    QDateTime lastModified = QDateTime::currentDateTime().addDays(-7);
+    QDateTime lastModified = QDateTime::currentDateTimeUtc().addDays(-7);
     QString etag = generateEtag();
     QByteArray fileId = generateFileId();
     QByteArray checksums;
@@ -302,9 +340,8 @@ public:
     QMap<QString, FileInfo> children;
     QString parentPath;
 
-private:
-    FileInfo *findInvalidatingEtags(const PathComponents &pathComponents) {
-        return find(pathComponents, true);
+    FileInfo *findInvalidatingEtags(PathComponents pathComponents) {
+        return find(std::move(pathComponents), true);
     }
 
     friend inline QDebug operator<<(QDebug dbg, const FileInfo& fi) {
@@ -347,7 +384,7 @@ public:
         auto writeFileResponse = [&](const FileInfo &fileInfo) {
             xml.writeStartElement(davUri, QStringLiteral("response"));
 
-            xml.writeTextElement(davUri, QStringLiteral("href"), prefix + fileInfo.path());
+            xml.writeTextElement(davUri, QStringLiteral("href"), prefix + QUrl::toPercentEncoding(fileInfo.path(), "/"));
             xml.writeStartElement(davUri, QStringLiteral("propstat"));
             xml.writeStartElement(davUri, QStringLiteral("prop"));
 
@@ -362,12 +399,13 @@ public:
             auto stringDate = QLocale::c().toString(gmtDate, "ddd, dd MMM yyyy HH:mm:ss 'GMT'");
             xml.writeTextElement(davUri, QStringLiteral("getlastmodified"), stringDate);
             xml.writeTextElement(davUri, QStringLiteral("getcontentlength"), QString::number(fileInfo.size));
-            xml.writeTextElement(davUri, QStringLiteral("getetag"), fileInfo.etag);
+            xml.writeTextElement(davUri, QStringLiteral("getetag"), QStringLiteral("\"%1\"").arg(fileInfo.etag));
             xml.writeTextElement(ocUri, QStringLiteral("permissions"), !fileInfo.permissions.isNull()
                 ? QString(fileInfo.permissions.toString())
                 : fileInfo.isShared ? QStringLiteral("SRDNVCKW") : QStringLiteral("RDNVCKW"));
             xml.writeTextElement(ocUri, QStringLiteral("id"), fileInfo.fileId);
             xml.writeTextElement(ocUri, QStringLiteral("checksums"), fileInfo.checksums);
+            xml.writeTextElement(ocUri, QStringLiteral("zsync"), QStringLiteral("true"));
             buffer.write(fileInfo.extraDavProperties);
             xml.writeEndElement(); // prop
             xml.writeTextElement(davUri, QStringLiteral("status"), "HTTP/1.1 200 OK");
@@ -424,24 +462,25 @@ public:
         setUrl(request.url());
         setOperation(op);
         open(QIODevice::ReadOnly);
+        fileInfo = perform(remoteRootFileInfo, request, putPayload);
+        QMetaObject::invokeMethod(this, "respond", Qt::QueuedConnection);
+    }
 
+    static FileInfo *perform(FileInfo &remoteRootFileInfo, const QNetworkRequest &request, const QByteArray &putPayload)
+    {
         QString fileName = getFilePathFromUrl(request.url());
         Q_ASSERT(!fileName.isEmpty());
-        if ((fileInfo = remoteRootFileInfo.find(fileName))) {
+        FileInfo *fileInfo = remoteRootFileInfo.find(fileName);
+        if (fileInfo) {
             fileInfo->size = putPayload.size();
             fileInfo->contentChar = putPayload.at(0);
         } else {
             // Assume that the file is filled with the same character
             fileInfo = remoteRootFileInfo.create(fileName, putPayload.size(), putPayload.at(0));
         }
-
-        if (!fileInfo) {
-            abort();
-            return;
-        }
         fileInfo->lastModified = OCC::Utility::qDateTimeFromTime_t(request.rawHeader("X-OC-Mtime").toLongLong());
         remoteRootFileInfo.find(fileName, /*invalidate_etags=*/true);
-        QMetaObject::invokeMethod(this, "respond", Qt::QueuedConnection);
+        return fileInfo;
     }
 
     Q_INVOKABLE virtual void respond()
@@ -573,6 +612,8 @@ public:
         QString fileName = getFilePathFromUrl(request.url());
         Q_ASSERT(!fileName.isEmpty());
         fileInfo = remoteRootFileInfo.find(fileName);
+        if (!fileInfo)
+            qWarning() << "Could not find file" << fileName << "on the remote";
         QMetaObject::invokeMethod(this, "respond", Qt::QueuedConnection);
     }
 
@@ -597,6 +638,7 @@ public:
     }
 
     void abort() override {
+        setError(OperationCanceledError, "Operation Canceled");
         aborted = true;
     }
     qint64 bytesAvailable() const override {
@@ -616,6 +658,83 @@ public:
     using QNetworkReply::setRawHeader;
 };
 
+class FakeGetWithDataReply : public QNetworkReply
+{
+    Q_OBJECT
+public:
+    const FileInfo *fileInfo;
+    QByteArray payload;
+    quint64 offset = 0;
+    bool aborted = false;
+
+    FakeGetWithDataReply(FileInfo &remoteRootFileInfo, const QByteArray &data, QNetworkAccessManager::Operation op, const QNetworkRequest &request, QObject *parent)
+        : QNetworkReply{ parent }
+    {
+        setRequest(request);
+        setUrl(request.url());
+        setOperation(op);
+        open(QIODevice::ReadOnly);
+
+        Q_ASSERT(!data.isEmpty());
+        payload = data;
+        QString fileName = getFilePathFromUrl(request.url());
+        Q_ASSERT(!fileName.isEmpty());
+        fileInfo = remoteRootFileInfo.find(fileName);
+        QMetaObject::invokeMethod(this, "respond", Qt::QueuedConnection);
+
+        if (request.hasRawHeader("Range")) {
+            QByteArray range = request.rawHeader("Range");
+            quint64 start, end;
+            const char *r = range.constData();
+            int res = sscanf(r, "bytes=%llu-%llu", &start, &end);
+            if (res == 2) {
+                payload = payload.mid(start, end - start + 1);
+            }
+        }
+    }
+
+    Q_INVOKABLE void respond()
+    {
+        if (aborted) {
+            setError(OperationCanceledError, "Operation Canceled");
+            emit metaDataChanged();
+            emit finished();
+            return;
+        }
+        setHeader(QNetworkRequest::ContentLengthHeader, payload.size());
+        setAttribute(QNetworkRequest::HttpStatusCodeAttribute, 200);
+        setRawHeader("OC-ETag", fileInfo->etag.toLatin1());
+        setRawHeader("ETag", fileInfo->etag.toLatin1());
+        setRawHeader("OC-FileId", fileInfo->fileId);
+        emit metaDataChanged();
+        if (bytesAvailable())
+            emit readyRead();
+        emit finished();
+    }
+
+    void abort() override
+    {
+        setError(OperationCanceledError, "Operation Canceled");
+        aborted = true;
+    }
+    qint64 bytesAvailable() const override
+    {
+        if (aborted)
+            return 0;
+        return payload.size() - offset + QIODevice::bytesAvailable();
+    }
+
+    qint64 readData(char *data, qint64 maxlen) override
+    {
+        qint64 len = std::min(payload.size() - offset, quint64(maxlen));
+        std::memcpy(data, payload.constData() + offset, len);
+        offset += len;
+        return len;
+    }
+
+    // useful to be public for testing
+    using QNetworkReply::setRawHeader;
+};
 
 class FakeChunkMoveReply : public QNetworkReply
 {
@@ -631,43 +750,77 @@ public:
         setUrl(request.url());
         setOperation(op);
         open(QIODevice::ReadOnly);
+        fileInfo = perform(uploadsFileInfo, remoteRootFileInfo, request);
+        if (!fileInfo) {
+            QTimer::singleShot(0, this, &FakeChunkMoveReply::respondPreconditionFailed);
+        } else {
+            QTimer::singleShot(0, this, &FakeChunkMoveReply::respond);
+        }
+    }
 
+    static FileInfo *perform(FileInfo &uploadsFileInfo, FileInfo &remoteRootFileInfo, const QNetworkRequest &request)
+    {
         QString source = getFilePathFromUrl(request.url());
+        bool zsync = false;
         Q_ASSERT(!source.isEmpty());
-        Q_ASSERT(source.endsWith("/.file"));
-        source = source.left(source.length() - qstrlen("/.file"));
+        Q_ASSERT(source.endsWith("/.file") || source.endsWith("/.file.zsync"));
+        if (source.endsWith("/.file"))
+            source = source.left(source.length() - qstrlen("/.file"));
+        if (source.endsWith("/.file.zsync")) {
+            source = source.left(source.length() - qstrlen("/.file.zsync"));
+            zsync = true;
+        }
         auto sourceFolder = uploadsFileInfo.find(source);
         Q_ASSERT(sourceFolder);
         Q_ASSERT(sourceFolder->isDir);
         int count = 0;
-        int size = 0;
+        qlonglong size = 0;
+        qlonglong prev = 0;
         char payload = '\0';
 
-        do {
-            QString chunkName = QString::number(count).rightJustified(8, '0');
-            if (!sourceFolder->children.contains(chunkName))
-                break;
-            auto &x = sourceFolder->children[chunkName];
-            Q_ASSERT(!x.isDir);
-            Q_ASSERT(x.size > 0); // There should not be empty chunks
-            size += x.size;
-            Q_ASSERT(!payload || payload == x.contentChar);
-            payload = x.contentChar;
-            ++count;
-        } while(true);
-
-        Q_ASSERT(count > 1); // There should be at least two chunks, otherwise why would we use chunking?
-        QCOMPARE(sourceFolder->children.count(), count); // There should not be holes or extra files
-
         QString fileName = getFilePathFromUrl(QUrl::fromEncoded(request.rawHeader("Destination")));
         Q_ASSERT(!fileName.isEmpty());
 
-        if ((fileInfo = remoteRootFileInfo.find(fileName))) {
-            QVERIFY(request.hasRawHeader("If")); // The client should put this header
-            if (request.rawHeader("If") != QByteArray("<" + request.rawHeader("Destination") +
-                                                "> ([\"" + fileInfo->etag.toLatin1() + "\"])")) {
-                QMetaObject::invokeMethod(this, "respondPreconditionFailed", Qt::QueuedConnection);
-                return;
+        // Ignore .zsync metadata
+        if (sourceFolder->children.contains(".zsync"))
+            sourceFolder->children.remove(".zsync");
+
+        // Compute the size and content from the chunks if possible
+        for (auto chunkName : sourceFolder->children.keys()) {
+            auto &x = sourceFolder->children[chunkName];
+            if (!zsync && chunkName.toLongLong() != prev)
+                break;
+            Q_ASSERT(!x.isDir);
+            Q_ASSERT(x.size > 0); // There should not be empty chunks
+            size += x.size;
+            Q_ASSERT(!payload || payload == x.contentChar || !"For zsync all chunks must start with the same character");
+            payload = x.contentChar;
+            ++count;
+            prev = chunkName.toLongLong() + x.size;
+        }
+        Q_ASSERT(sourceFolder->children.count() == count); // There should not be holes or extra files
+
+        // For zsync, get the size from the header, and allow no-chunk uploads (shrinking files)
+        if (zsync) {
+            size = request.rawHeader("OC-Total-File-Length").toLongLong();
+            if (count == 0) {
+                if (auto info = remoteRootFileInfo.find(fileName))
+                    payload = info->contentChar;
+            }
+        }
+
+        // NOTE: This does not actually assemble the file data from the chunks!
+        FileInfo *fileInfo = remoteRootFileInfo.find(fileName);
+        if (fileInfo) {
+            // The client should put this header
+            Q_ASSERT(request.hasRawHeader("If"));
+
+            // And it should condition on the destination file
+            auto start = QByteArray("<" + request.rawHeader("Destination") + ">");
+            Q_ASSERT(request.rawHeader("If").startsWith(start));
+
+            if (request.rawHeader("If") != start + " ([\"" + fileInfo->etag.toLatin1() + "\"])") {
+                return nullptr;
             }
             fileInfo->size = size;
             fileInfo->contentChar = payload;
@@ -676,15 +829,10 @@ public:
             // Assume that the file is filled with the same character
             fileInfo = remoteRootFileInfo.create(fileName, size, payload);
         }
-
-        if (!fileInfo) {
-            abort();
-            return;
-        }
         fileInfo->lastModified = OCC::Utility::qDateTimeFromTime_t(request.rawHeader("X-OC-Mtime").toLongLong());
         remoteRootFileInfo.find(fileName, /*invalidate_etags=*/true);
 
-        QTimer::singleShot(0, this, &FakeChunkMoveReply::respond);
+        return fileInfo;
     }
 
     Q_INVOKABLE virtual void respond()
@@ -713,6 +861,140 @@ public:
     qint64 readData(char *, qint64) override { return 0; }
 };
 
+class FakeChunkZsyncMoveReply : public QNetworkReply
+{
+    Q_OBJECT
+    FileInfo *fileInfo;
+
+public:
+    FakeChunkZsyncMoveReply(FileInfo &uploadsFileInfo, FileInfo &remoteRootFileInfo,
+        QNetworkAccessManager::Operation op, const QNetworkRequest &request,
+        quint64 delayMs, QVector<quint64> &mods, QObject *parent)
+        : QNetworkReply{ parent }
+    {
+        setRequest(request);
+        setUrl(request.url());
+        setOperation(op);
+        open(QIODevice::ReadOnly);
+
+        Q_ASSERT(!mods.isEmpty());
+
+        QString source = getFilePathFromUrl(request.url());
+        Q_ASSERT(!source.isEmpty());
+        Q_ASSERT(source.endsWith("/.file.zsync"));
+        source = source.left(source.length() - qstrlen("/.file.zsync"));
+        auto sourceFolder = uploadsFileInfo.find(source);
+        Q_ASSERT(sourceFolder);
+        Q_ASSERT(sourceFolder->isDir);
+        int count = 0;
+
+        // Ignore .zsync metadata
+        if (sourceFolder->children.contains(".zsync"))
+            sourceFolder->children.remove(".zsync");
+
+        for (auto chunkName : sourceFolder->children.keys()) {
+            auto &x = sourceFolder->children[chunkName];
+            Q_ASSERT(!x.isDir);
+            Q_ASSERT(x.size > 0); // There should not be empty chunks
+            quint64 start = quint64(chunkName.toLongLong());
+            auto it = mods.begin();
+            while (it != mods.end()) {
+                if (*it >= start && *it < start + x.size) {
+                    ++count;
+                    mods.erase(it);
+                } else
+                    ++it;
+            }
+        }
+
+        Q_ASSERT(count > 0); // There should be at least one chunk
+        Q_ASSERT(mods.isEmpty()); // All files should match a modification
+
+        QString fileName = getFilePathFromUrl(QUrl::fromEncoded(request.rawHeader("Destination")));
+        Q_ASSERT(!fileName.isEmpty());
+
+        fileInfo = remoteRootFileInfo.find(fileName);
+        Q_ASSERT(fileInfo);
+        if (!fileInfo) {
+            abort();
+            return;
+        }
+
+        QVERIFY(request.hasRawHeader("If")); // The client should put this header
+        if (request.rawHeader("If") != QByteArray("<" + request.rawHeader("Destination") + "> ([\"" + fileInfo->etag.toLatin1() + "\"])")) {
+            QMetaObject::invokeMethod(this, "respondPreconditionFailed", Qt::QueuedConnection);
+            return;
+        }
+
+        fileInfo->lastModified = OCC::Utility::qDateTimeFromTime_t(request.rawHeader("X-OC-Mtime").toLongLong());
+        remoteRootFileInfo.find(fileName, /*invalidate_etags=*/true);
+
+        QTimer::singleShot(delayMs, this, &FakeChunkZsyncMoveReply::respond);
+    }
+
+    Q_INVOKABLE void respond()
+    {
+        setAttribute(QNetworkRequest::HttpStatusCodeAttribute, 201);
+        setRawHeader("OC-ETag", fileInfo->etag.toLatin1());
+        setRawHeader("ETag", fileInfo->etag.toLatin1());
+        setRawHeader("OC-FileId", fileInfo->fileId);
+        emit metaDataChanged();
+        emit finished();
+    }
+
+    Q_INVOKABLE void respondPreconditionFailed()
+    {
+        setAttribute(QNetworkRequest::HttpStatusCodeAttribute, 412);
+        setError(InternalServerError, "Precondition Failed");
+        emit metaDataChanged();
+        emit finished();
+    }
+
+    void abort() override {}
+    qint64 readData(char *, qint64) override { return 0; }
+};
+
+class FakePayloadReply : public QNetworkReply
+{
+    Q_OBJECT
+public:
+    FakePayloadReply(QNetworkAccessManager::Operation op, const QNetworkRequest &request,
+        const QByteArray &body, QObject *parent)
+        : QNetworkReply{ parent }
+        , _body(body)
+    {
+        setRequest(request);
+        setUrl(request.url());
+        setOperation(op);
+        open(QIODevice::ReadOnly);
+        QTimer::singleShot(10, this, &FakePayloadReply::respond);
+    }
+
+    void respond()
+    {
+        setAttribute(QNetworkRequest::HttpStatusCodeAttribute, 200);
+        setHeader(QNetworkRequest::ContentLengthHeader, _body.size());
+        emit metaDataChanged();
+        emit readyRead();
+        setFinished(true);
+        emit finished();
+    }
+
+    void abort() override {}
+    qint64 readData(char *buf, qint64 max) override
+    {
+        max = qMin<qint64>(max, _body.size());
+        memcpy(buf, _body.constData(), max);
+        _body = _body.mid(max);
+        return max;
+    }
+    qint64 bytesAvailable() const override
+    {
+        return _body.size();
+    }
+    QByteArray _body;
+};
+
 
 class FakeErrorReply : public QNetworkReply
 {
@@ -720,23 +1002,27 @@ class FakeErrorReply : public QNetworkReply
 public:
     FakeErrorReply(QNetworkAccessManager::Operation op, const QNetworkRequest &request,
                    QObject *parent, int httpErrorCode, const QByteArray &body = QByteArray())
-    : QNetworkReply{parent}, _httpErrorCode(httpErrorCode), _body(body) {
+    : QNetworkReply{parent}, _body(body) {
         setRequest(request);
         setUrl(request.url());
         setOperation(op);
         open(QIODevice::ReadOnly);
+        setAttribute(QNetworkRequest::HttpStatusCodeAttribute, httpErrorCode);
+        setError(InternalServerError, "Internal Server Fake Error");
         QMetaObject::invokeMethod(this, "respond", Qt::QueuedConnection);
     }
 
     Q_INVOKABLE virtual void respond() {
-        setAttribute(QNetworkRequest::HttpStatusCodeAttribute, _httpErrorCode);
-        setError(InternalServerError, "Internal Server Fake Error");
         emit metaDataChanged();
         emit readyRead();
         // finishing can come strictly after readyRead was called
         QTimer::singleShot(5, this, &FakeErrorReply::slotSetFinished);
     }
 
+    // make public to give tests easy interface
+    using QNetworkReply::setError;
+    using QNetworkReply::setAttribute;
+
 public slots:
     void slotSetFinished() {
         setFinished(true);
@@ -755,7 +1041,6 @@ public:
         return _body.size();
     }
 
-    int _httpErrorCode;
     QByteArray _body;
 };
 
@@ -898,7 +1183,7 @@ public:
         : _localModifier(_tempDir.path())
     {
         // Needs to be done once
-        OCC::SyncEngine::minimumFileAgeForUpload = 0;
+        OCC::SyncEngine::minimumFileAgeForUpload = std::chrono::milliseconds(0);
         OCC::Logger::instance()->setLogFile("-");
 
         QDir rootDir{_tempDir.path()};
@@ -912,15 +1197,47 @@ public:
         _account->setDavDisplayName("fakename");
         _account->setServerVersion("10.0.0");
 
-        _journalDb.reset(new OCC::SyncJournalDb(localPath() + "._sync_test.db"));
+        _journalDb.reset(new OCC::SyncJournalDb(localPath() + ".sync_test.db"));
         _syncEngine.reset(new OCC::SyncEngine(_account, localPath(), "", _journalDb.get()));
+        // Ignore temporary files from the download. (This is in the default exclude list, but we don't load it)
+        _syncEngine->excludedFiles().addManualExclude("]*.~*");
+
+        // Ensure we have a valid VfsOff instance "running"
+        switchToVfs(_syncEngine->syncOptions()._vfs);
 
         // A new folder will update the local file state database on first sync.
         // To have a state matching what users will encounter, we have to a sync
         // using an identical local/remote file tree first.
-        syncOnce();
+        ENFORCE(syncOnce());
     }
 
+    void switchToVfs(QSharedPointer<OCC::Vfs> vfs, bool enableShell = false)
+    {
+        auto opts = _syncEngine->syncOptions();
+
+        opts._vfs->stop();
+        QObject::disconnect(_syncEngine.get(), 0, opts._vfs.data(), 0);
+
+        opts._vfs = vfs;
+        _syncEngine->setSyncOptions(opts);
+
+        OCC::VfsSetupParams vfsParams;
+        vfsParams.filesystemPath = localPath();
+        vfsParams.remotePath = "/";
+        vfsParams.account = _account;
+        vfsParams.journal = _journalDb.get();
+        vfsParams.providerName = "OC-TEST";
+        vfsParams.providerVersion = "0.1";
+        vfsParams.enableShellIntegration = enableShell;
+        QObject::connect(_syncEngine.get(), &QObject::destroyed, vfs.data(), [vfs]() {
+            vfs->stop();
+            vfs->unregisterFolder();
+        });
+
+        vfs->start(vfsParams);
+    }
+
+    OCC::AccountPtr account() const { return _account; }
     OCC::SyncEngine &syncEngine() const { return *_syncEngine; }
     OCC::SyncJournalDb &syncJournal() const { return *_journalDb; }
 
@@ -936,6 +1253,7 @@ public:
 
     FileInfo currentRemoteState() { return _fakeQnam->currentRemoteState(); }
     FileInfo &uploadState() { return _fakeQnam->uploadState(); }
+    FileInfo dbState() const;
 
     struct ErrorList {
         FakeQNAM *_qnam;
@@ -1031,6 +1349,42 @@ private:
     }
 };
 
+static FileInfo &findOrCreateDirs(FileInfo &base, PathComponents components)
+{
+    if (components.isEmpty())
+        return base;
+    auto childName = components.pathRoot();
+    auto it = base.children.find(childName);
+    if (it != base.children.end()) {
+        return findOrCreateDirs(*it, components.subComponents());
+    }
+    auto &newDir = base.children[childName] = FileInfo{childName};
+    newDir.parentPath = base.path();
+    return findOrCreateDirs(newDir, components.subComponents());
+}
+
+inline FileInfo FakeFolder::dbState() const
+{
+    FileInfo result;
+    _journalDb->getFilesBelowPath("", [&](const OCC::SyncJournalFileRecord &record) {
+        auto components = PathComponents(QString::fromUtf8(record._path));
+        auto &parentDir = findOrCreateDirs(result, components.parentDirComponents());
+        auto name = components.fileName();
+        auto &item = parentDir.children[name];
+        item.name = name;
+        item.parentPath = parentDir.path();
+        item.size = record._fileSize;
+        item.isDir = record._type == ItemTypeDirectory;
+        item.permissions = record._remotePerm;
+        item.etag = record._etag;
+        item.lastModified = OCC::Utility::qDateTimeFromTime_t(record._modtime);
+        item.fileId = record._fileId;
+        item.checksums = record._checksumHeader;
+        // item.contentChar can't be set from the db
+    });
+    return result;
+}
+
 /* Return the FileInfo for a conflict file for the specified relative filename */
 inline const FileInfo *findConflict(FileInfo &dir, const QString &filename)
 {
@@ -1047,6 +1401,21 @@ inline const FileInfo *findConflict(FileInfo &dir, const QString &filename)
     return nullptr;
 }
 
+struct ItemCompletedSpy : QSignalSpy {
+    explicit ItemCompletedSpy(FakeFolder &folder)
+        : QSignalSpy(&folder.syncEngine(), &OCC::SyncEngine::itemCompleted)
+    {}
+
+    OCC::SyncFileItemPtr findItem(const QString &path) const
+    {
+        for (const QList<QVariant> &args : *this) {
+            auto item = args[0].value<OCC::SyncFileItemPtr>();
+            if (item->destination() == path)
+                return item;
+        }
+        return OCC::SyncFileItemPtr::create();
+    }
+};
 
 // QTest::toString overloads
 namespace OCC {
@@ -1058,18 +1427,53 @@ namespace OCC {
 inline void addFiles(QStringList &dest, const FileInfo &fi)
 {
     if (fi.isDir) {
-        dest += QString("%1 - dir").arg(fi.name);
+        dest += QString("%1 - dir").arg(fi.path());
         foreach (const FileInfo &fi, fi.children)
             addFiles(dest, fi);
     } else {
-        dest += QString("%1 - %2 %3-bytes").arg(fi.name).arg(fi.size).arg(fi.contentChar);
+        dest += QString("%1 - %2 %3-bytes").arg(fi.path()).arg(fi.size).arg(fi.contentChar);
     }
 }
 
-inline char *toString(const FileInfo &fi)
+inline QString toStringNoElide(const FileInfo &fi)
 {
     QStringList files;
     foreach (const FileInfo &fi, fi.children)
         addFiles(files, fi);
+    files.sort();
+    return QString("FileInfo with %1 files(\n\t%2\n)").arg(files.size()).arg(files.join("\n\t"));
+}
+
+inline char *toString(const FileInfo &fi)
+{
+    return QTest::toString(toStringNoElide(fi));
+}
+
+inline void addFilesDbData(QStringList &dest, const FileInfo &fi)
+{
+    // could include etag, permissions etc, but would need extra work
+    if (fi.isDir) {
+        dest += QString("%1 - %2 %3 %4").arg(
+            fi.name,
+            fi.isDir ? "dir" : "file",
+            QString::number(fi.lastModified.toSecsSinceEpoch()),
+            fi.fileId);
+        foreach (const FileInfo &fi, fi.children)
+            addFilesDbData(dest, fi);
+    } else {
+        dest += QString("%1 - %2 %3 %4 %5").arg(
+            fi.name,
+            fi.isDir ? "dir" : "file",
+            QString::number(fi.size),
+            QString::number(fi.lastModified.toSecsSinceEpoch()),
+            fi.fileId);
+    }
+}
+
+inline char *printDbData(const FileInfo &fi)
+{
+    QStringList files;
+    foreach (const FileInfo &fi, fi.children)
+        addFilesDbData(files, fi);
     return QTest::toString(QString("FileInfo with %1 files(%2)").arg(files.size()).arg(files.join(", ")));
 }
