@@ -204,7 +204,24 @@ private slots:
             //Server support finger print, but none is set.
             fakeFolder.remoteModifier().extraDavProperties = "<oc:data-fingerprint></oc:data-fingerprint>";
         }
+
+        int fingerprintRequests = 0;
+        fakeFolder.setServerOverride([&](QNetworkAccessManager::Operation, const QNetworkRequest &request, QIODevice *stream) -> QNetworkReply * {
+            auto verb = request.attribute(QNetworkRequest::CustomVerbAttribute);
+            if (verb == "PROPFIND") {
+                auto data = stream->readAll();
+                if (data.contains("data-fingerprint")) {
+                    if (request.url().path().endsWith("webdav/"))
+                        ++fingerprintRequests;
+                    else
+                        fingerprintRequests = -10000; // fingerprint queried on incorrect path
+                }
+            }
+            return nullptr;
+        });
+
         QVERIFY(fakeFolder.syncOnce());
+        QCOMPARE(fingerprintRequests, 1);
         // First sync, we did not change the finger print, so the file should be downloaded as normal
         QCOMPARE(fakeFolder.currentLocalState(), fakeFolder.currentRemoteState());
         QCOMPARE(fakeFolder.currentRemoteState().find("C/c1")->contentChar, 'N');
@@ -230,6 +247,7 @@ private slots:
         fakeFolder.remoteModifier().extraDavProperties = "<oc:data-fingerprint>new_finger_print</oc:data-fingerprint>";
 
         QVERIFY(fakeFolder.syncOnce());
+        QCOMPARE(fingerprintRequests, 2);
         auto currentState = fakeFolder.currentLocalState();
         // Altough the local file is kept as a conflict, the server file is downloaded
         QCOMPARE(currentState.find("A/a1")->contentChar, 'O');
@@ -253,6 +271,56 @@ private slots:
 
         QCOMPARE(fakeFolder.currentLocalState(), fakeFolder.currentRemoteState());
     }
+
+    void testSingleFileRenamed() {
+        FakeFolder fakeFolder{FileInfo{}};
+
+        int aboutToRemoveAllFilesCalled = 0;
+        QObject::connect(&fakeFolder.syncEngine(), &SyncEngine::aboutToRemoveAllFiles,
+            [&](SyncFileItem::Direction , bool *) {
+                aboutToRemoveAllFilesCalled++;
+                QFAIL("should not be called");
+            });
+
+        // add a single file
+        fakeFolder.localModifier().insert("hello.txt");
+        QVERIFY(fakeFolder.syncOnce());
+        QCOMPARE(aboutToRemoveAllFilesCalled, 0);
+        QCOMPARE(fakeFolder.currentLocalState(), fakeFolder.currentRemoteState());
+
+        // rename it
+        fakeFolder.localModifier().rename("hello.txt", "goodbye.txt");
+
+        QVERIFY(fakeFolder.syncOnce());
+        QCOMPARE(aboutToRemoveAllFilesCalled, 0);
+        QCOMPARE(fakeFolder.currentLocalState(), fakeFolder.currentRemoteState());
+    }
+
+    void testSelectiveSyncNoPopup() {
+        // Unselecting all folder should not cause the popup to be shown
+        FakeFolder fakeFolder{FileInfo::A12_B12_C12_S12()};
+
+        int aboutToRemoveAllFilesCalled = 0;
+        QObject::connect(&fakeFolder.syncEngine(), &SyncEngine::aboutToRemoveAllFiles,
+            [&](SyncFileItem::Direction , bool *) {
+                aboutToRemoveAllFilesCalled++;
+                QFAIL("should not be called");
+            });
+
+        QVERIFY(fakeFolder.syncOnce());
+        QCOMPARE(aboutToRemoveAllFilesCalled, 0);
+        QCOMPARE(fakeFolder.currentLocalState(), fakeFolder.currentRemoteState());
+
+        fakeFolder.syncEngine().journal()->setSelectiveSyncList(SyncJournalDb::SelectiveSyncBlackList,
+            QStringList() << "A/" << "B/" << "C/" << "S/");
+
+        QVERIFY(fakeFolder.syncOnce());
+        QCOMPARE(fakeFolder.currentLocalState(), FileInfo{}); // all files should be one localy
+        QCOMPARE(fakeFolder.currentRemoteState(), FileInfo::A12_B12_C12_S12()); // Server not changed
+        QCOMPARE(aboutToRemoveAllFilesCalled, 0); // But we did not show the popup
+    }
+
+
 };
 
 QTEST_GUILESS_MAIN(TestAllFilesDeleted)
